using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;

/// <summary>
/// Extended bound field for use within UniGrid
/// </summary>
public class ExtendedBoundField : BoundField
{
    public const string ALL_DATA = "##ALL##";


    /// <summary>
    /// Event raised when external source data required
    /// </summary>
    public event OnExternalDataBoundEventHandler OnExternalDataBound;

    string[] actionParameters = null;
    string mNavigateUrl = null;
    string mTarget = null;
    string mExternalSourceName = null;
    bool mLocalizeStrings = false;


    #region "Public properties"

    /// <summary>
    /// Item action parameters fields
    /// </summary>
    public string ActionParameters
    {
        set
        {
            actionParameters = value.Split(';');
        }
    }


    /// <summary>
    /// Field hyperlink url
    /// </summary>
    public string NavigateUrl
    {
        get
        {
            return mNavigateUrl;
        }
        set
        {
            mNavigateUrl = value;
        }
    }


    /// <summary>
    /// Field hyperlink target
    /// </summary>
    public string Target
    {
        get
        {
            return mTarget;
        }
        set
        {
            mTarget = value;
        }
    }


    /// <summary>
    /// External source name
    /// </summary>
    public string ExternalSourceName
    {
        get
        {
            return mExternalSourceName;
        }
        set
        {
            mExternalSourceName = value;
        }
    }


    /// <summary>
    /// Localize the string in the displayed data?
    /// </summary>
    public bool LocalizeStrings
    {
        get
        {
            return mLocalizeStrings;
        }
        set
        {
            mLocalizeStrings = value;
        }
    }

    #endregion


    /// <summary>
    /// Constructor
    /// </summary>
    public ExtendedBoundField()
    {
    }


    /// <summary>
    /// Bind data event handler
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected override void OnDataBindField(object sender, EventArgs e)
    {
        bool allData = (this.DataField.ToLower() == ALL_DATA.ToLower());
        if ( !allData )
        {
            base.OnDataBindField(sender, e);
        }
        DataControlFieldCell cell = (DataControlFieldCell)sender;
        GridViewRow container = (GridViewRow)cell.NamingContainer;

        // Use external binding if set
        if (this.ExternalSourceName != null)
        {
            if (!allData)
            {
                cell.Text = OnExternalDataBound(sender, this.ExternalSourceName, ((DataRowView)container.DataItem)[this.DataField]).ToString();
            }
            else
            {
                cell.Text = OnExternalDataBound(sender, this.ExternalSourceName, ((DataRowView)container.DataItem)).ToString();
            }
        }

        // Create link
        if ( this.NavigateUrl != null )
        {
            string link = "<a href=\"" + cell.ResolveUrl(ApplyParameters(this.NavigateUrl, container)) + "\"";
            if ( this.Target != null )
            {
                link += " target=\"" + this.Target + "\"";
            }
            link += ">";
            cell.Text = link + cell.Text + "</a>";
        }

        // Localize
        if (this.LocalizeStrings)
        {
            cell.Text = ResHelper.LocalizeString(cell.Text);
        }
    }


    /// <summary>
    /// Applies the action parameters to the given string
    /// </summary>
    /// <param name="text">Text to process</param>
    /// <param name="container">Container with the data</param>
    private string ApplyParameters(string text, GridViewRow container)
    {
        if (actionParameters != null)
        {
            string result = text;
            int index = 0;
            foreach (string param in actionParameters)
            {
                result = result.Replace("{" + index.ToString() + "}", ((DataRowView)container.DataItem)[param].ToString());
                index++;
            }
            return result;
        }
        else
        {
            return text;
        }
    }


}
