using System;
using System.Xml;
using System.Data;
using System.Globalization;
using System.Web.UI.WebControls;
using System.Collections;

using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.FormEngine;
using CMS.DataProviderSQL;
using CMS.CMSHelper;
using CMS.ExtendedControls;


public partial class FieldEditor : System.Web.UI.UserControl
{
    private FormInfo fi = new FormInfo();
    private FormFieldInfo ffi = null;
    private FormCategoryInfo fci = null;
    private string[] columnNames = null;

    private string mClassName = "";
    private string mImageDirectoryPath = null; 
    private bool mDisplaySourceFieldSelection = true;
    private int mWebPartId = 0;
    private FieldEditorModeEnum mMode;
    private bool mEnableSimplifiedMode = false;


    #region "Public properties"

    /// <summary>
    /// Class name
    /// </summary>
    public string ClassName
    {
        get
        {
            return mClassName;
        }
        set
        {
            mClassName = value;
        }

    }


    /// <summary>
    /// Directory path for images
    /// </summary>
    public string ImageDirectoryPath
    {
        get
        {
            return mImageDirectoryPath;
        }
        set
        {
            if (!value.EndsWith("/"))
            {
                mImageDirectoryPath = value + "/";
            }
            else
            {
                mImageDirectoryPath = value;
            }
        }

    }


    /// <summary>
    /// Indicates if display source field selection
    /// </summary>
    public bool DisplaySourceFieldSelection
    {
        get
        {
            return mDisplaySourceFieldSelection;
        }

        set
        {
            mDisplaySourceFieldSelection = value;
        }
    }


    /// <summary>
    /// Webpart Id.
    /// </summary>
    public int WebPartId
    {
        get
        {
            return mWebPartId;
        }

        set
        {
            mWebPartId = value;
        }
    }


    /// <summary>
    /// Mode.
    /// </summary>
    public FieldEditorModeEnum Mode
    {
        get
        {
            return mMode;
        }
        set
        {
            mMode = value;
        }

    }


    /// <summary>
    /// Indicates if simplified mode is enabled.
    /// </summary>
    public bool EnableSimplifiedMode
    {
        get
        {
            return mEnableSimplifiedMode;
        }

        set
        {
            mEnableSimplifiedMode = value;
        }
    }

    #endregion

    #region "Private properties"

    
    /// <summary>
    /// Selected mode.
    /// </summary>
    private FieldEditorSelectedModeEnum SelectedMode
    {
        get
        {
            FieldEditorSelectedModeEnum mode;

            if (ViewState["SelectedMode"] == null)
            {
                mode  = GetDefaultSelectedMode();
            }
            else
            {

                switch (Convert.ToInt32(ViewState["SelectedMode"]))
                {
                    case 0:
                        mode = FieldEditorSelectedModeEnum.Simplified;
                        break;

                    case 1:
                        mode = FieldEditorSelectedModeEnum.Advanced;
                        break;

                    default:
                        mode = GetDefaultSelectedMode();
                        break;
                }
            }

            return mode;
        }

        set
        {
            switch (value)
            {
                case FieldEditorSelectedModeEnum.Simplified:
                    ViewState["SelectedMode"] = 0;
                    break;

                case FieldEditorSelectedModeEnum.Advanced:
                    ViewState["SelectedMode"] = 1;
                    break;
            }
        }
    }


    /// <summary>
    /// Indicates whether new item is edited.
    /// </summary>
    private bool IsNewItemEdited
    {
        get
        {
            object obj = ViewState["IsNewItemEdited"];
            return (obj == null) ? false : (bool)obj;
        }

        set
        {
            ViewState["IsNewItemEdited"] = value;
        }
    }


    /// <summary>
    /// Selected item name.
    /// </summary>
    private string SelectedItemName
    {
        get
        {
            object obj = ViewState["SelectedItemName"];
            return (obj == null) ? "" : (string)obj;
        }

        set
        {
            ViewState["SelectedItemName"] = value;
        }
    }


    /// <summary>
    /// Selected item type.
    /// </summary>
    private FieldEditorSelectedItemEnum SelectedItemType
    {
        get
        {
            object obj = ViewState["SelectedItemType"];
            return (obj == null) ? 0 : (FieldEditorSelectedItemEnum)obj;
        }

        set
        {
            ViewState["SelectedItemType"] = value;
        }
    }


    /// <summary>
    /// Is field primary.
    /// </summary>
    private bool IsFieldPrimary
    {
        get
        {
            object obj = ViewState["IsFieldPrimary"];
            return (obj == null) ? false : (bool)obj;
        }

        set
        {
            ViewState["IsFieldPrimary"] = value;
        }
    }

    /// <summary>
    /// Field data type.
    /// </summary>
    private string FieldDataType
    {
        get
        {
            object obj = ViewState["FieldDataType"];
            return (obj == null) ? "" : (string)obj;
        }

        set
        {
            ViewState["FieldDataType"] = (value != null) ? value.ToLower() : value;
        }
    }



    #endregion

    #region "Global definitions"

    // constants
    private const string newCategPreffix = "#categ##new#";
    private const string newFieldPreffix = "#field##new#";
    private const string categPreffix = "#categ#";
    private const string fieldPreffix = "#field#";
    private const int preffixLength = 7; // length of categPreffix = length of fieldPreffix = 7
    private const string dtPickerFolder = "~/CMSAdminControls/Calendar";
    const string controlPrefix = "#uc#";

    // basic controls for both modes (simplified and advanced)
    //--------------------------------------------------------

    // attribute name
    Label lblAttributeName = new Label();
    TextBox txtAttributeName = new TextBox();

    // allow empty 
    Label lblAttributeAllowEmpty = new Label();
    CheckBox chkAllowEmpty = new CheckBox();

    // default value types
    Label lblDefaultValue = new Label();
    TextBox txtDefaultValue = new TextBox();
    CheckBox chkDefaultValue = new CheckBox();
    DateTimePicker datetimeDefaultValue = new DateTimePicker();

    // field caption
    Label lblFieldCaption = new Label();
    TextBox txtFieldCaption = new TextBox();

    // field type
    Label lblFieldType = new Label();
    DropDownList lstFieldType = new DropDownList();
    
    // additional controls for both modes (simplified and advanced)
    //--------------------------------------------------------

    // panel with text area settings
    Panel pnlTextAreaSettings = new Panel();
        
    // text area columns
    Label lblTextAreaCols = new Label();
    TextBox txtTextAreaCols = new TextBox();

    // text area rows
    Label lblTextAreaRows = new Label();
    TextBox txtTextAreaRows = new TextBox();

    // text area size
    Label lblTextAreaSize = new Label();
    TextBox txtTextAreaSize = new TextBox();

    //----------------------------------------

    // panel with html area settings
    Panel pnlHTMLAreaSettings = new Panel();
        
    // html area width
    Label lblHTMLAreaWidth = new Label();
    TextBox txtHTMLAreaWidth = new TextBox();

    // html area height
    Label lblHTMLAreaHeight = new Label();
    TextBox txtHTMLAreaHeight = new TextBox();

    // html area size
    Label lblHTMLAreaSize = new Label();
    TextBox txtHTMLAreaSize = new TextBox();
                                
    //----------------------------------------

    // panel with drop down list settings
    Panel pnlDropDownListSettings = new Panel();
        
    // drop down list options radiobutton
    RadioButton radDropDownListOptions = new RadioButton();

    // drop down list sql radiobutton 
    RadioButton radDropDownListSQL = new RadioButton();

    // drop down list options textbox    
    TextBox txtDropDownListOptions = new TextBox();

    //----------------------------------------

    // panel with calendar settings
    Panel pnlCalendarSettings = new Panel();
        
    // edit time checkbox
    CheckBox chkCalendarSettingsEditTime = new CheckBox();

    //----------------------------------------

    // panel with multiple choice settings
    Panel pnlMultipleChoice = new Panel();

    // multiple choice options radiobutton
    RadioButton radMultipleChoiceOptions = new RadioButton();

    // multiple choice sql radiobutton 
    RadioButton radMultipleChoiceSQL = new RadioButton();

    // multiple choice options textbox    
    TextBox txtMultipleChoiceOptions = new TextBox();
   
    //----------------------------------------

    // panel with image selection settings
    Panel pnlImageSelection = new Panel();

    Label lblImageWidth = new Label();
    Label lblImageHeight = new Label();

    TextBox txtImageWidth = new TextBox();
    TextBox txtImageHeight = new TextBox();

    //----------------------------------------

    // panel public field option
    Panel pnlPublicField = new Panel();

    // public field checkbox
    Label lblPublicField = new Label();

    // public field checkbox
    CheckBox chkPublicField = new CheckBox();

    //----------------------------------------

    // panel with all editable controls
    Panel pnlAttributeEdit = new Panel();

    #endregion 

    protected void Page_PreRender(object sender, EventArgs e)
    {
        InitializeTabs();
        SetCssClasses();
    }

    protected void Page_Load(object sender, EventArgs e)
    {        
        if (mImageDirectoryPath == null)
        {
            mImageDirectoryPath = "~/App_Themes/" + Page.Theme + "/Images/CMSAdminControls/FieldEditor/";
        }
        
        //set images url
        btnNewCategory.ImageUrl = this.ImageDirectoryPath + "newcategory.gif";
        btnNewAttribute.ImageUrl = this.ImageDirectoryPath + "new.gif";
        btnDeleteItem.ImageUrl = this.ImageDirectoryPath + "delete.gif";
        btnUpAttribute.ImageUrl = this.ImageDirectoryPath + "up.gif";
        btnDownAttribute.ImageUrl = this.ImageDirectoryPath + "down.gif";
        
        this.lblSectionDatabase.Text = ResHelper.GetString("TemplateDesigner.Section.Database");
        this.lblSectionField.Text = ResHelper.GetString("TemplateDesigner.Section.Field");
        this.lblSectionValidation.Text = ResHelper.GetString("TemplateDesigner.Section.Validation");
        this.lblSectionDesign.Text = ResHelper.GetString("TemplateDesigner.Section.Design");
        this.lblSectionCategory.Text = ResHelper.GetString("TemplateDesigner.Section.Category");

        this.btnDeleteItem.ToolTip = ResHelper.GetString("TemplateDesigner.DeleteItem");
        this.btnNewCategory.ToolTip = ResHelper.GetString("TemplateDesigner.NewCategory");
        this.btnNewAttribute.ToolTip = ResHelper.GetString("TemplateDesigner.NewAttribute");
        this.btnDownAttribute.ToolTip = ResHelper.GetString("TemplateDesigner.DownAttribute");
        this.btnUpAttribute.ToolTip = ResHelper.GetString("TemplateDesigner.UpAttribute");

        this.lblCategoryName.Text = ResHelper.GetString("TemplateDesigner.CategoryName");
        this.lblAttributeSize.Text = ResHelper.GetString("TemplateDesigner.AttributeSize");
        this.lblAttributeType.Text = ResHelper.GetString("TemplateDesigner.AttributeType");
        this.lblFieldDescription.Text = ResHelper.GetString("TemplateDesigner.FieldDescription");
        this.btnOK.Text = ResHelper.GetString("TemplateDesigner.OK");
        this.chkDisplayInForm.Text = ResHelper.GetString("TemplateDesigner.DisplayInForm");
        this.ltlConfirmText.Text = "<input type=\"hidden\" id=\"confirmdelete\" value=\"" + ResHelper.GetString("TemplateDesigner.ConfirmDelete") + "\"/>";
        this.btnDeleteItem.Attributes.Add("onclick", "javascript:return confirmDelete();");
        //this.lblUserControlSettingsName.Text = ResHelper.GetString("TemplateDesigner.UserControlSettingsName");

        this.lblRegExpr.Text = ResHelper.GetString("TemplateDesigner.RegularExpression");
        this.lblMinValue.Text = ResHelper.GetString("TemplateDesigner.MinValue");
        this.lblMaxValue.Text = ResHelper.GetString("TemplateDesigner.MaxValue");
        this.lblMinLength.Text = ResHelper.GetString("TemplateDesigner.MinLength");
        this.lblMaxLength.Text = ResHelper.GetString("TemplateDesigner.MaxLength");
        this.lblFrom.Text = ResHelper.GetString("TemplateDesigner.From");
        this.lblTo.Text = ResHelper.GetString("TemplateDesigner.To");
        this.lblFileExtensions.Text = ResHelper.GetString("TemplateDesigner.Extensions");
        this.lblErrorMessage.Text = ResHelper.GetString("TemplateDesigner.ErrorMessage");

        this.lblCaptionStyle.Text = ResHelper.GetString("TemplateDesigner.CaptionStyle");
        this.lblInputStyle.Text = ResHelper.GetString("TemplateDesigner.InputStyle");
        this.lblSourceField.Text = ResHelper.GetString("TemplateDesigner.SourceField");

        this.dateTo.SupportFolder = dtPickerFolder;
        this.dateFrom.SupportFolder = dtPickerFolder;        

        this.lblInfo.Visible = false;        
        this.lblError.Visible = true;
        this.lblError.Text = "&nbsp;";
        this.ltlScript.Text = "";

        btnOK.Enabled = true;
        btnUpAttribute.Enabled = true;
        btnDownAttribute.Enabled = true;
        btnDeleteItem.Enabled = true;
        btnNewAttribute.Enabled = true;
        btnNewCategory.Enabled = true;

        // set universal controls
        SetUniversalControls();

        if (!IsPostBack)
        {
            this.Reload(null);
        }

        // mark categories in the listbox
        MarkCategories();

        // display or hide tabs
        DisplayOrHideTabs();

    }

    /// <summary>
    /// Reload field editor
    /// </summary>
    /// <param name="selectedValue">Selected field in field list</param>
    public void Reload(string selectedValue)
    {
        bool isModeSelected = false;
        bool isItemSelected = false;

        switch (mMode)
        {
            case FieldEditorModeEnum.ClassFormDefinition:
            case FieldEditorModeEnum.BizFormDefinition:
            case FieldEditorModeEnum.SystemTable:

                if ((mClassName != null) && (mClassName != ""))
                {
                    isModeSelected = true;
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "Class name is not set."; // TODO: save to resources
                }
                break;

            case FieldEditorModeEnum.WebPartProperties:

                if ((mWebPartId > 0))
                {
                    isModeSelected = true;
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "WebPartID is not set."; // TODO: save to resources
                }
                break;

            default:                
                lblError.Visible = true;
                lblError.Text = "No field editor mode is set."; // TODO: save to resources
                break;
        }

        if (isModeSelected)
        {        
                                    
            this.LoadFormDefinition();
            this.LoadAttributesList(selectedValue);            

            if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
            {
                isItemSelected = true;
                
                if (EnableSimplifiedMode)
                {
                    HighlightSelectedTab();    
                }
                
                DisplaySelectedTabContent();
                SetUniversalControls();                               
                LoadSelectedField();
                
            }
            else if (SelectedItemType == FieldEditorSelectedItemEnum.Category)
            {
                isItemSelected = true;
                LoadSelectedCategory();
            }            
        }

        if (!isItemSelected)
        {
            HideAllPanels();
            btnOK.Enabled = false;
            btnUpAttribute.Enabled = false;
            btnDownAttribute.Enabled = false;
            btnDeleteItem.Enabled = false;
            btnNewAttribute.Enabled = true; // only new items can be added
            btnNewCategory.Enabled = true; // only new items can be added
        }

        DisplayOrHideSourceFieldSelection();
    }


    /// <summary>
    /// Load xml definition of the form
    /// </summary>
    private void LoadFormDefinition()
    {
        string xmlString = "";
        bool isError = false;

        switch (mMode)
        {
            case FieldEditorModeEnum.WebPartProperties:

                // load xml definition from webpartinfo
                WebPartInfo wpi = WebPartInfoProvider.GetWebPartInfo(mWebPartId);
                if (wpi != null)
                {
                    xmlString = wpi.WebPartProperties;
                }
                else
                {
                    isError = false;
                }
                break;

            case FieldEditorModeEnum.ClassFormDefinition:
            case FieldEditorModeEnum.BizFormDefinition:
            case FieldEditorModeEnum.SystemTable:

                // load xml definition from dataclassinfo
                DataClassInfo dci = DataClassInfoProvider.GetDataClass(mClassName);
                if (dci != null)
                {
                    xmlString = dci.ClassFormDefinition;
                }
                else
                {
                    isError = true;
                }
                break;
        }

        if (isError)
        {
            lblError.Visible = true;
            lblError.Text = "[ FieldEditor.LoadFormDefinition() ]: No xml definition was loaded."; // TODO: save to resources
        }

        fi.LoadXmlDefinition(xmlString);

    }


    /// <summary>
    /// Fill attribute list
    /// </summary>
    /// <param name="selectedValue">Selected value in attribute list, null if first item is selected</param>
    private void LoadAttributesList(string selectedValue)
    {
        
        FormFieldInfo formField = null;
        FormCategoryInfo formCategory = null;
        ListItem li = null;
        string itemDisplayName = "";
        string itemCodeName = "";

        // reload list only if new item is not edited
        if (!IsNewItemEdited)
        {

            // clear item list
            lstAttributes.Items.Clear();

            // get all list items (fields and categories)        
            ArrayList itemList = fi.GetFormElements(true, true);

            if (itemList != null)
            {
                foreach (object item in itemList)
                {
                    if (item is FormFieldInfo)
                    {
                        formField = ((FormFieldInfo)(item));
                        itemDisplayName = formField.Name;
                        itemCodeName = fieldPreffix + formField.Name;
                        li = new ListItem(itemDisplayName, itemCodeName);

                    }
                    else if (item is FormCategoryInfo)
                    {
                        formCategory = ((FormCategoryInfo)(item));
                        itemDisplayName = formCategory.CategoryName;
                        itemCodeName = categPreffix + formCategory.CategoryName;
                        li = new ListItem(itemDisplayName, itemCodeName);
                        // highlight categories in the list 
                        li.Attributes.Add("class", "FieldEditorCategoryItem");
                    }
                    if (li != null)
                    {
                        lstAttributes.Items.Add(li);
                    }
                }
            }

            // set selected item
            if (lstAttributes.Items.Count > 0)
            {

                if ((selectedValue != null) && (selectedValue != ""))
                {
                    lstAttributes.SelectedValue = selectedValue;
                }
                else
                {
                    //select first item of the list       
                    lstAttributes.SelectedIndex = 0;
                }
            }

            // default values - list is empty
            SelectedItemName = "";
            SelectedItemType = 0;

            // save selected item info
            if (lstAttributes.SelectedValue != null)
            {
                if (lstAttributes.SelectedValue.StartsWith(fieldPreffix))
                {
                    SelectedItemName = lstAttributes.SelectedValue.Substring(preffixLength);
                    SelectedItemType = FieldEditorSelectedItemEnum.Field;

                }
                else if (lstAttributes.SelectedValue.StartsWith(categPreffix))
                {
                    SelectedItemName = lstAttributes.SelectedValue.Substring(preffixLength);
                    SelectedItemType = FieldEditorSelectedItemEnum.Category;

                }
            }
        }  
    }


    /// <summary>
    /// Mark catories in list with different color
    /// </summary>
    private void MarkCategories()
    {              
        foreach (ListItem item in lstAttributes.Items)
        {            
            // mark category item with different color
            if (item.Value.StartsWith(categPreffix))
            {
                item.Attributes.Add("class", "FieldEditorCategoryItem");
            }
        }
    }



    /// <summary>
    /// Set all values of the category edit form to defaults.
    /// </summary>
    private void LoadDefaultCategoryEditForm()
    {
        pnlAttributeEdit.Visible = false;
        pnlAdvancedAttributeEdit.Visible = false;
        pnlSimpleAttributeEdit.Visible = false;
        pnlCategoryEdit.Visible = true;
        txtCategoryName.Text = "";
    }

    /// <summary>
    /// Set all values of form to defaults.
    /// </summary>
    private void LoadDefaultAttributeEditForm()
    {
        pnlAttributeEdit.Visible = true;
        pnlCategoryEdit.Visible = false;

        LoadAttributeTypes();        
        if (lstAttributeType.Items.Count > 0)
        {
            this.lstAttributeType.SelectedIndex = 0;
        }
        LoadFieldTypes(false);
        ShowValidationOptions(false);

        txtAttributeName.Text = "";
        txtAttributeSize.Text = "";
        chkAllowEmpty.Checked = (Mode == FieldEditorModeEnum.SystemTable);        
        chkDisplayInForm.Checked = true;
        chkPublicField.Checked = true;
        txtDescription.Text = "";
        txtFieldCaption.Text = "";

        txtSimpleTextBoxMaxLength.Text = "";

        // set advanced options values
        txtTextAreaCols.Text = "";
        txtTextAreaRows.Text = "";
        txtHTMLAreaHeight.Text = "";
        txtHTMLAreaWidth.Text = "";
        radDropDownListOptions.Checked = true;
        radDropDownListSQL.Checked = false;
        radMultipleChoiceOptions.Checked = true;
        radMultipleChoiceSQL.Checked = false;
        txtDropDownListOptions.Text = "";
        txtMultipleChoiceOptions.Text = "";
        chkCalendarSettingsEditTime.Checked = true;
        txtImageHeight.Text = "";
        txtImageWidth.Text = "";

        // validation section
        txtRegExpr.Text = "";
        txtMinValue.Text = "";
        txtMaxValue.Text = "";
        txtMinLength.Text = "";
        txtMaxLength.Text = "";
        dateFrom.SelectedDateTime = DateTimePicker.NOT_SELECTED;
        dateTo.SelectedDateTime = DateTimePicker.NOT_SELECTED;
        txtFileExtensions.Text = "";
        txtErrorMessage.Text = "";

        // design section
        txtCaptionStyle.Text = "";
        txtInputStyle.Text = "";

        // default value controls
        txtDefaultValue.Text = "";
        chkDefaultValue.Checked = false;
        datetimeDefaultValue.SelectedDateTime = DateTimePicker.NOT_SELECTED;

        EnableFieldEditing();
        EnableOrDisableAttributeSize();
        ShowOrHideFieldDetails();
        ShowAdvancedOptions();
        ShowDefaultValue();
        SetDefaultValue();        
    }

    /// <summary>
    /// Fill form with selected category data
    /// </summary>    
    private void LoadSelectedCategory()
    {

        pnlAttributeEdit.Visible = false;
        pnlAdvancedAttributeEdit.Visible = false;
        pnlSimpleAttributeEdit.Visible = false;
        pnlCategoryEdit.Visible = true;

        fci = fi.GetFormCategory(SelectedItemName);

        if (fci != null)
        {
            txtCategoryName.Text = fci.CategoryName;
        }
        else
        {
            LoadDefaultCategoryEditForm();
        }

    }


    /// <summary>
    /// Fill form with selected attribute data
    /// </summary>    
    private void LoadSelectedField()
    {

        pnlAttributeEdit.Visible = true;
        pnlCategoryEdit.Visible = false;

        // load selected field data from xml into FormFieldInfo structure
        ffi = fi.GetFormField(SelectedItemName);

        // fill form
        if (ffi != null)
        {
            IsFieldPrimary = ffi.PrimaryKey;
            FieldDataType = FormHelper.GetFormFieldDataTypeString(ffi.DataType);
            
            this.txtAttributeName.Text = ffi.Name;
            this.txtAttributeSize.Text = (ffi.Size == 0) ? "" : Convert.ToString(ffi.Size);

            // load attribute types
            this.LoadAttributeTypes();
            if (ffi.DataType != FormFieldDataTypeEnum.Unknown)
            {
                try
                {
                    if (((Mode == FieldEditorModeEnum.BizFormDefinition) || (Mode == FieldEditorModeEnum.SystemTable) ) 
                        && (ffi.FieldType == FormFieldControlTypeEnum.UploadControl))
                    {
                        // in "bizform" and "systemtalbe" modes 'file' data type is representing as nvarchar(500) to allow to save "<guid>.<extenstion>"
                        this.lstAttributeType.SelectedValue = FormFieldDataTypeCode.FILE;
                    }
                    else
                    {
                        this.lstAttributeType.SelectedValue = FormHelper.GetFormFieldDataTypeString(ffi.DataType);
                    }
                }
                catch
                {
                    // if not such value exist in the list
                }
            }

            // load field types
            this.LoadFieldTypes(ffi.PrimaryKey);
            if (ffi.FieldType != FormFieldControlTypeEnum.Unknown)
            {
                try
                {
                    // select user control
                    if (ffi.FieldType == FormFieldControlTypeEnum.CustomUserControl)
                    {
                        this.lstFieldType.SelectedValue = (controlPrefix + Convert.ToString(ffi.Settings["controlname"]).ToLower());
                    }
                    // select default control
                    else
                    {                        
                        if (SelectedMode == FieldEditorSelectedModeEnum.Simplified)
                        {
                            if (ffi.FieldType == FormFieldControlTypeEnum.TextBoxControl)
                            {
                                if ((ffi.DataType == FormFieldDataTypeEnum.Integer))
                                {
                                    ffi.FieldType = FormFieldControlTypeEnum.IntegerNumberTextBox;
                                }
                                else if (ffi.DataType == FormFieldDataTypeEnum.Decimal)
                                {
                                    ffi.FieldType = FormFieldControlTypeEnum.DecimalNumberTextBox;
                                }
                            }
                            else if (ffi.FieldType == FormFieldControlTypeEnum.LabelControl)
                            {   // add "Label" option to dropdown
                                lstFieldType.Items.Add(new ListItem(ResHelper.GetString("TemplateDesigner.FieldControlTypes.Label"),FormFieldControlTypeCode.LABEL));                                
                            }
                        }

                        this.lstFieldType.SelectedValue = FormHelper.GetFormFieldControlTypeString(ffi.FieldType);
                    }
                }
                catch
                {
                    // if not such value exist in the list
                }
            }

            this.chkAllowEmpty.Checked = ffi.AllowEmpty;
            this.chkDisplayInForm.Checked = ffi.Visible;
            this.txtDescription.Text = ffi.Description;
            this.txtFieldCaption.Text = ffi.Caption;

            if (Mode == FieldEditorModeEnum.BizFormDefinition)
            {
                this.chkPublicField.Checked = ffi.PublicField;
            }

            // show validation options
            this.ShowValidationOptions(ffi.PrimaryKey);

            // set advanced options values
            this.txtTextAreaCols.Text = "";
            this.txtTextAreaRows.Text = "";
            this.txtHTMLAreaHeight.Text = "";
            this.txtHTMLAreaWidth.Text = "";
            this.radDropDownListOptions.Checked = true;
            this.radDropDownListSQL.Checked = false;
            this.radMultipleChoiceOptions.Checked = true;
            this.radMultipleChoiceSQL.Checked = false;
            this.txtDropDownListOptions.Text = "";
            this.txtMultipleChoiceOptions.Text = "";
            this.chkCalendarSettingsEditTime.Checked = true;
            this.txtImageWidth.Text = "";
            this.txtImageHeight.Text = "";
            

            // validation section
            this.txtRegExpr.Text = ffi.RegularExpression;
            this.txtMinValue.Text = (ffi.MinNumericValue > -1) ? Convert.ToString(ffi.MinNumericValue) : "";
            this.txtMaxValue.Text = (ffi.MaxNumericValue > -1) ? Convert.ToString(ffi.MaxNumericValue) : "";
            this.txtMinLength.Text = (ffi.MinStringLength > -1) ? Convert.ToString(ffi.MinStringLength) : "";
            this.txtMaxLength.Text = (ffi.MaxStringLength > -1) ? Convert.ToString(ffi.MaxStringLength) : "";            
            this.dateFrom.SelectedDateTime = ffi.MinDateTimeValue;
            this.dateTo.SelectedDateTime = ffi.MaxDateTimeValue;            
            this.txtFileExtensions.Text = ffi.FileExtensions;
            this.txtErrorMessage.Text = ffi.ValidationErrorMessage;

            // design section
            this.txtCaptionStyle.Text = ffi.CaptionStyle;
            this.txtInputStyle.Text = ffi.InputControlStyle;

            // primary key not allowed to change
            if (ffi.PrimaryKey)
            {
                DisableFieldEditing();
            }
            else
            {
                EnableFieldEditing();
            }

            // system field not allowed to change
            if (ffi.System)
            {
                DisableFieldEditing();
                this.txtAttributeName.Enabled = false;
                this.txtAttributeSize.Enabled = false;
            }

            ShowAdvancedOptions();

            // fill advanced options        
            switch (ffi.FieldType)
            {
                case FormFieldControlTypeEnum.TextBoxControl:
                    // --------------------------------------------------
                    if (SelectedMode == FieldEditorSelectedModeEnum.Simplified)
                    {
                        txtSimpleTextBoxMaxLength.Text = (ffi.Size == 0) ? "" : Convert.ToString(ffi.Size);
                    }
                    break;
                       
                case FormFieldControlTypeEnum.TextAreaControl:
                    // --------------------------------------------------

                    this.txtTextAreaCols.Text = Convert.ToString(ffi.Settings["cols"]);
                    this.txtTextAreaRows.Text = Convert.ToString(ffi.Settings["rows"]);
                    this.txtTextAreaSize.Text = Convert.ToString(ffi.Settings["size"]);

                    break;

                case FormFieldControlTypeEnum.HtmlAreaControl:
                    // --------------------------------------------------

                    this.txtHTMLAreaWidth.Text = Convert.ToString(ffi.Settings["width"]);
                    this.txtHTMLAreaHeight.Text = Convert.ToString(ffi.Settings["height"]);
                    this.txtHTMLAreaSize.Text = Convert.ToString(ffi.Settings["size"]);

                    break;

                case FormFieldControlTypeEnum.DropDownListControl:
                case FormFieldControlTypeEnum.RadioButtonsControl:
                    // --------------------------------------------------

                    if (ffi.Settings.ContainsKey("query"))
                    {
                        this.radDropDownListSQL.Checked = true;
                        this.radDropDownListOptions.Checked = false;
                        this.txtDropDownListOptions.Text = Convert.ToString(ffi.Settings["query"]);
                    }

                    else if (ffi.Settings.ContainsKey("options"))
                    {
                        XmlDocument xmlDoc = new XmlDocument();

                        xmlDoc.LoadXml("<options>" + Convert.ToString(ffi.Settings["options"]) + "</options>");

                        if (xmlDoc != null)
                        {
                            foreach (XmlNode optionItem in xmlDoc.SelectNodes("options/item"))
                            {
                                this.txtDropDownListOptions.Text += optionItem.Attributes["value"].Value + ";" + optionItem.Attributes["text"].Value + "\n";
                            }
                        }

                    }
                    break;


                case FormFieldControlTypeEnum.MultipleChoiceControl:
                    // --------------------------------------------------

                    if (ffi.Settings.ContainsKey("query"))
                    {
                        this.radMultipleChoiceSQL.Checked = true;
                        this.radMultipleChoiceOptions.Checked = false;
                        this.txtMultipleChoiceOptions.Text = Convert.ToString(ffi.Settings["query"]);
                    }

                    else if (ffi.Settings.ContainsKey("options"))
                    {
                        XmlDocument xmlDoc = new XmlDocument();

                        xmlDoc.LoadXml("<options>" + Convert.ToString(ffi.Settings["options"] + "</options>"));

                        if (xmlDoc != null)
                        {
                            foreach (XmlNode optionItem in xmlDoc.SelectNodes("options/item"))
                            {
                                this.txtMultipleChoiceOptions.Text += optionItem.Attributes["value"].Value + ";" + optionItem.Attributes["text"].Value + "\n";
                            }
                        }

                    }
                    break;

                case FormFieldControlTypeEnum.CalendarControl:
                    // --------------------------------------------------

                    if (Convert.ToString(ffi.Settings["editTime"]).ToLower() == "true")
                    {
                        this.chkCalendarSettingsEditTime.Checked = true;
                    }
                    else
                    {
                        this.chkCalendarSettingsEditTime.Checked = false;
                    }
                    break;

                case FormFieldControlTypeEnum.ImageSelectionControl:
                    // --------------------------------------------------
                    this.txtImageWidth.Text = Convert.ToString(ffi.Settings["width"]);
                    this.txtImageHeight.Text = Convert.ToString(ffi.Settings["height"]);                    
                    break;
            }

            EnableOrDisableAttributeSize();

            ShowOrHideFieldDetails();                        
            ShowDefaultValue();
            SetDefaultValue();

        }
        else
        {
            LoadDefaultAttributeEditForm();
        }
    }

    /// <summary>
    /// Displays or hides source field selection, according to mode.
    /// </summary>
    protected void DisplayOrHideSourceFieldSelection()
    {        
        pnlSourceField.Visible = false;

        // display or hide source field selection
        if (mMode == FieldEditorModeEnum.ClassFormDefinition)
        {
            // fill source field drop down list
            if (DisplaySourceFieldSelection)
            {
                pnlSourceField.Visible = true;

                // add implicit list item
                drpSourceField.Items.Clear();
                drpSourceField.Items.Add(new ListItem(ResHelper.GetString("TemplateDesigner.ImplicitSourceField"), ""));

                string[] columnNames = fi.GetColumnNames();
                if (columnNames != null)
                {
                    // add attribute list item to the list of attributes
                    foreach (string name in columnNames)
                    {
                        FormFieldInfo ffiColumn = fi.GetFormField(name);
                        // don't add primary key field
                        if (!ffiColumn.PrimaryKey)
                        {
                            drpSourceField.Items.Add(new ListItem(name, name));
                        }
                    }
                }

                // set selected value
                DataClassInfo dci = DataClassInfoProvider.GetDataClass(ClassName);
                if (dci != null)
                {
                    try
                    {
                        drpSourceField.SelectedValue = dci.ClassNodeNameSource;
                    }
                    catch
                    {
                    }
                }

            }
        }        
    }

    /// <summary>
    /// When attribute up button is clicked
    /// </summary>
    protected void btnUpAttribute_Click(System.Object sender, System.Web.UI.ImageClickEventArgs e)
    {

        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            // check 'EditForm' permission
            if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.form", "EditForm"))
            {
                CMSDeskPage.RedirectToAccessDenied("cms.form", "EditForm");
            }
        }

        LoadFormDefinition();

        // first item of the attribute list cannot be moved higher
        if (this.lstAttributes.SelectedValue == null || this.lstAttributes.SelectedValue == "" || this.lstAttributes.SelectedIndex == 0)
        {
            return;
        }

        // 'new (not saved)' attribute cannot be moved
        else if ((SelectedItemName == newCategPreffix) || (SelectedItemName == newFieldPreffix))
        {
            this.ShowMessage(ResHelper.GetString("TemplateDesigner.AlertNewAttributeCannotBeMoved"));
            return;
        }

        if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
        {
            // move attribute up in attribute list                        
            fi.MoveFormFieldUp(SelectedItemName);
        }
        else if (SelectedItemType == FieldEditorSelectedItemEnum.Category)
        {
            // move category up in attribute list                        
            fi.MoveFormCategoryUp(SelectedItemName);

        }

        switch (mMode)
        {
            case FieldEditorModeEnum.WebPartProperties:

                // save xml string to CMS_WebPart table
                WebPartInfo wpi = WebPartInfoProvider.GetWebPartInfo(mWebPartId);
                if (wpi != null)
                {
                    wpi.WebPartProperties = fi.GetXmlDefinition();
                    WebPartInfoProvider.SetWebPartInfo(wpi);
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "[ FieldEditor.btnUpAttribute_Click() ]: No changes were saved in xml definition."; // TODO

                }
                break;

            case FieldEditorModeEnum.ClassFormDefinition:
            case FieldEditorModeEnum.BizFormDefinition:
            case FieldEditorModeEnum.SystemTable:

                // save xml string to CMS_Class table
                DataClassInfo dci = DataClassInfoProvider.GetDataClass(mClassName);
                if (dci != null)
                {
                    dci.ClassFormDefinition = fi.GetXmlDefinition();
                    DataClassInfoProvider.SetDataClass(dci);
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "[ FieldEditor.btnUpAttribute_Click() ]: No changes were saved in xml definition."; // TODO

                }
                break;
        }

        // reload attibute list
        LoadAttributesList(this.lstAttributes.SelectedValue);

    }


    /// <summary>
    /// When attribute down button is clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnDownAttribute_Click(System.Object sender, System.Web.UI.ImageClickEventArgs e)
    {
        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            // check 'EditForm' permission
            if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.form", "EditForm"))
            {
                CMSDeskPage.RedirectToAccessDenied("cms.form", "EditForm");
            }
        }

        LoadFormDefinition();

        // last item of the attribute list cannot be moved lower
        if (this.lstAttributes.SelectedValue == null || this.lstAttributes.SelectedValue == "" || this.lstAttributes.SelectedIndex >= this.lstAttributes.Items.Count - 1)
        {
            return;
        }

        // 'new and not saved' attribute cannot be moved
        else if ((SelectedItemName == newCategPreffix) || (SelectedItemName == newFieldPreffix))
        {
            this.ShowMessage(ResHelper.GetString("TemplateDesigner.AlertNewAttributeCannotBeMoved"));
            return;
        }

        if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
        {
            // move attribute down in attribute list                        
            fi.MoveFormFieldDown(SelectedItemName);
        }
        else if (SelectedItemType == FieldEditorSelectedItemEnum.Category)
        {
            // move category down in attribute list                        
            fi.MoveFormCategoryDown(SelectedItemName);

        }

        switch (mMode)
        {
            case FieldEditorModeEnum.WebPartProperties:

                // save xml string to CMS_WebPart table
                WebPartInfo wpi = WebPartInfoProvider.GetWebPartInfo(mWebPartId);
                if (wpi != null)
                {
                    wpi.WebPartProperties = fi.GetXmlDefinition();
                    WebPartInfoProvider.SetWebPartInfo(wpi);
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "[ FieldEditor.btnDownAttribute_Click() ]: No changes were saved in xml definition.";

                }
            break;

            case FieldEditorModeEnum.ClassFormDefinition:
            case FieldEditorModeEnum.BizFormDefinition:
            case FieldEditorModeEnum.SystemTable:

                // save xml string to CMS_Class table
                DataClassInfo dci = DataClassInfoProvider.GetDataClass(mClassName);
                if (dci != null)
                {
                    dci.ClassFormDefinition = fi.GetXmlDefinition();
                    DataClassInfoProvider.SetDataClass(dci);
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "[ FieldEditor.btnDownAttribute_Click() ]: No changes were saved in xml definition.";

                }
            break;
        }

        // reload attibute list
        LoadAttributesList(this.lstAttributes.SelectedValue);
    }

    /// <summary>
    /// When chkDisplayInForm checkbox checked changed
    /// </summary>
    protected void chkDisplayInForm_CheckedChanged(System.Object sender, System.EventArgs e)
    {
        ShowOrHideFieldDetails();
    }


    /// <summary>
    /// When selected attribute changed
    /// </summary>
    protected void lstAttributes_SelectedIndexChanged(System.Object sender, System.EventArgs e)
    {
        bool isNewCreated = false;

        // check if new attribute is editing -> select it and avoid selecting another attribute
        for (int i = 0; i <= lstAttributes.Items.Count - 1; i++)
        {
            if (lstAttributes.Items[i].Value == newCategPreffix)
            {
                isNewCreated = true;
                this.lstAttributes.SelectedValue = newCategPreffix;
            }
            else if (lstAttributes.Items[i].Value == newFieldPreffix)
            {
                isNewCreated = true;
                this.lstAttributes.SelectedValue = newFieldPreffix;
            }

            if (isNewCreated)
            {
                this.ShowMessage(ResHelper.GetString("TemplateDesigner.AlertSaveNewItemOrDeleteItFirst"));
                EnableFieldEditing();
                return;
            }
        }

        // reload data
        Reload(lstAttributes.SelectedValue);
    }

    /// <summary>
    /// Show or hide details according to chkDisplayInForm checkbox is checked or not
    /// </summary>   
    private void ShowOrHideFieldDetails()
    {
        if ((this.chkDisplayInForm.Checked) || (SelectedMode == FieldEditorSelectedModeEnum.Simplified) )
        {
            ShowFieldDetails();
        }
        else
        {
            HideFieldDetails();
        }
    }


    /// <summary>
    /// Show field details
    /// </summary>
    private void ShowFieldDetails()
    {
        this.pnlDisplayInForm.Enabled = true;
        this.txtFieldCaption.ReadOnly = false;
        this.txtDescription.ReadOnly = false;
        /*   if (!(this.lstFieldType.Attributes["disabled"] == null))
           {
               this.lstFieldType.Attributes.Remove("disabled");
           }*/
    }


    /// <summary>
    /// Hide field details
    /// </summary>
    private void HideFieldDetails()
    {

        this.pnlDisplayInForm.Enabled = false;
        this.txtFieldCaption.ReadOnly = true;
        this.txtDescription.ReadOnly = true;
        
        //  this.lstFieldType.Attributes.Add("disabled", "true");
    }



    /// <summary>
    /// Disables field editing controls, except field name
    /// </summary>
    private void DisableFieldEditing()
    {
        this.txtDefaultValue.Enabled = false;
        this.chkDefaultValue.Enabled = false;
        this.datetimeDefaultValue.Enabled = false;
        this.lstAttributeType.Enabled = false;
        this.chkAllowEmpty.Enabled = false;
        this.txtAttributeName.Enabled = false;
    }


    /// <summary>
    /// Enables field editing controls, except field name
    /// </summary>
    private void EnableFieldEditing()
    {
        this.txtDefaultValue.Enabled = true;
        this.chkDefaultValue.Enabled = true;
        this.datetimeDefaultValue.Enabled = true;
        this.lstAttributeType.Enabled = true;
        this.chkAllowEmpty.Enabled = (this.Mode != FieldEditorModeEnum.SystemTable);
        this.txtAttributeName.Enabled = true;
    }


    /// <summary>
    /// Save/update button clicked
    /// </summary>
    protected void btnOK_Click(System.Object sender, System.EventArgs e)
    {

        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            // check 'EditForm' permission
            if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.form", "EditForm"))
            {
                CMSDeskPage.RedirectToAccessDenied("cms.form", "EditForm");
            }
        }

        string errorMessage = "";

        if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
        {        

            // validate basic options
            errorMessage += ValidateForm();

            // validate advanced options
            errorMessage += ValidateAdvancedOptions();

        }

        // check occured errors
        if (errorMessage != "")
        {
            this.lblError.Visible = true;
            lblError.Text = errorMessage;
        }
        else
        {
            lblError.Text = "";
            // save selected field
            SaveSelectedField();
        }
    }

    /// <summary>
    /// Save selected field
    /// </summary>
    private void SaveSelectedField()
    {
        System.Globalization.CultureInfo ciEnUs = new System.Globalization.CultureInfo("en-us"); // culture info for datetime conversion to string

        FormFieldInfo ffiUpdated = null;    // FormFieldInfo structure with data from updated form                
        FormCategoryInfo fciUpdated = null; // FormCategoryInfo structure with data from updated form                
        bool isNewItem = false;        // determines whether it is a new attribute (or attribute to update)
        string errorMessage = "";
        DataClassInfo dci = null;
        WebPartInfo wpi = null;

        // variables for changes in DB tables
        string tableName = "";
        string oldColumnName = "";
        string newColumnName = "";
        string newColumnSize = "";
        string newColumnType = "";
        string newColumnDefaultValue = null;  // no default value
        bool newColumnAllowNull = true;        

        switch (mMode)
        {
            case FieldEditorModeEnum.WebPartProperties:

                // fill WebPartInfo structure with data from database
                wpi = WebPartInfoProvider.GetWebPartInfo(mWebPartId);
            break;

            case FieldEditorModeEnum.ClassFormDefinition:
            case FieldEditorModeEnum.BizFormDefinition:
            case FieldEditorModeEnum.SystemTable:

                // fill DataClassInfo structure with data from database
                dci = DataClassInfoProvider.GetDataClass(mClassName);

                if (dci != null)
                {
                    // set table name 
                    tableName = dci.ClassTableName;
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = "Table name is not set.";  // TODO
                    return;
                }
            break;
        }

        // load current xml form definition
        this.LoadFormDefinition();


        if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
        {
            //fill FormFieldInfo structure with original data
            ffi = fi.GetFormField(SelectedItemName);

            // fill FormFieldInfo structure with updated form data        
            ffiUpdated = FillFormFieldInfoStructure(ffi);

            // determine whether it is a new attribute or not
            isNewItem = (ffi == null) ? true : false;


            // check if the attribe name already exists
            //-----------------------------------------        
            if (isNewItem || (ffi.Name != ffiUpdated.Name))
            {

                columnNames = fi.GetColumnNames();

                if (columnNames != null)
                {
                    foreach (string colName in columnNames)
                    {
                        // if name already exists
                        if (ffiUpdated.Name == colName)
                        {
                            this.lblError.Visible = true;
                            this.lblError.Text = ResHelper.GetString("TemplateDesigner.ErrorExistingColumnName");
                            return;
                        }
                    }
                }

            }

            // ------------------------------
            // new node                
            // ------------------------------   
            if (isNewItem)
            {

                ffiUpdated.PrimaryKey = false;
                ffiUpdated.System = false;

                newColumnName = ffiUpdated.Name;
                newColumnAllowNull = ffiUpdated.AllowEmpty;
                newColumnSize = "";


                // set implicit default value
                if (!(newColumnAllowNull) && (ffiUpdated.DefaultValue == ""))
                {
                    switch (ffiUpdated.DataType)
                    {
                        case FormFieldDataTypeEnum.Integer:
                        case FormFieldDataTypeEnum.Decimal:
                        case FormFieldDataTypeEnum.Boolean:

                            newColumnDefaultValue = "0";

                            break;


                        case FormFieldDataTypeEnum.Text:
                        case FormFieldDataTypeEnum.LongText:

                            newColumnDefaultValue = "";
                            break;


                        case FormFieldDataTypeEnum.DateTime:
                            newColumnDefaultValue = DateTime.Now.ToString(ciEnUs.DateTimeFormat);
                            break;

                        case FormFieldDataTypeEnum.File:
                            // 32 digits, empty Guid
                            newColumnDefaultValue = Guid.Empty.ToString();
                            break;
                    }
                }

                // Check if default value is in required format
                else if (ffiUpdated.DefaultValue != "")
                {
                    switch (ffiUpdated.DataType)
                    {

                        case FormFieldDataTypeEnum.Integer:
                            try
                            {
                                int i = Int32.Parse(ffiUpdated.DefaultValue);
                                newColumnDefaultValue = i.ToString();
                            }
                            catch
                            {
                                newColumnDefaultValue = "0";
                                errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueInteger");
                            }
                            break;

                        case FormFieldDataTypeEnum.Decimal:
                            try
                            {
                                double d = Double.Parse(ffiUpdated.DefaultValue);
                                newColumnDefaultValue = d.ToString();
                            }
                            catch
                            {
                                newColumnDefaultValue = "0";
                                errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueDouble");
                            }
                            break;

                        case FormFieldDataTypeEnum.DateTime:
                            try
                            {
                                DateTime dat = DateTime.Parse(ffiUpdated.DefaultValue, ciEnUs.DateTimeFormat);
                                newColumnDefaultValue = dat.ToString();
                            }
                            catch
                            {
                                newColumnDefaultValue = DateTime.Now.ToString(ciEnUs.DateTimeFormat);
                                errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueDateTime");
                            }
                            break;
                        
                        case FormFieldDataTypeEnum.File:
                            try
                            {
                                Guid g = new Guid(ffiUpdated.DefaultValue);
                                newColumnDefaultValue = g.ToString();

                            }
                            catch
                            {
                                newColumnDefaultValue = Guid.Empty.ToString();
                                errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueGuid");

                            }
                            break;

                        case FormFieldDataTypeEnum.LongText:
                        case FormFieldDataTypeEnum.Text:
                        case FormFieldDataTypeEnum.Boolean:

                            newColumnDefaultValue = ffiUpdated.DefaultValue;
                            break;
                    }
                }


                // set column type
                switch (ffiUpdated.DataType)
                {

                    case FormFieldDataTypeEnum.Integer:
                        newColumnType = "int";
                        break;

                    case FormFieldDataTypeEnum.Decimal:
                        newColumnType = "float";
                        break;

                    case FormFieldDataTypeEnum.Boolean:
                        newColumnType = "bit";
                        break;

                    case FormFieldDataTypeEnum.DateTime:
                        newColumnType = "datetime";
                        break;

                    case FormFieldDataTypeEnum.LongText:
                        newColumnType = "ntext";
                        break;

                    case FormFieldDataTypeEnum.File:
                        newColumnType = "uniqueidentifier";
                        break;

                    default:
                        newColumnSize = Convert.ToString(ffiUpdated.Size);
                        newColumnType = "nvarchar(" + newColumnSize + ")";
                        break;
                }


                if (errorMessage == "")
                {
                    switch (mMode)
                    {
                        case FieldEditorModeEnum.ClassFormDefinition:
                        case FieldEditorModeEnum.BizFormDefinition:
                        case FieldEditorModeEnum.SystemTable:
                                       

                            // add new column to specified table  
                            try
                            {
                                TableManager.AddTableColumn(tableName, newColumnName, newColumnType, newColumnAllowNull, newColumnDefaultValue);
                            }
                            catch (Exception ex)
                            {
                                lblError.Visible = true;
                                lblError.Text = ex.Message;
                                return;
                            }

                        break;
                    }
                }
                // some error has occured
                else
                {
                    lblError.Visible = true;
                    lblError.Text = errorMessage;
                    return;
                }

            }
            // ------------------------------
            // existing node
            // ------------------------------
            else
            {

                // get info whether it is a primary key or system fild
                ffiUpdated.PrimaryKey = ffi.PrimaryKey;
                ffiUpdated.System = ffi.System;

                // if attribute is a primary key
                if (ffi.PrimaryKey)
                {

                    // check if the attribute type is integer number
                    if (ffiUpdated.DataType != FormFieldDataTypeEnum.Integer)
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorPKNotInteger");
                    }

                    // check if allow empty is disabled
                    //if (this.chkAllowEmpty.Checked)
                    if (ffiUpdated.AllowEmpty)
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorPKAllowsNulls");
                    }

                    // check that the field type is label
                    if (ffiUpdated.FieldType != FormFieldControlTypeEnum.LabelControl)
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorPKisNotLabel");
                    }

                    // some error has occured
                    if (errorMessage != "")
                    {
                        lblError.Visible = true;
                        lblError.Text = ResHelper.GetString("TemplateDesigner.ErrorPKThisIsPK") + errorMessage;
                        return;
                    }
                }

                // if table column update is needed
                if (((ffi.PrimaryKey) && (ffi.Name != ffiUpdated.Name)) ||
                     ((!ffi.PrimaryKey) &&
                        ((ffi.Name != ffiUpdated.Name) ||
                         (ffi.DataType != ffiUpdated.DataType) ||
                         (ffi.AllowEmpty != ffiUpdated.AllowEmpty) ||
                         (ffi.Size != ffiUpdated.Size) ||
                         (ffi.DefaultValue != ffiUpdated.DefaultValue))
                     )
                   )
                {
                    // set variables needed for changes in DB                
                    oldColumnName = ffi.Name;
                    newColumnName = ffiUpdated.Name;
                    newColumnAllowNull = ffiUpdated.AllowEmpty;
                    newColumnSize = "";

                    // set implicit default value
                    if (!(newColumnAllowNull) && (ffiUpdated.DefaultValue == ""))
                    {
                        switch (ffiUpdated.DataType)
                        {
                            case FormFieldDataTypeEnum.Integer:
                            case FormFieldDataTypeEnum.Decimal:
                            case FormFieldDataTypeEnum.Boolean:

                                newColumnDefaultValue = "0";

                                break;


                            case FormFieldDataTypeEnum.Text:
                            case FormFieldDataTypeEnum.LongText:

                                newColumnDefaultValue = "";
                                break;

                            case FormFieldDataTypeEnum.DateTime:

                                newColumnDefaultValue = DateTime.Now.ToString(ciEnUs.DateTimeFormat);
                                break;

                            case FormFieldDataTypeEnum.File:
                                // 32 digits, empty Guid
                                newColumnDefaultValue = Guid.Empty.ToString();
                                break;
                        }
                    }

                    // Check if default value is in required format
                    else if (ffiUpdated.DefaultValue != "")
                    {
                        switch (ffiUpdated.DataType)
                        {

                            case FormFieldDataTypeEnum.Integer:
                                try
                                {
                                    int i = Int32.Parse(ffiUpdated.DefaultValue);
                                    newColumnDefaultValue = i.ToString();
                                }
                                catch
                                {
                                    newColumnDefaultValue = "0";
                                    errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueInteger");
                                }
                                break;

                            case FormFieldDataTypeEnum.Decimal:
                                try
                                {
                                    double d = Double.Parse(ffiUpdated.DefaultValue);
                                    newColumnDefaultValue = d.ToString();
                                }
                                catch
                                {
                                    newColumnDefaultValue = "0";
                                    errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueDouble");
                                }
                                break;

                            case FormFieldDataTypeEnum.DateTime:
                                try
                                {
                                    DateTime dat = DateTime.Parse(ffiUpdated.DefaultValue, ciEnUs.DateTimeFormat);
                                    newColumnDefaultValue = dat.ToString();
                                }
                                catch
                                {
                                    newColumnDefaultValue = DateTime.Now.ToString(ciEnUs.DateTimeFormat);
                                    errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueDateTime");
                                }
                                break;

                            case FormFieldDataTypeEnum.File:
                                try
                                {
                                    Guid g = new Guid(ffiUpdated.DefaultValue);
                                    newColumnDefaultValue = g.ToString();

                                }
                                catch
                                {
                                    newColumnDefaultValue = Guid.Empty.ToString();
                                    errorMessage = ResHelper.GetString("TemplateDesigner.ErrorDefaultValueGuid");

                                }
                                break;

                            case FormFieldDataTypeEnum.LongText:
                            case FormFieldDataTypeEnum.Text:
                            case FormFieldDataTypeEnum.Boolean:

                                newColumnDefaultValue = ffiUpdated.DefaultValue;
                                break;
                        }
                    }


                    // set column type
                    switch (ffiUpdated.DataType)
                    {

                        case FormFieldDataTypeEnum.Integer:
                            newColumnType = "int";
                            break;

                        case FormFieldDataTypeEnum.Decimal:
                            newColumnType = "float";
                            break;

                        case FormFieldDataTypeEnum.Boolean:
                            newColumnType = "bit";
                            break;

                        case FormFieldDataTypeEnum.DateTime:
                            newColumnType = "datetime";
                            break;

                        case FormFieldDataTypeEnum.LongText:
                            newColumnType = "ntext";
                            break;

                        case FormFieldDataTypeEnum.File:
                            newColumnType = "uniqueidentifier";
                            break;

                        default:
                            newColumnSize = Convert.ToString(ffiUpdated.Size);
                            newColumnType = "nvarchar(" + newColumnSize + ")";
                            break;
                    }

                    if (errorMessage == "")
                    {

                        switch (mMode)
                        {
                            case FieldEditorModeEnum.ClassFormDefinition:
                            case FieldEditorModeEnum.BizFormDefinition:
                            case FieldEditorModeEnum.SystemTable:
                                       
                                try
                                {
                                    // change table column id database  
                                    TableManager.AlterTableColumn(tableName, oldColumnName, newColumnName, newColumnType, newColumnAllowNull, newColumnDefaultValue);
                                }
                                catch (Exception ex)
                                {
                                    lblError.Visible = true;
                                    lblError.Text = ex.Message;
                                    return;
                                }

                            break;
                        }
                    }
                    else// some error has ocured
                    {
                        lblError.Visible = true;
                        lblError.Text = errorMessage;
                        return;
                    }

                } // end update needed

            } // end existing node

            // update xml schema and xml form definition
            //------------------------------------------

            if ((newColumnDefaultValue != null) && (ffiUpdated.DefaultValue != newColumnDefaultValue))
            {
              //  ffiUpdated.DefaultValue = newColumnDefaultValue;
            }

            if (isNewItem)
            {
                // insert new field
                InsertFormItem(ffiUpdated);
            }
            else
            {
                // update current field 
                fi.UpdateFormField(ffi.Name, ffiUpdated);

            }

        }
        else if (SelectedItemType == FieldEditorSelectedItemEnum.Category)
        {
            //fill FormCategoryInfo structure with original data
            fci = fi.GetFormCategory(SelectedItemName);

            // fill FormCategoryInfo structure with updated form data        
            fciUpdated = new FormCategoryInfo();
            fciUpdated.CategoryName = txtCategoryName.Text.Trim();

            // check if the category name is empty 
            if (fciUpdated.CategoryName == "")
            {
                this.lblError.Visible = true;
                this.lblError.Text = ResHelper.GetString("TemplateDesigner.ErrorCategoryNameEmpty");
                return;
            }

            // determine whether it is a new attribute or not
            isNewItem = (fci == null) ? true : false;

            // check if the category name already exists      
            if (isNewItem || (fci.CategoryName != fciUpdated.CategoryName))
            {
                string[] categoryNames = fi.GetCategoryNames();

                if (categoryNames != null)
                {
                    foreach (string name in categoryNames)
                    {
                        // if name already exists
                        if (fciUpdated.CategoryName == name)
                        {
                            this.lblError.Visible = true;
                            this.lblError.Text = ResHelper.GetString("TemplateDesigner.ErrorExistingCategoryName");
                            return;
                        }
                    }
                }

            }

            if (isNewItem)
            {
                // insert new category
                InsertFormItem(fciUpdated);
            }
            else
            {
                // update current 
                fi.UpdateFormCategory(fci.CategoryName, fciUpdated);

            }
        }

        // make changes in database
        //------------------------------------------
        if (SelectedItemType != 0)
        {

            bool isError = false;
            switch (mMode)
            {

                case FieldEditorModeEnum.WebPartProperties:

                    if (wpi != null)
                    {
                        // update xml definition
                        wpi.WebPartProperties = fi.GetXmlDefinition();

                        try
                        {
                            WebPartInfoProvider.SetWebPartInfo(wpi);
                        }
                        catch
                        {
                            isError = true;
                        }
                    }
                    else
                    {
                        isError = true;
                    }
                    break;

                case FieldEditorModeEnum.ClassFormDefinition:
                case FieldEditorModeEnum.BizFormDefinition:
                case FieldEditorModeEnum.SystemTable:

                    if (dci != null)
                    {
                        // update xml definition
                        dci.ClassFormDefinition = fi.GetXmlDefinition();

                        //update xml schema
                        dci.ClassXmlSchema = TableManager.GetXmlSchema(dci.ClassTableName);

                        // when updating existing field
                        if (ffi != null)
                        {
                            // update ClassNodeNameSource field
                            if (dci.ClassNodeNameSource == ffi.Name)
                            {
                                dci.ClassNodeNameSource = ffiUpdated.Name;
                            }
                        }

                        // update changes in DB
                        try
                        {
                            DataClassInfoProvider.SetDataClass(dci);
                        }
                        catch
                        {
                            isError = true;
                        }

                        if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
                        {
                            // regenerate queries                            
                            if ( !dci.ClassIsDocumentType || dci.ClassIsCoupledClass)
                            {
                                SqlGenerator.GenerateQuery(ClassName, "select", SqlOperationTypeEnum.SelectQuery, false);
                                SqlGenerator.GenerateQuery(ClassName, "delete", SqlOperationTypeEnum.DeleteQuery, false);
                                SqlGenerator.GenerateQuery(ClassName, "insert", SqlOperationTypeEnum.InsertQuery, true);
                                SqlGenerator.GenerateQuery(ClassName, "update", SqlOperationTypeEnum.UpdateQuery, false);
                                SqlGenerator.GenerateQuery(ClassName, "selectall", SqlOperationTypeEnum.SelectAll, false);
                            }

                            if (dci.ClassIsDocumentType)
                            {
                                SqlGenerator.GenerateQuery(ClassName, "searchtree", SqlOperationTypeEnum.SearchTree, false);
                                SqlGenerator.GenerateQuery(ClassName, "selectdocuments", SqlOperationTypeEnum.SelectDocuments, false);
                                SqlGenerator.GenerateQuery(ClassName, "selectversions", SqlOperationTypeEnum.SelectVersions, false);
                            }
                        }
                    }
                    else
                    {
                        isError = true;
                    }
                    break;
            }
            if (isError)
            {
                lblError.Visible = true;
                lblError.Text = "[ FieldEditor.SaveSelectedField() ]: Changes were not saved in database.";
            }
            else
            {
                IsNewItemEdited = false;
                if (SelectedItemType == FieldEditorSelectedItemEnum.Category)
                {
                    Reload(categPreffix + fciUpdated.CategoryName);
                }
                else if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
                {
                    Reload(fieldPreffix + ffiUpdated.Name);
                }
                                
                this.lblError.Visible = false;
                this.lblInfo.Visible = true;
                this.lblInfo.Text = ResHelper.GetString("TemplateDesigner.InfoChangesSaved");                
            }
        }
    }


    /// <summary>
    /// When attribute type selected index is changed
    /// </summary>
    protected void lstAttributeType_SelectedIndexChanged(System.Object sender, System.EventArgs e)
    {

        EnableOrDisableAttributeSize();
        LoadFieldTypes(IsFieldPrimary);
        ShowAdvancedOptions();
        ShowValidationOptions(IsFieldPrimary);
        ShowDefaultValue();
    }


    /// <summary>
    /// Enable or disable size attribute
    /// </summary>
    private void EnableOrDisableAttributeSize()
    {
        if (lstAttributeType.SelectedValue == "text")
        {
            this.txtAttributeSize.Enabled = true;
        }
        else
        {
            this.txtAttributeSize.Enabled = false;
            this.txtAttributeSize.Text = "";
        }
    }


    /// <summary>
    /// New category button clicked
    /// </summary>
    protected void btnNewCategory_Click(System.Object sender, System.Web.UI.ImageClickEventArgs e)
    {

        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            // check 'EditForm' permission
            if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.form", "EditForm"))
            {
                CMSDeskPage.RedirectToAccessDenied("cms.form", "EditForm");
            }
        }

        if (IsNewItemEdited)
        {
            // only one new item can be edited
            ShowMessage(ResHelper.GetString("TemplateDesigner.ErrorCannotCreateAnotherNewItem"));
        }
        else
        {

            // create #categ##new# item in list
            ListItem newListItem = new ListItem(ResHelper.GetString("TemplateDesigner.NewCategory"), newCategPreffix);

            if ((lstAttributes.Items.Count > 0) && (lstAttributes.SelectedIndex >= 0))
            {
                // add behind the selected item 
                this.lstAttributes.Items.Insert(lstAttributes.SelectedIndex + 1, newListItem);
            }
            else
            {
                // add at the end of the item collection
                this.lstAttributes.Items.Add(newListItem);
            }

            // select new item 
            this.lstAttributes.SelectedIndex = lstAttributes.Items.IndexOf(newListItem);

            SelectedItemType = FieldEditorSelectedItemEnum.Category;
            SelectedItemName = newCategPreffix;

            LoadDefaultCategoryEditForm();

            IsNewItemEdited = true;
        }
    }


    /// <summary>
    /// New attribute button clicked
    /// </summary>
    protected void btnNewAttribute_Click(System.Object sender, System.Web.UI.ImageClickEventArgs e)
    {
        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            // check 'EditForm' permission
            if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.form", "EditForm"))
            {
                CMSDeskPage.RedirectToAccessDenied("cms.form", "EditForm");
            }
        }

        IsFieldPrimary = false;

        if (IsNewItemEdited)
        {
            // only one new item can be edited
            ShowMessage(ResHelper.GetString("TemplateDesigner.ErrorCannotCreateAnotherNewItem"));
        }
        else
        {
            // enable editing
            txtSimpleColumnName.Enabled = true;
            txtSimpleColumnName.ReadOnly = false;
            lstSimpleFieldType.Enabled = true; 

            // create #new# attribute in attribute list
            ListItem newListItem = new ListItem(ResHelper.GetString("TemplateDesigner.NewAttribute"), newFieldPreffix);

            if ( (lstAttributes.Items.Count > 0) && (lstAttributes.SelectedIndex >= 0) )
            {
                // add behind the selected item 
                this.lstAttributes.Items.Insert(lstAttributes.SelectedIndex + 1, newListItem);
            }
            else
            {
                // add at the end of the item collection
                this.lstAttributes.Items.Add(newListItem);
            }
            
            // select new item 
            this.lstAttributes.SelectedIndex = lstAttributes.Items.IndexOf(newListItem);

            SelectedItemType = FieldEditorSelectedItemEnum.Field;
            SelectedItemName = newFieldPreffix;

            IsNewItemEdited = true;
            SetUniversalControls();            

            LoadDefaultAttributeEditForm();            
        }
    }


    /// <summary>
    /// Delete attribute button clicked
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnDeleteItem_Click(System.Object sender, System.Web.UI.ImageClickEventArgs e)
    {
        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            // check 'EditForm' permission
            if (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.form", "EditForm"))
            {
                CMSDeskPage.RedirectToAccessDenied("cms.form", "EditForm");
            }
        }

        FormFieldInfo ffiSelected = null;
        FormCategoryInfo fciSelected = null;
        DataClassInfo dci = null;
        WebPartInfo wpi = null;        
        bool isError = false;
        string newSelectedValue = null;
        string deletedItemPreffix = "";

        // load current xml form definition
        this.LoadFormDefinition();

        if ((SelectedItemName != "") && (!IsNewItemEdited))
        {
            if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
            {

                ffiSelected = fi.GetFormField(SelectedItemName);

                deletedItemPreffix = fieldPreffix;

                if (ffiSelected != null)
                {

                    // do not allow deleting of the primary key
                    if (ffiSelected.PrimaryKey)
                    {
                        this.lblError.Visible = true;
                        this.lblError.Text = ResHelper.GetString("TemplateDesigner.ErrorCannotDeletePK");
                        return;
                    }

                    // do not allow deleting of the system field
                    if (ffiSelected.System)
                    {
                        this.lblError.Visible = true;
                        this.lblError.Text = ResHelper.GetString("TemplateDesigner.ErrorCannotDeleteSystemField");
                        return;
                    }

                    // remove specifield field from xml form definition
                    fi.RemoveFormField(SelectedItemName);

                    switch (mMode)
                    {
                        case FieldEditorModeEnum.WebPartProperties:

                            wpi = WebPartInfoProvider.GetWebPartInfo(mWebPartId);
                            if (wpi != null)
                            {
                                wpi.WebPartProperties = fi.GetXmlDefinition();
                                WebPartInfoProvider.SetWebPartInfo(wpi);
                            }
                            else
                            {
                                isError = true;
                            }
                            break;

                        case FieldEditorModeEnum.ClassFormDefinition:
                        case FieldEditorModeEnum.BizFormDefinition:
                        case FieldEditorModeEnum.SystemTable:

                            dci = DataClassInfoProvider.GetDataClass(mClassName);

                            if (dci != null)
                            {
                                // update xml definition
                                dci.ClassFormDefinition = fi.GetXmlDefinition();

                                try
                                {
                                    // remove corresponding column from table
                                    TableManager.DropTableColumn(DataClassInfoProvider.GetDataClass(ClassName).ClassTableName, SelectedItemName);
                                }
                                catch
                                {
                                    // column doesnt exists
                                }

                                //update xml schema
                                dci.ClassXmlSchema = TableManager.GetXmlSchema(dci.ClassTableName);
                                
                                // deleted field is used as ClassNodeNameSource -> remove node name source
                                if (dci.ClassNodeNameSource == SelectedItemName)
                                {
                                    dci.ClassNodeNameSource = "";
                                }

                                // update changes in database
                                DataClassInfoProvider.SetDataClass(dci);

                                // regenerate queries                                
                                if (!(dci.ClassIsDocumentType && !dci.ClassIsCoupledClass))
                                {
                                    SqlGenerator.GenerateQuery(ClassName, "select", SqlOperationTypeEnum.SelectQuery, false);
                                    SqlGenerator.GenerateQuery(ClassName, "delete", SqlOperationTypeEnum.DeleteQuery, false);
                                    SqlGenerator.GenerateQuery(ClassName, "insert", SqlOperationTypeEnum.InsertQuery, true);
                                    SqlGenerator.GenerateQuery(ClassName, "update", SqlOperationTypeEnum.UpdateQuery, false);
                                }

                                if (dci.ClassIsDocumentType)
                                {
                                    SqlGenerator.GenerateQuery(ClassName, "searchtree", SqlOperationTypeEnum.SearchTree, false);
                                    SqlGenerator.GenerateQuery(ClassName, "selectdocuments", SqlOperationTypeEnum.SelectDocuments, false);
                                    SqlGenerator.GenerateQuery(ClassName, "selectversions", SqlOperationTypeEnum.SelectVersions, false);
                                }

                            }
                            else
                            {
                                isError = true;
                            }
                            break;


                    }

                }
            }
            else if (SelectedItemType == FieldEditorSelectedItemEnum.Category)
            {
                fciSelected = fi.GetFormCategory(SelectedItemName);

                deletedItemPreffix = categPreffix;

                // remove specifield category from xml form definition
                fi.RemoveFormCategory(SelectedItemName);

                switch (mMode)
                {
                    case FieldEditorModeEnum.WebPartProperties:

                        wpi = WebPartInfoProvider.GetWebPartInfo(mWebPartId);
                        if (wpi != null)
                        {
                            wpi.WebPartProperties = fi.GetXmlDefinition();
                            WebPartInfoProvider.SetWebPartInfo(wpi);
                        }
                        else
                        {
                            isError = true;
                        }
                        break;

                    case FieldEditorModeEnum.ClassFormDefinition:
                    case FieldEditorModeEnum.BizFormDefinition:
                    case FieldEditorModeEnum.SystemTable:

                        dci = DataClassInfoProvider.GetDataClass(mClassName);

                        if (dci != null)
                        {
                            // update xml definition
                            dci.ClassFormDefinition = fi.GetXmlDefinition();

                            // update changes in database
                            DataClassInfoProvider.SetDataClass(dci);
                        }
                        else
                        {
                            isError = true;
                        }
                        break;
                }

            }

            if (isError)
            {
                lblError.Visible = true;
                lblError.Text = "[ FieldEditor.btnDeleteItem_Click() ]: Changes were not saved to database."; // TODO
            }


        }
        else
        {
            // "delete" new item from the list
            IsNewItemEdited = false;
        }

        // set new selected value
        ListItem deletedItem = lstAttributes.Items.FindByValue(deletedItemPreffix + SelectedItemName);
        int deletedItemIndex = lstAttributes.Items.IndexOf(deletedItem);

        if ((deletedItemIndex > 0) && (lstAttributes.Items[deletedItemIndex - 1] != null))
        {
            newSelectedValue = lstAttributes.Items[deletedItemIndex - 1].Value;
        }

        // reload data
        Reload(newSelectedValue);
    }


    /// <summary>
    /// Fill field types list
    /// </summary>
    /// <param name="isPrimary">Determines whether the attribute is primary key</param>
    private void LoadFieldTypes(bool isPrimary)
    {
        bool isSimplifiedMode = (SelectedMode == FieldEditorSelectedModeEnum.Simplified) ? true : false;        

        // clear list
        this.lstFieldType.Items.Clear();
        this.lstFieldType.DataSource = FormHelper.GetFieldControlTypesWithUserControls(this.lstAttributeType.SelectedValue, Mode, isSimplifiedMode, isPrimary);
        this.lstFieldType.DataBind();    
    }

    /*
    /// <summary>
    /// Fill user controls drop down list.
    /// </summary>    
    private void LoadUserControls()
    {
        // clear list
        this.drpUserControls.Items.Clear();

        this.drpUserControls.DataSource = FormHelper.GetFormUserControlsForDataType(this.lstAttributeType.SelectedValue, true);        
        this.drpUserControls.DataBind();
        this.drpUserControls.SelectedIndex = 0;
    }
    */

    /// <summary>
    /// Show javasript's alert message
    /// </summary>
    /// <param name="message">Message to show</param>
    private void ShowMessage(string message)
    {
        this.ltlScript.Text = @"<script language=""javascript"">" + " alert('" + message.Replace("'", "\'") + "'); " + "</script>";
    }


    /// <summary>
    /// Called when field type selected index changed
    /// </summary>
    protected void lstFieldType_SelectedIndexChanged(object sender, EventArgs e)
    {        
        EnableOrDisableAttributeSize();
        ShowAdvancedOptions();

        if (SelectedMode == FieldEditorSelectedModeEnum.Simplified)
        {
            ShowDefaultValue();
        }

        ShowValidationOptions(IsFieldPrimary);
    }

    /// <summary>
    /// Called when source field selected index changed
    /// </summary>
    protected void drpSourceField_SelectedIndexChanged(object sender, EventArgs e)
    {
        if (mMode == FieldEditorModeEnum.ClassFormDefinition)
        {
            DataClassInfo dci = DataClassInfoProvider.GetDataClass(ClassName);
            if (dci != null)
            {
                // set document name source field
                if (drpSourceField.SelectedValue == "")
                {
                    // use extra field
                    dci.ClassNodeNameSource = "";
                }
                else
                {
                    // use specified name
                    dci.ClassNodeNameSource = drpSourceField.SelectedValue;
                }

                try
                {
                    DataClassInfoProvider.SetDataClass(dci);

                    this.lblError.Visible = false;
                    this.lblInfo.Visible = true;
                    this.lblInfo.Text = ResHelper.GetString("TemplateDesigner.SourceFieldSaved");
                }
                catch
                {
                }
            }
            else
            {
                lblError.Visible = true;
                lblError.Text = "[ FieldEditor.drpSourceField_SelectedIndexChanged() ]: Changes were not saved in database."; //TODO
            }
        }
    }

    /// <summary>
    /// Show advanced options according to selected field type
    /// </summary>
    private void ShowAdvancedOptions()
    {

        this.pnlCalendarSettings.Visible = false;
        this.pnlDropDownListSettings.Visible = false;
        this.pnlHTMLAreaSettings.Visible = false;
        this.pnlTextAreaSettings.Visible = false;        
        this.pnlMultipleChoice.Visible = false;
        this.pnlSimpleTextBox.Visible = false;
        this.pnlImageSelection.Visible = false;

        string selectedFieldType = this.lstFieldType.SelectedValue.ToLower();
        string selectedControlName = "";        

        // display textbox with max length size when in simplified mode and when default field type for user controls is TEXT
        if ( (SelectedMode == FieldEditorSelectedModeEnum.Simplified) && (selectedFieldType.StartsWith(controlPrefix)) )
        {
            selectedControlName = selectedFieldType.Substring(controlPrefix.Length);
            string[] controlDefaultDataType = GetUserControlDefaultDataType(selectedControlName);            
                        
            if (controlDefaultDataType[0] == FormFieldDataTypeCode.TEXT)
            {
                this.pnlSimpleTextBox.Visible = true;
                if (IsNewItemEdited)
                {
                    this.txtSimpleTextBoxMaxLength.Text = ValidationHelper.GetString(controlDefaultDataType[1], "");
                }
                else
                {
                    this.txtSimpleTextBoxMaxLength.Text = (ffi.Size == 0) ? "" : Convert.ToString(ffi.Size);
                }
            }                        
        }
        else
        {
            switch (selectedFieldType)
            {

                case FormFieldControlTypeCode.TEXTAREA:
                    this.pnlTextAreaSettings.Visible = true;
                    break;

                case FormFieldControlTypeCode.HTMLAREA:
                    this.pnlHTMLAreaSettings.Visible = true;
                    break;

                case FormFieldControlTypeCode.DROPDOWNLIST:
                case FormFieldControlTypeCode.RADIOBUTTONS:
                    this.pnlDropDownListSettings.Visible = true;
                    break;

                case FormFieldControlTypeCode.CALENDAR:
                    this.pnlCalendarSettings.Visible = true;
                    break;

                case FormFieldControlTypeCode.MULTIPLECHOICE:
                    this.pnlMultipleChoice.Visible = true;
                    break;

                case FormFieldControlTypeCode.IMAGEPATH:
                    this.pnlImageSelection.Visible = true;
                    break;

                //only for simplified mode
                case FormFieldControlTypeCode.TEXTBOX:
                    if (SelectedMode == FieldEditorSelectedModeEnum.Simplified)
                    {
                        this.pnlSimpleTextBox.Visible = true;
                        this.txtSimpleTextBoxMaxLength.Text = "";
                    }
                    break;
            }
        }

    }


    /// <summary>
    /// Show validation options according to selected attribute type
    /// </summary>
    /// <param name="isPrimary">Indicates whether it is a primary key</param> 
    private void ShowValidationOptions(bool isPrimary)
    {
        pnlSectionValidation.Visible = false;

        pnlTextValidation.Visible = false;
        pnlDateTimeValidation.Visible = false;
        pnlNumberValidation.Visible = false;
        pnlFileValidation.Visible = false;
        pnlErrorMessageValidation.Visible = false;

        // Hide Min max length for selection controls
        pnlMinMaxLengthValidation.Visible = true;
        switch (lstFieldType.SelectedValue.ToLower())
        {
            case FormFieldControlTypeCode.DROPDOWNLIST:
            case FormFieldControlTypeCode.RADIOBUTTONS:
            case FormFieldControlTypeCode.MULTIPLECHOICE:
                pnlMinMaxLengthValidation.Visible = false;
                break;        
        }

        if ((lstFieldType.SelectedValue != "label") && (!isPrimary))
        {

            switch (lstAttributeType.SelectedValue.ToLower())
            {
                case "text":
                case "longtext":
                    pnlSectionValidation.Visible = true;
                    pnlTextValidation.Visible = true;
                    pnlErrorMessageValidation.Visible = true;
                    break;

                case "datetime":
                    pnlSectionValidation.Visible = true;
                    pnlDateTimeValidation.Visible = true;
                    pnlErrorMessageValidation.Visible = true;
                    break;

                case "integer":
                case "double":
                    pnlSectionValidation.Visible = true;
                    pnlNumberValidation.Visible = true;
                    pnlErrorMessageValidation.Visible = true;
                    break;

                case "file":
                    pnlSectionValidation.Visible = true;
                    pnlFileValidation.Visible = true;
                    break;

            }
        }

    }


    /// <summary>
    /// Validates advanced options and returns validation errorr message
    /// </summary>
    /// <returns></returns>
    private string ValidateAdvancedOptions()
    {

        string errorMessage = "";

        switch (this.lstFieldType.SelectedValue.ToLower())
        {

            case "textarea":
                // check cols and rows values
                if ((txtTextAreaCols.Text.Trim() != "") && !(ValidationHelper.IsInteger(txtTextAreaCols.Text)))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorTextAreaColsNotNumber");
                }
                if ((txtTextAreaRows.Text.Trim() != "") && !(ValidationHelper.IsInteger(txtTextAreaRows.Text)))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorTextAreaRowsNotNumber");
                }
                if (txtTextAreaSize.Text.Trim() != "")
                {
                    if (!(ValidationHelper.IsInteger(txtTextAreaSize.Text)))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorTextAreaSizeNotNumber");
                    }
                    else if (Int32.Parse(txtTextAreaSize.Text) <= 0)
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorTextAreaSizeLessThanZero");
                    }
                }
                break;

            case "htmlarea":
                // check width and height values
                if ((txtHTMLAreaHeight.Text.Trim() != "") && !(ValidationHelper.IsInteger(txtHTMLAreaHeight.Text)))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorHTMLAreaHeightNotNumber");
                }
                if ((txtHTMLAreaWidth.Text.Trim() != "") && !(ValidationHelper.IsInteger(txtHTMLAreaWidth.Text)))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorHTMLAreaWidthNotNumber");
                }
                if (txtHTMLAreaSize.Text.Trim() != "")
                {
                    if (!(ValidationHelper.IsInteger(txtHTMLAreaSize.Text)))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorHTMLAreaSizeNotNumber");
                    }
                    else if (Int32.Parse(txtHTMLAreaSize.Text) <= 0)
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorHTMLAreaSizeLessThanZero");
                    }
                }
                break;

            case "dropdownlist":
            case "radiobuttons":
                // check options or sql query
                if (radDropDownListSQL.Checked && (this.txtDropDownListOptions.Text.Trim() == ""))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorDropDownListQueryEmpty");
                }
                if (this.radDropDownListOptions.Checked)
                {

                    if (this.txtDropDownListOptions.Text.Trim() == "")
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorDropDownListOptionsEmpty");
                    }
                    else
                    {
                        int lineIndex = 0;
                        string[] lines = this.txtDropDownListOptions.Text.Split('\n');
                        for (lineIndex = 0; lineIndex <= lines.GetUpperBound(0); lineIndex++)
                        {

                            if ((lines[lineIndex] != null) && (lines[lineIndex].Trim() != "") && (lines[lineIndex].Split(';').Length != 2))
                            {
                                errorMessage += ResHelper.GetString("TemplateDesigner.ErrorDropDownListOptionsInvalidFormat");
                            }
                        }
                    }
                }
                this.pnlDropDownListSettings.Visible = true;
                break;

            case "multiplechoice":
                // check options or sql query
                if (radMultipleChoiceSQL.Checked && (this.txtMultipleChoiceOptions.Text.Trim() == ""))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMultipleChoiceQueryEmpty");
                }
                if (this.radMultipleChoiceOptions.Checked)
                {
                    if (this.txtMultipleChoiceOptions.Text.Trim() == "")
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMultipleChoiceOptionsEmpty");
                    }
                    else
                    {
                        int lineIndex = 0;
                        string[] lines = this.txtMultipleChoiceOptions.Text.Split('\n');
                        for (lineIndex = 0; lineIndex <= lines.GetUpperBound(0); lineIndex++)
                        {
                            if ((lines[lineIndex] != null) && (lines[lineIndex].Trim() != "") && (lines[lineIndex].Split(';').Length != 2))
                            {
                                errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMultipleChoiceOptionsInvalidFormat");
                            }
                        }
                    }
                }
                this.pnlMultipleChoice.Visible = true;
                break;

            case "calendar":
                // do nothing
                break;
        }

        return errorMessage;
    }

    /// <summary>
    /// Validates form and returns validation errorr message
    /// </summary>
    /// <returns></returns>
    private string ValidateForm()
    {
        string errorMessage = "";
        const string INVALIDCHARACTERS = @".,;'`:/\*|?""&%$!-+=()[]{} ";
        txtAttributeName.Text = txtAttributeName.Text.Trim();

        bool MinAndMaxLengthInCorrectFormat = true;
        bool MinAndMaxValueInCorrectFormat = true;


        // validation of the controls shared by both modes (simplified and advanced)
        //--------------------------------------------------------------------------

        // check if attribute name isn't empty
        if (txtAttributeName.Text == "")
        {
            errorMessage += ResHelper.GetString("TemplateDesigner.ErrorEmptyAttributeName");
        }

        // check if attribute name starts with a letter or '_' (if it is an identificator)
        if (!ValidationHelper.IsIdentificator(txtAttributeName.Text))
        {
            errorMessage += ResHelper.GetString("TemplateDesigner.ErrorAttributeNameDoesNotStartWithLetter");
        }

        // check attribute name for invalid characters
        for (int i = 0; i <= INVALIDCHARACTERS.Length - 1; i++)
        {
            if (txtAttributeName.Text.Contains(Convert.ToString(INVALIDCHARACTERS[i])))
            {
                errorMessage += ResHelper.GetString("TemplateDesigner.ErrorInvalidCharacter") + INVALIDCHARACTERS + ".";
            }
        }

        // check if field caption is entered
        if (this.chkDisplayInForm.Checked)
        {
            if (this.txtFieldCaption.Text.Trim() == "")
            {
                errorMessage += ResHelper.GetString("TemplateDesigner.ErrorEmptyFieldCaption") + " ";
            }
        }

        // validation of the advanced mode controls
        //-----------------------------------------
        if (SelectedMode == FieldEditorSelectedModeEnum.Advanced)
        {

            // check attribute size value
            if (lstAttributeType.SelectedValue.ToLower() == "text")
            {
                // attribute size is empty -> error
                if (txtAttributeSize.Text == "")
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorEmptyAttributeSize");
                }
                else
                {
                    // attribute size is invalid -> error
                    if ((lstAttributeType.SelectedValue.ToLower() == "text") && (ValidationHelper.GetInteger(txtAttributeSize.Text,0) == 0))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorInvalidAttributeSize");
                    }

                    // attribute size is invalid -> error
                    if ((lstFieldType.SelectedValue.ToLower() == "htmlarea") && (ValidationHelper.GetInteger(txtAttributeSize.Text,0) == 0))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.HtmlArea.ErrorInvalidAttributeSize");
                    }
                }
            }
            
            // check min length value
            if (txtMinLength.Text.Trim() != "")
            {
                if ((!ValidationHelper.IsInteger(txtMinLength.Text)) ||
                     ((ValidationHelper.IsInteger(txtMinLength.Text)) && (Convert.ToInt32(txtMinLength.Text.Trim()) < 0)))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMinLengthNotInteger");
                    MinAndMaxLengthInCorrectFormat = false;
                }
            }                       

            // check max length value
            if (txtMaxLength.Text.Trim() != "")
            {
                if ((!ValidationHelper.IsInteger(txtMaxLength.Text)) ||
                     ((ValidationHelper.IsInteger(txtMaxLength.Text)) && (Convert.ToInt32(txtMaxLength.Text.Trim()) < 0)))                
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMaxLengthNotInteger");
                    MinAndMaxLengthInCorrectFormat = false;
                }
            }

            // min and max length are specified and in correct format -> check if min length is less than max length
            if ((txtMinLength.Text.Trim() != "") && (txtMaxLength.Text.Trim() != "") && (MinAndMaxLengthInCorrectFormat))
            {
                if ((Convert.ToInt32(txtMinLength.Text.Trim())) > (Convert.ToInt32(txtMaxLength.Text.Trim())))
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMinLengthGreater");
                }
            }
            

            if (lstAttributeType.SelectedValue == FormFieldDataTypeCode.DOUBLE)
            {

                // check min value
                if ((txtMinValue.Text.Trim() != ""))
                {
                    if ((!ValidationHelper.IsDouble(txtMinValue.Text)) ||
                         ((ValidationHelper.IsDouble(txtMinValue.Text)) && (Convert.ToDouble(txtMinValue.Text.Trim()) < 0)))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMinValueNotDouble");
                        MinAndMaxValueInCorrectFormat = false;
                    }
                }

                // check max value
                if (txtMaxValue.Text.Trim() != "")
                {
                    if ((!ValidationHelper.IsDouble(txtMaxValue.Text)) ||
                         ((ValidationHelper.IsDouble(txtMaxValue.Text)) && (Convert.ToDouble(txtMaxValue.Text.Trim()) < 0)))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMaxValueNotDouble");
                        MinAndMaxValueInCorrectFormat = false;
                    }
                }

                // min and max value are specified and in correct format -> check if min value is less than max value
                if ((txtMinValue.Text.Trim() != "") && (txtMaxValue.Text.Trim() != "") && (MinAndMaxValueInCorrectFormat))
                {
                    if ((Convert.ToDouble(txtMinValue.Text.Trim())) > (Convert.ToDouble(txtMaxValue.Text.Trim())))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMinValueGreater");
                    }
                }
            }
            else if (lstAttributeType.SelectedValue == FormFieldDataTypeCode.INTEGER) 
            {
                // check min value
                if ((txtMinValue.Text.Trim() != ""))
                {
                    if ((!ValidationHelper.IsInteger(txtMinValue.Text)) ||
                         ((ValidationHelper.IsInteger(txtMinValue.Text)) && (Convert.ToInt32(txtMinValue.Text.Trim()) < 0)))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMinValueNotInteger");
                        MinAndMaxValueInCorrectFormat = false;
                    }
                }

                // check max value
                if (txtMaxValue.Text.Trim() != "")
                {
                    if ((!ValidationHelper.IsInteger(txtMaxValue.Text)) ||
                         ((ValidationHelper.IsInteger(txtMaxValue.Text)) && (Convert.ToInt32(txtMaxValue.Text.Trim()) < 0)))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMaxValueNotInteger");
                        MinAndMaxValueInCorrectFormat = false;
                    }
                }

                // min and max value are specified and in correct format -> check if min value is less than max value
                if ((txtMinValue.Text.Trim() != "") && (txtMaxValue.Text.Trim() != "") && (MinAndMaxValueInCorrectFormat))
                {
                    if ((Convert.ToInt32(txtMinValue.Text.Trim())) > (Convert.ToInt32(txtMaxValue.Text.Trim())))
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorMinValueGreater");
                    }
                }
            }
        }
        // validation of the simplified mode controls
        else
        {

            // check attribute size value
            if (lstFieldType.SelectedValue.ToLower() == FormFieldControlTypeCode.TEXTBOX)
            {
                // textbox length size is empty -> error
                if (txtSimpleTextBoxMaxLength.Text == "")
                {
                    errorMessage += ResHelper.GetString("TemplateDesigner.ErrorEmptyTextBoxMaxLength");
                }
                else
                {
                    // attribute size is invalid -> error
                    try 
                    {
                        uint i = UInt32.Parse(txtSimpleTextBoxMaxLength.Text.Trim());                                                
                    }
                    catch
                    {
                        errorMessage += ResHelper.GetString("TemplateDesigner.ErrorInvalidTextBoxMaxLength") + " ";
                    }
                }
            }
        }

        return errorMessage;
    }

    /// <summary>
    /// Returns FormFieldInfo structure with form data
    /// </summary>   
    /// <param name="ffiOriginal">Original field info.</param>
    private FormFieldInfo FillFormFieldInfoStructure(FormFieldInfo ffiOriginal)
    {
        FormFieldInfo ffi = new FormFieldInfo();
        string selectedType = this.lstFieldType.SelectedValue.ToLower();

        // part of database section
        ffi.Name = this.txtAttributeName.Text.Trim();
        ffi.Caption = this.txtFieldCaption.Text.Trim();
        ffi.AllowEmpty = this.chkAllowEmpty.Checked;
        ffi.DefaultValue = GetDefaultValue();

        if (Mode == FieldEditorModeEnum.BizFormDefinition)
        {
            ffi.PublicField = this.chkPublicField.Checked;
        }
        else
        {
            ffi.PublicField = false;
        }


        string controlName = selectedType.Substring(controlPrefix.Length);

        // user control is selected
        if (selectedType.StartsWith(controlPrefix))
        {
            ffi.Settings.Add("controlname", controlName);
            ffi.FieldType = FormFieldControlTypeEnum.CustomUserControl;
        }
        // one of the default controls is selected
        else
        {
            ffi.FieldType = FormHelper.GetFormFieldControlType(selectedType);
        }
       
        // update advanced options
        switch (selectedType)
        {

            case FormFieldControlTypeCode.TEXTAREA:
                ffi.Settings.Add("cols", this.txtTextAreaCols.Text.Trim());
                ffi.Settings.Add("rows", this.txtTextAreaRows.Text.Trim());
                ffi.Settings.Add("size", this.txtTextAreaSize.Text.Trim());
                break;

            case FormFieldControlTypeCode.HTMLAREA:
                ffi.Settings.Add("width", this.txtHTMLAreaWidth.Text.Trim());
                ffi.Settings.Add("height", this.txtHTMLAreaHeight.Text.Trim());
                ffi.Settings.Add("size", this.txtHTMLAreaSize.Text.Trim());
                break;

            case FormFieldControlTypeCode.DROPDOWNLIST:
            case FormFieldControlTypeCode.RADIOBUTTONS:

                // query text 
                if (radDropDownListSQL.Checked)
                {
                    ffi.Settings.Add("query", this.txtDropDownListOptions.Text);
                }
                // dropdownlist options
                else
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    XmlNode optionsNode;
                    string[] optionItemLines;
                    XmlNode optionItemNode;

                    // add options node to xml document
                    optionsNode = xmlDoc.CreateNode(XmlNodeType.Element, "options", "");
                    xmlDoc.AppendChild(optionsNode);

                    // split input options text according to the new line character
                    optionItemLines = this.txtDropDownListOptions.Text.Split('\n');

                    // process each option line
                    foreach (string line in optionItemLines)
                    {
                        if (line.IndexOf(";") >= 0)
                        {

                            optionItemNode = xmlDoc.CreateNode(XmlNodeType.Element, "item", "");
                            if (optionItemNode != null)
                            {
                                XmlAttribute valueAttribute;
                                XmlAttribute textAttribute;

                                // value attribute
                                valueAttribute = xmlDoc.CreateAttribute("value");
                                valueAttribute.Value = line.Substring(0, line.IndexOf(";")).Trim();

                                // text attribute
                                textAttribute = xmlDoc.CreateAttribute("text");
                                textAttribute.Value = line.Substring(line.IndexOf(";") + 1).Trim();

                                // add attributes to option item node
                                optionItemNode.Attributes.Append(valueAttribute);
                                optionItemNode.Attributes.Append(textAttribute);

                                // add option item node to options node
                                optionsNode.AppendChild(optionItemNode);
                            }
                        }
                    }
                    // add OptionsNode's inner xml to hashtable 
                    ffi.Settings.Add("options", optionsNode.InnerXml);

                }
                break;

            case FormFieldControlTypeCode.MULTIPLECHOICE:

                // query text 
                if (radMultipleChoiceSQL.Checked)
                {
                    ffi.Settings.Add("query", this.txtMultipleChoiceOptions.Text);
                }
                // dropdownlist options
                else
                {
                    XmlDocument xmlDoc = new XmlDocument();
                    XmlNode optionsNode;
                    string[] optionItemLines;
                    XmlNode optionItemNode;

                    // add options node to xml document
                    optionsNode = xmlDoc.CreateNode(XmlNodeType.Element, "options", "");
                    xmlDoc.AppendChild(optionsNode);

                    // split input options text according to the new line character
                    optionItemLines = this.txtMultipleChoiceOptions.Text.Split('\n');

                    // process each option line
                    foreach (string line in optionItemLines)
                    {
                        if (line.IndexOf(";") >= 0)
                        {

                            optionItemNode = xmlDoc.CreateNode(XmlNodeType.Element, "item", "");
                            if (optionItemNode != null)
                            {
                                XmlAttribute valueAttribute;
                                XmlAttribute textAttribute;

                                // value attribute
                                valueAttribute = xmlDoc.CreateAttribute("value");
                                valueAttribute.Value = line.Substring(0, line.IndexOf(";")).Trim();

                                // text attribute
                                textAttribute = xmlDoc.CreateAttribute("text");
                                textAttribute.Value = line.Substring(line.IndexOf(";") + 1).Trim();

                                // add attributes to option item node
                                optionItemNode.Attributes.Append(valueAttribute);
                                optionItemNode.Attributes.Append(textAttribute);

                                // add option item node to options node
                                optionsNode.AppendChild(optionItemNode);
                            }
                        }
                    }
                    // add OptionsNode's inner xml to hashtable 
                    ffi.Settings.Add("options", optionsNode.InnerXml);

                }
                break;

            case FormFieldControlTypeCode.CALENDAR:

                ffi.Settings.Add("editTime", this.chkCalendarSettingsEditTime.Checked.ToString().ToLower());
                break;

            case FormFieldControlTypeCode.IMAGEPATH:
                ffi.Settings.Add("width", this.txtImageWidth.Text.Trim());
                ffi.Settings.Add("height", this.txtImageHeight.Text.Trim());                
                break;

        }
        switch (SelectedMode)
        {
        
            case FieldEditorSelectedModeEnum.Advanced:

                if (((Mode == FieldEditorModeEnum.BizFormDefinition) || (Mode == FieldEditorModeEnum.SystemTable)) 
                    && (this.lstAttributeType.SelectedValue == FormFieldDataTypeCode.FILE))
                {
                    // allow to save <guid>.<extension>
                    ffi.DataType = FormFieldDataTypeEnum.Text;
                    ffi.Size = 500;
                }
                else
                {
                    // allow to save just <guid>
                    ffi.DataType = FormHelper.GetFormFieldDataType(this.lstAttributeType.SelectedValue);
                    ffi.Size = (this.txtAttributeSize.Enabled) ? Convert.ToInt32(this.txtAttributeSize.Text.Trim()) : 0;
                }

                // part of database section                                
                ffi.Visible = this.chkDisplayInForm.Checked;
                ffi.Description = this.txtDescription.Text.Trim();                

                // validation section
                ffi.RegularExpression = txtRegExpr.Text.Trim();
                ffi.MinStringLength = (txtMinLength.Text.Trim() == "") ? -1 : Convert.ToInt32(txtMinLength.Text.Trim());
                ffi.MaxStringLength = (txtMaxLength.Text.Trim() == "") ? -1 : Convert.ToInt32(txtMaxLength.Text.Trim());
                ffi.MinNumericValue = (txtMinValue.Text.Trim() == "") ? -1 : Convert.ToDouble(txtMinValue.Text.Trim());
                ffi.MaxNumericValue = (txtMaxValue.Text.Trim() == "") ? -1 : Convert.ToDouble(txtMaxValue.Text.Trim());
                ffi.MinDateTimeValue = dateFrom.SelectedDateTime;
                ffi.MaxDateTimeValue = dateTo.SelectedDateTime;
                ffi.FileExtensions = txtFileExtensions.Text.Trim();
                ffi.ValidationErrorMessage = txtErrorMessage.Text.Trim();

                // design section
                ffi.CaptionStyle = txtCaptionStyle.Text.Trim();
                ffi.InputControlStyle = txtInputStyle.Text.Trim();

                break;
            
            case FieldEditorSelectedModeEnum.Simplified:

                ffi.Visible = true;
               
                // new field
                if (IsNewItemEdited)
                {
                    // Set field properties.
                    switch (selectedType)
                    {
                        case FormFieldControlTypeCode.TEXTBOX:
                            ffi.Size = Convert.ToInt32(txtSimpleTextBoxMaxLength.Text);
                            ffi.DataType = FormFieldDataTypeEnum.Text;
                            break;

                        case FormFieldControlTypeCode.TEXTAREA:
                        case FormFieldControlTypeCode.HTMLAREA:
                            ffi.Size = 0;
                            ffi.DataType = FormFieldDataTypeEnum.LongText;
                            break;

                        case FormFieldControlTypeCode.INTEGER_NUMBER_TEXTBOX:
                            ffi.Size = 0;
                            ffi.DataType = FormFieldDataTypeEnum.Integer;
                            break;

                        case FormFieldControlTypeCode.DECIMAL_NUMBER_TEXTBOX:
                            ffi.Size = 0;
                            ffi.DataType = FormFieldDataTypeEnum.Decimal;
                            break;

                        case FormFieldControlTypeCode.CHECKBOX:
                            ffi.Size = 0;
                            ffi.DataType = FormFieldDataTypeEnum.Boolean;
                            break;

                        case FormFieldControlTypeCode.CALENDAR:
                            ffi.Size = 0;
                            ffi.DataType = FormFieldDataTypeEnum.DateTime;
                            break;

                        case FormFieldControlTypeCode.UPLOAD:
                            if ((Mode == FieldEditorModeEnum.BizFormDefinition) || (Mode == FieldEditorModeEnum.SystemTable))
                            {
                                // allow to save <guid>.<extension>
                                ffi.Size = 500;
                                ffi.DataType = FormFieldDataTypeEnum.Text;
                            }
                            else
                            {
                                // allow to save just <guid>
                                ffi.Size = 0;
                                ffi.DataType = FormFieldDataTypeEnum.File;
                            }
                            break;

                        case FormFieldControlTypeCode.MULTIPLECHOICE:
                        case FormFieldControlTypeCode.DROPDOWNLIST:
                        case FormFieldControlTypeCode.RADIOBUTTONS:
                        case FormFieldControlTypeCode.FILEPATH:
                        case FormFieldControlTypeCode.IMAGEPATH:
                            ffi.Size = 500;
                            ffi.DataType = FormFieldDataTypeEnum.Text;
                            break;

                        default:
                            if (selectedType.StartsWith(controlPrefix))
                            {
                                // new field -> use default datatype and size
                                if (ffiOriginal == null)
                                {
                                    // get control data type
                                    string[] controlDefaultDataType = GetUserControlDefaultDataType(controlName);
                                    ffi.DataType = FormHelper.GetFormFieldDataType(controlDefaultDataType[0]);

                                    if (ffi.DataType == FormFieldDataTypeEnum.Text)
                                    {
                                        ffi.Size = Convert.ToInt32(txtSimpleTextBoxMaxLength.Text);
                                    }
                                    else
                                    {
                                        ffi.Size = 0;
                                    }
                                }
                            }
                            break;
                    }
                }
                // existing field -> set advanced settings to original settings
                else
                {
                    if (selectedType == FormFieldControlTypeCode.TEXTBOX)
                    {
                        ffi.Size = Convert.ToInt32(txtSimpleTextBoxMaxLength.Text);
                    }
                    else if (selectedType.StartsWith(controlPrefix))
                    {
                        // get control data type
                        string[] controlDefaultDataType = GetUserControlDefaultDataType(controlName);
                        ffi.DataType = FormHelper.GetFormFieldDataType(controlDefaultDataType[0]);

                        if (ffi.DataType == FormFieldDataTypeEnum.Text)
                        {
                            ffi.Size = Convert.ToInt32(txtSimpleTextBoxMaxLength.Text);
                        }
                        else
                        {
                            ffi.Size = 0;
                        }                       
                    }
                    else
                    {
                        ffi.Size = ffiOriginal.Size;
                    }
                    
                    ffi.DataType = ffiOriginal.DataType;

                    // part of database section                    
                    ffi.Description = ffiOriginal.Description;

                    // validation section
                    ffi.RegularExpression = ffiOriginal.RegularExpression;
                    ffi.MinStringLength = ffiOriginal.MinStringLength;
                    ffi.MaxStringLength = ffiOriginal.MaxStringLength;
                    ffi.MinNumericValue = ffiOriginal.MinNumericValue;
                    ffi.MaxNumericValue = ffiOriginal.MaxNumericValue;
                    ffi.MinDateTimeValue = ffiOriginal.MinDateTimeValue;
                    ffi.MaxDateTimeValue = ffiOriginal.MaxDateTimeValue;
                    ffi.FileExtensions = ffiOriginal.FileExtensions;
                    ffi.ValidationErrorMessage = ffiOriginal.ValidationErrorMessage;

                    // design section
                    ffi.CaptionStyle = ffiOriginal.CaptionStyle;
                    ffi.InputControlStyle = ffiOriginal.InputControlStyle;
                }
                break;
        }

        return ffi;
    }

    /// <summary>
    /// Fill attribute types list
    /// </summary>
    private void LoadAttributeTypes()
    {
        if (this.lstAttributeType.Items.Count > 0)
        {
            return;
        }

        this.lstAttributeType.DataSource = FormHelper.GetFieldTypes();
        this.lstAttributeType.DataBind();
    }


    /// <summary>
    /// Show default value control according attribute type
    /// </summary>
    protected void ShowDefaultValue()
    {

        chkDefaultValue.Visible = false;
        txtDefaultValue.Visible = false;
        datetimeDefaultValue.Visible = false;
        string fieldType = "";
      
        switch (SelectedMode)
        {
            case FieldEditorSelectedModeEnum.Simplified:

                fieldType = lstFieldType.SelectedValue.ToLower();

                switch (fieldType)
                {
                    // CALENDAR
                    case FormFieldControlTypeCode.CALENDAR:
                        datetimeDefaultValue.Visible = true;
                        break;

                    // CHECKBOX
                    case FormFieldControlTypeCode.CHECKBOX:
                        chkDefaultValue.Visible = true;
                        break;
                    
                    default:

                       // USER CONTROL                       
                        if (fieldType.StartsWith(controlPrefix))
                        {

                            string controlName = fieldType.Substring(controlPrefix.Length);
                            string [] defaultControlDataType = GetUserControlDefaultDataType(controlName);
                            string controlDataType = "";

                            // display default value according to default datatype
                            if (IsNewItemEdited)
                            {
                                controlDataType = defaultControlDataType[0];
                            }
                            // display default value according to saved datatype
                            else
                            {
                                controlDataType = FieldDataType;
                            }

                            switch (controlDataType)
                            {
                                case FormFieldDataTypeCode.DATETIME:
                                    datetimeDefaultValue.Visible = true;
                                    break;

                                case FormFieldDataTypeCode.BOOLEAN:
                                    chkDefaultValue.Visible = true;
                                    break;

                                default:
                                    txtDefaultValue.Visible = true;
                                    break;
                            }

                        }
                        // OTHER CONTROLS
                        else
                        {
                            txtDefaultValue.Visible = true;
                        }
                        break;
                }
                break;

            case FieldEditorSelectedModeEnum.Advanced:

                switch (lstAttributeType.SelectedValue.ToLower())
                {
                    case FormFieldDataTypeCode.DATETIME:
                        datetimeDefaultValue.Visible = true;
                        break;

                    case FormFieldDataTypeCode.BOOLEAN:
                        chkDefaultValue.Visible = true;
                        break;

                    default:
                        txtDefaultValue.Visible = true;
                        break;
                }
                break;
        }


    }

    /// <summary>
    /// Set default value according to attribute type
    /// </summary>
    protected void SetDefaultValue()
    {
        string fieldType = lstFieldType.SelectedValue.ToLower();

        if (ffi != null)
        {
            switch (SelectedMode)
            {
                // simplified mode
                case FieldEditorSelectedModeEnum.Simplified:                    
                                        
                    // Calendar or UserControl with boolean datatype
                    if ((fieldType == FormFieldControlTypeCode.CALENDAR) ||
                        ((fieldType.StartsWith(controlPrefix)) && (FieldDataType == FormFieldDataTypeCode.DATETIME)))
                    {
                        if (ffi.DefaultValue == "")
                        {
                            datetimeDefaultValue.DateTimeTextBox.Text = "";
                        }
                        else
                        {
                            datetimeDefaultValue.SelectedDateTime = Convert.ToDateTime(ffi.DefaultValue);
                        }
                    }
                    // Checkbox or UserControl with boolean datatype
                    else if ((fieldType == FormFieldControlTypeCode.CHECKBOX) ||
                        ((fieldType.StartsWith(controlPrefix)) && (FieldDataType == FormFieldDataTypeCode.BOOLEAN)))
                    {
                        chkDefaultValue.Checked = (ffi.DefaultValue.ToLower() == "true") ? true : false;
                    }
                    else
                    {
                        txtDefaultValue.Text = ffi.DefaultValue;
                    }
                    break;

                // advanced mode
                case FieldEditorSelectedModeEnum.Advanced:

                    switch (lstAttributeType.SelectedValue.ToLower())
                    {
                        case FormFieldDataTypeCode.DATETIME:
                            if (ffi.DefaultValue == "")
                            {
                                datetimeDefaultValue.DateTimeTextBox.Text = "";
                            }
                            else
                            {
                                datetimeDefaultValue.SelectedDateTime = Convert.ToDateTime(ffi.DefaultValue);
                            }
                            break;

                        case FormFieldDataTypeCode.BOOLEAN:
                            chkDefaultValue.Checked = (ffi.DefaultValue.ToLower() == "true") ? true : false;
                            break;

                        default:
                            txtDefaultValue.Text = ffi.DefaultValue;
                            break;
                    }
                    break;
            }

        }
        else
        {
            txtDefaultValue.Text = "";
            chkDefaultValue.Checked = false;
            datetimeDefaultValue.SelectedDateTime = DateTimePicker.NOT_SELECTED;
        }
    }


    /// <summary>
    /// Returns default value according to attribute type
    /// </summary>
    /// <returns></returns>
    protected string GetDefaultValue()
    {
        string defaultValue = "";
        System.Globalization.CultureInfo ciEnUs = new System.Globalization.CultureInfo("en-us"); // culture info for datetime conversion to string
        
        switch (SelectedMode)
        {
            // simplified mode
            case FieldEditorSelectedModeEnum.Simplified:

                string controlDataType = "";
                string fieldType = lstFieldType.SelectedValue.ToLower();

             /*   if (fieldType.StartsWith(controlPrefix))
                {
                    string controlName = fieldType.Substring(controlPrefix.Length);
                    string[] defaultControlDataType = GetUserControlDefaultDataType(controlName);
                    
                    // display default value according to default datatype
                    if (IsNewItemEdited)
                    {
                        controlDataType = defaultControlDataType[0];
                    }
                    // display default value according to saved datatype
                    else
                    {
                        controlDataType = FieldDataType;
                    }
                }*/

                // Calendar or UserControl with boolean datatype
                if ((fieldType == FormFieldControlTypeCode.CALENDAR) ||
                    ((fieldType.StartsWith(controlPrefix)) && (controlDataType == FormFieldDataTypeCode.DATETIME)))
                {
                    if (datetimeDefaultValue.SelectedDateTime == DateTimePicker.NOT_SELECTED)
                    {
                        defaultValue = "";
                    }
                    else
                    {
                        defaultValue = datetimeDefaultValue.SelectedDateTime.ToString(ciEnUs.DateTimeFormat); ;
                    }
                }
                // Checkbox or UserControl with boolean datatype
                else if ((fieldType == FormFieldControlTypeCode.CHECKBOX) ||
                    ((fieldType.StartsWith(controlPrefix)) && (controlDataType == FormFieldDataTypeCode.BOOLEAN)))
                {
                    defaultValue = Convert.ToString(chkDefaultValue.Checked).ToLower();
                }
                else
                {
                    defaultValue = txtDefaultValue.Text.Trim();
                }

                break;

            // advanced mode
            case FieldEditorSelectedModeEnum.Advanced:

                switch (lstAttributeType.SelectedValue.ToLower())
                {
                    case FormFieldDataTypeCode.DATETIME:
                        if (datetimeDefaultValue.SelectedDateTime == DateTimePicker.NOT_SELECTED)
                        {
                            defaultValue = "";
                        }
                        else
                        {
                            defaultValue = datetimeDefaultValue.SelectedDateTime.ToString(ciEnUs.DateTimeFormat); ;
                        }
                        break;

                    case FormFieldDataTypeCode.BOOLEAN:
                        defaultValue = Convert.ToString(chkDefaultValue.Checked).ToLower();
                        break;

                    default:
                        defaultValue = txtDefaultValue.Text.Trim();
                        break;
                }
                break;
        }

        return defaultValue;
    }


    /// <summary>
    /// Display or hide tabs.
    /// </summary>
    protected void DisplayOrHideTabs()
    {
        if (EnableSimplifiedMode)
        {
            // display tabs
            pnlTabsHolder.Visible = true;

            InitializeTabs();
            HighlightSelectedTab();      
        }
        else
        {
            // hide tabs
            pnlTabsHolder.Visible = false;
        }
    }


    /// <summary>
    // Initializes tabs.
    /// </summary>
    protected void InitializeTabs()
    {
        if (EnableSimplifiedMode)
        {
            // tab initializations
            BasicTabControlMenu.Tabs = new string[2, 4];
            BasicTabControlMenu.Tabs[0, 0] = ResHelper.GetString("fieldeditor.tabs.simplified");
            BasicTabControlMenu.Tabs[1, 0] = ResHelper.GetString("fieldeditor.tabs.advanced");
            
            // set no url redirect to both tabs
            BasicTabControlMenu.Tabs[0, 2] = "";
            BasicTabControlMenu.Tabs[1, 2] = "";

            if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
            {
                // add javascript postback to enable mode changing
                BasicTabControlMenu.Tabs[0, 1] = "javascript: " + Page.ClientScript.GetPostBackEventReference(btnSimplified, "");
                BasicTabControlMenu.Tabs[1, 1] = "javascript: " + Page.ClientScript.GetPostBackEventReference(btnAdvanced, "");
            }            
        }

    }


    /// <summary>
    /// Displays selected tab content.
    /// </summary>
    protected void DisplaySelectedTabContent()
    {
        switch (SelectedMode)
        {
            case FieldEditorSelectedModeEnum.Simplified:
                pnlAdvancedAttributeEdit.Visible = false;
                pnlSimpleAttributeEdit.Visible = true;                
                break;

            case FieldEditorSelectedModeEnum.Advanced:
                pnlAdvancedAttributeEdit.Visible = true;
                pnlSimpleAttributeEdit.Visible = false;
                break;
        } 
    }


    /// <summary>
    /// Highlights selected tab.
    /// </summary>
    protected void HighlightSelectedTab()
    {
        switch (SelectedMode)
        {
            case FieldEditorSelectedModeEnum.Simplified:
                BasicTabControlMenu.SelectedTab = 0;
                break;

            case FieldEditorSelectedModeEnum.Advanced:
                BasicTabControlMenu.SelectedTab = 1;
                break;
        }  
    }


    /// <summary>
    /// Called by javascript postback when tab is clicked 
    /// </summary>
    protected void btnSimplified_Command(object sender, CommandEventArgs e)
    {
        SelectedMode = FieldEditorSelectedModeEnum.Simplified;
        Reload(this.lstAttributes.SelectedValue);
    }


    /// <summary>
    /// Called by javascript postback when tab is clicked
    /// </summary>
    protected void btnAdvanced_Command(object sender, CommandEventArgs e)
    {
        SelectedMode = FieldEditorSelectedModeEnum.Advanced;
        Reload(this.lstAttributes.SelectedValue);
    }


    /// <summary>
    /// Returns default selected mode.
    /// </summary>
    /// <returns></returns>
    private FieldEditorSelectedModeEnum GetDefaultSelectedMode()
    {
        
        if (EnableSimplifiedMode)
        {
            return FieldEditorSelectedModeEnum.Simplified;
        }
        else
        {
            return FieldEditorSelectedModeEnum.Advanced;
        }
    }


    /// <summary>
    /// Sets universal controls to controls of the selected mode.
    /// </summary>
    private void SetUniversalControls()
    {
        if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
        {
            switch (SelectedMode)
            {
                case FieldEditorSelectedModeEnum.Simplified:

                    #region "Set universal controls to simplified mode controls"

                    // basic controls 
                    //--------------------------------------------------------

                    // attribute name
                    lblAttributeName = lblSimpleColumnName;
                    txtAttributeName = txtSimpleColumnName;

                    // allow empty 
                    lblAttributeAllowEmpty = lblSimpleAllowEmpty;
                    chkAllowEmpty = chkSimpleAllowEmpty;

                    // default value types
                    lblDefaultValue = lblSimpleDefaultValue;
                    txtDefaultValue = txtSimpleDefault;
                    chkDefaultValue = chkSimpleDefault;
                    datetimeDefaultValue = datetimeSimpleDefault;

                    // field caption
                    lblFieldCaption = lblSimpleFieldCaption;
                    txtFieldCaption = txtSimpleFieldCaption;

                    // field type
                    lblFieldType = lblSimpleFieldType;
                    lstFieldType = lstSimpleFieldType;

                    // additional controls 
                    //--------------------------------------------------------

                    // panel with text area settings
                    pnlTextAreaSettings = pnlSimpleTextAreaSettings;

                    // text area columns
                    lblTextAreaCols = lblSimpleTextAreaCols;
                    txtTextAreaCols = txtSimpleTextAreaCols;

                    // text area rows
                    lblTextAreaRows = lblSimpleTextAreaRows;
                    txtTextAreaRows = txtSimpleTextAreaRows;

                    // text area size
                    lblTextAreaSize = lblSimpleTextAreaSize;
                    txtTextAreaSize = txtSimpleTextAreaSize;

                    //----------------------------------------

                    // panel with html area settings
                    pnlHTMLAreaSettings = pnlSimpleHTMLAreaSettings;

                    // html area width
                    lblHTMLAreaWidth = lblSimpleHTMLAreaWidth;
                    txtHTMLAreaWidth = txtSimpleHTMLAreaWidth;

                    // html area height
                    lblHTMLAreaHeight = lblSimpleHTMLAreaHeight;
                    txtHTMLAreaHeight = txtSimpleHTMLAreaHeight;

                    // html area size
                    lblHTMLAreaSize = lblSimpleHTMLAreaSize;
                    txtHTMLAreaSize = txtSimpleHTMLAreaSize;

                    //----------------------------------------

                    // panel with drop down list settings
                    pnlDropDownListSettings = pnlSimpleDropDownListSettings;

                    // drop down list options radiobutton
                    radDropDownListOptions = radSimpleDropDownListOptions;

                    // drop down list sql radiobutton 
                    radDropDownListSQL = radSimpleDropDownListSQL;

                    // drop down list options textbox    
                    txtDropDownListOptions = txtSimpleDropDownListOptions;

                    //----------------------------------------

                    // panel with calendar settings
                    pnlCalendarSettings = pnlSimpleCalendarSettings;

                    // edit time checkbox
                    chkCalendarSettingsEditTime = chkSimpleCalendarSettingsEditTime;

                    //----------------------------------------

                    // panel with multiple choice settings
                    pnlMultipleChoice = pnlSimpleMultipleChoice;

                    // multiple choice options radiobutton
                    radMultipleChoiceOptions = radSimpleMultipleChoiceOptions;

                    // multiple choice sql radiobutton 
                    radMultipleChoiceSQL = radSimpleMultipleChoiceSQL;

                    // multiple choice options textbox    
                    txtMultipleChoiceOptions = txtSimpleMultipleChoiceOptions;

                    //----------------------------------------

                    // panel with image selection settings
                    pnlImageSelection = pnlSimpleImageSelection;

                    lblImageWidth = lblSimpleImageWidth;
                    lblImageHeight = lblSimpleImageHeight;
                    txtImageWidth = txtSimpleImageWidth;
                    txtImageHeight = txtSimpleImageHeight;

                    //----------------------------------------

                    // panel with public field option
                    pnlPublicField = pnlSimplePublicField;
                    lblPublicField = lblSimplePublicField;
                    chkPublicField = chkSimplePublicField;

                    //----------------------------------------
                    // panel with all editable controls
                    pnlAttributeEdit = pnlSimpleAttributeEdit;

                    #endregion


                    // enable or disable some controls in simplified mode while editing existing field
                    if (SelectedItemType == FieldEditorSelectedItemEnum.Field)
                    {
                        if (IsNewItemEdited)
                        {
                            // editing new item
                            txtSimpleColumnName.Enabled = true;
                            txtSimpleColumnName.ReadOnly = false;
                            lstSimpleFieldType.Enabled = true;
                        }
                        else
                        {
                            // editing existing item
                            txtSimpleColumnName.Enabled = false;
                            txtSimpleColumnName.ReadOnly = true;
                            lstSimpleFieldType.Enabled = false;
                        }
                    }

                    this.lblAttributeName.Text = ResHelper.GetString("TemplateDesigner.ColumName");
                    this.lblDefaultValue.Text = ResHelper.GetString("TemplateDesigner.ColumnDefaultValue");
                    this.lblSimpleTextBoxMaxLength.Text = ResHelper.GetString("TemplateDesigner.TextBoxLength");

                    break;

                case FieldEditorSelectedModeEnum.Advanced:

                    #region "Set universal controls to advanced mode controls"

                    // basic controls 
                    //--------------------------------------------------------

                    // attribute name
                    lblAttributeName = lblAdvancedAttributeName;
                    txtAttributeName = txtAdvancedAttributeName;

                    // allow empty 
                    lblAttributeAllowEmpty = lblAdvancedAllowEmpty;
                    chkAllowEmpty = chkAdvancedAllowEmpty;

                    // default value types
                    lblDefaultValue = lblAdvancedDefaultValue;
                    txtDefaultValue = txtAdvancedDefaultValue;
                    chkDefaultValue = chkAdvancedDefaultValue;
                    datetimeDefaultValue = datetimeAdvancedDefatulValue;

                    // field caption
                    lblFieldCaption = lblAdvancedFieldCaption;
                    txtFieldCaption = txtAdvancedFieldCaption;

                    // field type
                    lblFieldType = lblAdvancedFieldType;
                    lstFieldType = lstAdvancedFieldType;

                    // additional controls 
                    //--------------------------------------------------------

                    // panel with text area settings
                    pnlTextAreaSettings = pnlAdvancedTextAreaSettings;

                    // text area columns
                    lblTextAreaCols = lblAdvancedTextAreaCols;
                    txtTextAreaCols = txtAdvancedTextAreaCols;

                    // text area rows
                    lblTextAreaRows = lblAdvancedTextAreaRows;
                    txtTextAreaRows = txtAdvancedTextAreaRows;

                    // text area size
                    lblTextAreaSize = lblAdvancedTextAreaSize;
                    txtTextAreaSize = txtAdvancedTextAreaSize;

                    //----------------------------------------

                    // panel with html area settings
                    pnlHTMLAreaSettings = pnlAdvancedHTMLAreaSettings;

                    // html area width
                    lblHTMLAreaWidth = lblAdvancedHTMLAreaWidth;
                    txtHTMLAreaWidth = txtAdvancedHTMLAreaWidth;

                    // html area height
                    lblHTMLAreaHeight = lblAdvancedHTMLAreaHeight;
                    txtHTMLAreaHeight = txtAdvancedHTMLAreaHeight;

                    // html area size
                    lblHTMLAreaSize = lblAdvancedHTMLAreaSize;
                    txtHTMLAreaSize = txtAdvancedHTMLAreaSize;

                    //----------------------------------------

                    // panel with drop down list settings
                    pnlDropDownListSettings = pnlAdvancedDropDownListSettings;

                    // drop down list options radiobutton
                    radDropDownListOptions = radAdvancedDropDownListOptions;

                    // drop down list sql radiobutton 
                    radDropDownListSQL = radAdvancedDropDownListSQL;

                    // drop down list options textbox    
                    txtDropDownListOptions = txtAdvancedDropDownListOptions;

                    //----------------------------------------

                    // panel with calendar settings
                    pnlCalendarSettings = pnlAdvancedCalendarSettings;

                    // edit time checkbox
                    chkCalendarSettingsEditTime = chkAdvancedCalendarSettingsEditTime;

                    //----------------------------------------

                    // panel with multiple choice settings
                    pnlMultipleChoice = pnlAdvancedMultipleChoice;

                    // multiple choice options radiobutton
                    radMultipleChoiceOptions = radAdvancedMultipleChoiceOptions;

                    // multiple choice sql radiobutton 
                    radMultipleChoiceSQL = radAdvancedMultipleChoiceSQL;

                    // multiple choice options textbox    
                    txtMultipleChoiceOptions = txtAdvancedMultipleChoiceOptions;

                    //----------------------------------------

                    // panel with image selection settings
                    pnlImageSelection = pnlAdvancedImageSelection;

                    lblImageWidth = lblAdvancedImageWidth;
                    lblImageHeight = lblAdvancedImageHeight;
                    txtImageWidth = txtAdvancedImageWidth;
                    txtImageHeight = txtAdvancedImageHeight;

                    //----------------------------------------

                    // panel with public field option
                    pnlPublicField = pnlAdvancedPublicField;
                    lblPublicField = lblAdvancedPublicField;
                    chkPublicField = chkAdvancedPublicField;

                    //----------------------------------------

                    // panel with all editable controls
                    pnlAttributeEdit = pnlAdvancedAttributeEdit;

                    #endregion

                    this.lblAttributeName.Text = ResHelper.GetString("TemplateDesigner.AttributeName");
                    this.lblDefaultValue.Text = ResHelper.GetString("TemplateDesigner.AttributeDefaultValue");

                    break;
            }

            // public field option initialization    
            if (Mode == FieldEditorModeEnum.BizFormDefinition)
            {
                pnlPublicField.Visible = true;
                lblPublicField.Text = ResHelper.GetString("TemplateDesigner.PublicField");
            }
            else
            {
                pnlPublicField.Visible = false;
            }            

            // universal control initialization        
            this.lblFieldCaption.Text = ResHelper.GetString("TemplateDesigner.FieldCaption");
            this.lblFieldType.Text = ResHelper.GetString("TemplateDesigner.FieldType");
            this.lblAttributeAllowEmpty.Text = ResHelper.GetString("TemplateDesigner.AttributeAllowEmpty");
            this.lblTextAreaCols.Text = ResHelper.GetString("TemplateDesigner.TextAreaCols");
            this.lblTextAreaRows.Text = ResHelper.GetString("TemplateDesigner.TextAreaRows");
            this.lblTextAreaSize.Text = ResHelper.GetString("TemplateDesigner.TextAreaSize");
            this.lblHTMLAreaHeight.Text = ResHelper.GetString("TemplateDesigner.HTMLAreaHeight");
            this.lblHTMLAreaWidth.Text = ResHelper.GetString("TemplateDesigner.HTMLAreaWidth");
            this.lblHTMLAreaSize.Text = ResHelper.GetString("TemplateDesigner.HTMLAreaSize");
            this.radDropDownListOptions.Text = ResHelper.GetString("TemplateDesigner.DropDownListOptions");
            this.radDropDownListSQL.Text = ResHelper.GetString("TemplateDesigner.DropDownListSQL");
            this.radMultipleChoiceOptions.Text = ResHelper.GetString("TemplateDesigner.MultipleChoiceOptions");
            this.radMultipleChoiceSQL.Text = ResHelper.GetString("TemplateDesigner.MultipleChoiceSQL");
            this.chkCalendarSettingsEditTime.Text = ResHelper.GetString("TemplateDesigner.CalendarSettingsEditTime");
            this.lblImageHeight.Text = ResHelper.GetString("TemplateDesigner.ImageHeight");
            this.lblImageWidth.Text = ResHelper.GetString("TemplateDesigner.ImageWidth");
            this.datetimeDefaultValue.SupportFolder = dtPickerFolder;
        }

    }


    /// <summary>
    /// Sets Field editor content and footer css class according to selected mode.
    /// </summary>
    protected void SetCssClasses()
    {
        if (EnableSimplifiedMode)
        {

            switch (SelectedMode)
            {
                case FieldEditorSelectedModeEnum.Simplified:
                    pnlFieldEditorContentCss.CssClass = "FieldEditorContentSimplifiedMode";
                    break;


                case FieldEditorSelectedModeEnum.Advanced:
                    pnlFieldEditorContentCss.CssClass = "FieldEditorContentAdvancedMode";
                    break;

                default:
                    pnlFieldEditorContentCss.CssClass = "FieldEditorContentAdvancedMode";
                    break;
            }
        }
        else
        {
            // default FieldEditorContent has no additonal css styles
            pnlFieldEditorContentCss.CssClass = "";
        }
    }


    /// <summary>
    /// Hides all editing panels
    /// </summary>
    protected void HideAllPanels()
    {        
        pnlCategoryEdit.Visible = false;
        pnlAdvancedAttributeEdit.Visible = false;
        pnlSimpleAttributeEdit.Visible = false;
    }


    /// <summary>
    /// Adds new form item (field or category) to the form definition.
    /// </summary>
    /// <param name="formItem">Form item to add.</param>
    protected void InsertFormItem(object formItem)
    {
        // set new item preffix
        string newItemPreffix = (formItem is FormFieldInfo) ? newFieldPreffix : newCategPreffix;

        ListItem newItem = this.lstAttributes.Items.FindByValue(newItemPreffix);
        int newItemIndex = this.lstAttributes.Items.IndexOf(newItem);

        if ((newItemIndex > 0) && (this.lstAttributes.Items[newItemIndex - 1] != null))
        {
            string predecessorValue = this.lstAttributes.Items[newItemIndex - 1].Value;
            string predecessorName = "";
            FormItemTypeEnum predecessorType = 0;

            // set predecessor item type
            if (predecessorValue.StartsWith(categPreffix))
            {
                predecessorType = FormItemTypeEnum.Category;
            }
            else if (predecessorValue.StartsWith(fieldPreffix))
            {
                predecessorType = FormItemTypeEnum.Field;
            }

            //set predecessor name
            predecessorName = predecessorValue.Substring(preffixLength);

            // add new item at the specified position
            this.fi.AddFormItem(formItem, predecessorName, predecessorType);
        }
        else
        {
            if (formItem is FormFieldInfo)
            {
                // add new field at the end of the collection
                fi.AddFormField((FormFieldInfo)formItem);
            }
            else if (formItem is FormCategoryInfo)
            {
                // add new category at the end of the collection
                fi.AddFormCategory((FormCategoryInfo)formItem);
            }
        }
    }


    /// <summary>
    /// Returns default datatype of the specified user control.
    /// </summary>
    /// <param name="controlName">User control name.</param>
    /// <returns></returns>
    protected string[] GetUserControlDefaultDataType(string controlName)
    {
        string[] defaultDataType = new string[2]; ;

        FormUserControlInfo control = FormUserControlInfoProvider.GetFormUserControlInfo(controlName);
        if (control != null)
        {            
            defaultDataType[0] = control.UserControlDefaultDataType.ToLower();
            defaultDataType[1] = Convert.ToString(control.UserControlDefaultDataTypeSize);
        }
        else
        {
            defaultDataType[0] = FormFieldDataTypeCode.TEXT;
            defaultDataType[1] = "500";
        }

        return defaultDataType;
    }
}
