using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.CMSSiteMapProvider;
using CMS.TreeEngine;
using CMS.SettingsProvider;
using CMS.CMSHelper;

public partial class CMSDesk_Content_ContentTree : System.Web.UI.UserControl
{
    private string mNodeTextTemplate = "##ICON####NODENAME##";
    private string mSelectedNodeTextTemplate = "##ICON####NODENAME##";
    private int mNodeId = 0;
    private int mExpandNodeId = 0;
    //private string mSiteName = null;
    private string mPath = "/";
    private string mBasePath = null;
    protected TreeSiteMapProvider mMapProvider = null;
    private int mMaxTreeNodes = 0;
    private string mMaxTreeNodeText = null;

    /// <summary>
    /// Template of the node text, use {0} to insert the original node text, {1} to insert the Node ID
    /// </summary>
    public string NodeTextTemplate
    {
        get
        {
            return mNodeTextTemplate;
        }
        set
        {
            mNodeTextTemplate = value;
        }
    }


    /// <summary>
    /// Template of the SelectedNode text, use {0} to insert the original SelectedNode text, {1} to insert the SelectedNode ID
    /// </summary>
    public string SelectedNodeTextTemplate
    {
        get
        {
            return mSelectedNodeTextTemplate;
        }
        set
        {
            mSelectedNodeTextTemplate = value;
        }
    }


    /// <summary>
    /// Gets or sets the current node ID
    /// </summary>
    public int NodeID
    {
        get
        {
            return mNodeId;
        }
        set
        {
            mNodeId = value;
        }
    }


    /// <summary>
    /// Gets or sets the node ID to expand
    /// </summary>
    public int ExpandNodeID
    {
        get
        {
            return mExpandNodeId;
        }
        set
        {
            mExpandNodeId = value;
        }
    }


    /// <summary>
    /// Site name to display
    /// </summary>
    public string SiteName
    {
        get
        {
            return ValidationHelper.GetString(ViewState["SiteName"], "");
            //return mSiteName;
        }
        set
        {
            ViewState["SiteName"] = value;
            //mSiteName = value;
        }
    }


    /// <summary>
    /// Nodes selecting path
    /// </summary>
    public string Path
    {
        get
        {
            return mPath;
        }
        set
        {
            mPath = value;
        }
    }


    /// <summary>
    /// Sitemap provider that the tree uses
    /// </summary>
    public TreeSiteMapProvider MapProvider
    {
        get
        {
            if (mMapProvider == null)
            {
                mMapProvider = new TreeSiteMapProvider();
            }
            return mMapProvider;
        }
    }


    /// <summary>
    /// Maximum number of tree nodes displayed within the tree
    /// </summary>
    public int MaxTreeNodes
    {
        get
        {
            if (mMaxTreeNodes <= 0)
            {
                mMaxTreeNodes = SettingsKeyProvider.GetIntValue(CMSContext.CurrentSite.SiteName + ".CMSMaxTreeNodes");
            }
            return mMaxTreeNodes;
        }
        set
        {
            mMaxTreeNodes = value;
        }
    }


    /// <summary>
    /// Text to appear within the latest node when max tree nodes applied
    /// </summary>
    public string MaxTreeNodeText
    {
        get
        {
            if (mMaxTreeNodeText == null)
            {
                mMaxTreeNodeText = ResHelper.GetString("ContentTree.SeeListing");
            }
            return mMaxTreeNodeText;
        }
        set
        {
            mMaxTreeNodeText = value;
        }
    }


    /// <summary>
    /// Nodes to expand
    /// </summary>
    protected ArrayList expandNodes = new ArrayList();


    protected void Page_Load(object sender, EventArgs e)
    {
        mBasePath = Request.Url.LocalPath;

        this.treeElem.LineImagesFolder = "~/App_Themes/" + this.Page.Theme + "/Images/Tree";
        if (CultureHelper.IsUICultureRTL())
        {
            this.treeElem.LineImagesFolder += "/RTL";
        }
        this.treeElem.ImageSet = TreeViewImageSet.Custom;
        this.treeElem.ExpandImageToolTip = ResHelper.GetString("ContentTree.Expand");
        this.treeElem.CollapseImageToolTip = ResHelper.GetString("ContentTree.Collapse");

        if (this.SiteName == "")
        {
            this.SiteName = CMSContext.CurrentSite.SiteName;
        }

        MapProvider.CultureCode = TreeProvider.ALL_CULTURES;
        MapProvider.SiteName = this.SiteName;
        MapProvider.BindNodeData = true;
        MapProvider.OrderBy = "NodeOrder ASC, NodeName ASC, NodeAlias ASC";
        MapProvider.SelectOnlyPublished = false;
        MapProvider.CombineWithDefaultCulture = true;
        MapProvider.Path = this.Path;
        MapProvider.MaxRelativeLevel = 0;
        MapProvider.MaxTreeNodes = this.MaxTreeNodes + 1;
        
        if (!this.Page.IsCallback)
        {
            ReloadData();
        }
    }


    public void ReloadData()
    {
        MapProvider.ReloadData();

        // Expand current node parent
        if ( (this.ExpandNodeID <= 0) && (this.NodeID > 0))
        {
            TreeProvider tree = new TreeProvider();
            CMS.TreeEngine.TreeNode node = tree.SelectSingleNode(this.NodeID, TreeProvider.ALL_CULTURES);
            if (node != null)
            {
                this.ExpandNodeID = node.NodeParentID;
            }
        }

        // If expand node set, set the node to expand
        if (this.ExpandNodeID > 0)
        {
            // Get node list to expand
            expandNodes.Clear();
            //TreeSiteMapNode targetNode = MapProvider.GetNodeById(this.ExpandNodeID);
            TreeProvider tree = new TreeProvider();
            CMS.TreeEngine.TreeNode node = tree.SelectSingleNode(this.ExpandNodeID, TreeProvider.ALL_CULTURES);
            if (node != null)
            {
                TreeSiteMapNode targetNode = MapProvider.GetNodeByAliasPath(node.NodeAliasPath);
                if (targetNode != null)
                {
                    expandNodes.Add(targetNode.TreeNode.NodeID);
                    while (targetNode.ParentNode != null)
                    {
                        expandNodes.Add(((TreeSiteMapNode)targetNode.ParentNode).TreeNode.NodeID);
                        targetNode = (TreeSiteMapNode)targetNode.ParentNode;
                    }
                }
            }
        }

        this.treeElem.Nodes.Clear();
        // Add root node
        this.treeElem.Nodes.Add(CreateNode((TreeSiteMapNode)MapProvider.RootNode, 0));
    }


    protected void treeElem_TreeNodePopulate(object sender, TreeNodeEventArgs e)
    {
        e.Node.ChildNodes.Clear();
        e.Node.PopulateOnDemand = false;

        int nodeId = ValidationHelper.GetInteger(e.Node.Value, 0);

        // check explore tree permission for current node
        bool userHasExploreTreePermission = (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.ExploreTree) == AuthorizationResultEnum.Allowed) ? true : false;

        if (userHasExploreTreePermission)
        {
            //TreeSiteMapNode node = MapProvider.GetNodeById(nodeId);
            //SiteMapNodeCollection childNodes = node.ChildNodes;

            SiteMapNodeCollection childNodes = MapProvider.GetChildNodes(nodeId);
            int index = 0;
            foreach (TreeSiteMapNode childNode in childNodes)
            {
                if (childNode.TreeNode.NodeID != nodeId)
                {
                    System.Web.UI.WebControls.TreeNode newNode = CreateNode(childNode, index);
                    e.Node.ChildNodes.Add(newNode);
                    index++;
                }
            }
        }
        else
        {
            // add 'access denied node'
            System.Web.UI.WebControls.TreeNode tempNode = new System.Web.UI.WebControls.TreeNode();
            tempNode.Text = ResHelper.GetString("ContentTree.ExploreChildsDenied");
            tempNode.NavigateUrl = mBasePath + "#";
            e.Node.ChildNodes.Add(tempNode);
        }
    }


    protected System.Web.UI.WebControls.TreeNode CreateNode(TreeSiteMapNode sourceNode, int index)
    {
        System.Web.UI.WebControls.TreeNode newNode = new System.Web.UI.WebControls.TreeNode();

        if (index < this.MaxTreeNodes)
        {
            newNode.Value = sourceNode.TreeNode.NodeID.ToString();
            newNode.NavigateUrl = mBasePath + "#";

            string imageUrl = ResolveUrl("~/App_Themes/" + this.Page.Theme + "/Images/CMSDesk/Icons/" + sourceNode.TreeNode.NodeClassName.Replace(".", "_") + ".gif");
            string imageTag = "<img src=\"" + imageUrl + "\" alt=\"\" style=\"border:0px;vertical-align:middle;\" onclick=\"return false;\"/>";

            if ((this.NodeID > 0) && (sourceNode.TreeNode.NodeID == this.NodeID))
            {
                newNode.Text = SelectedNodeTextTemplate.Replace("##NODEID##", sourceNode.TreeNode.NodeID.ToString()).Replace("##NODENAMEJAVA##", sourceNode.TreeNode.NodeName.Replace("'", "\\'")).Replace("##NODENAME##", sourceNode.TreeNode.NodeName).Replace("##ICON##", imageTag); //.DocumentName
            }
            else
            {
                newNode.Text = NodeTextTemplate.Replace("##NODEID##", sourceNode.TreeNode.NodeID.ToString()).Replace("##NODENAMEJAVA##", sourceNode.TreeNode.NodeName.Replace("'", "\\'")).Replace("##NODENAME##", sourceNode.TreeNode.NodeName).Replace("##ICON##", imageTag); //.DocumentName
            }

            if ((sourceNode.TreeNode.NodeAliasPath != "/") && (!expandNodes.Contains(sourceNode.TreeNode.NodeID)))
            {
                newNode.Expanded = false;
            }
            else
            {
                newNode.Expanded = true;
            }

            if ((sourceNode.ChildNodes.Count > 0) || (!sourceNode.ChildNodesLoaded))
            {
                newNode.PopulateOnDemand = true;
            }
        }
        else
        {
            newNode.Value = sourceNode.TreeNode.NodeID.ToString();
            newNode.Text = this.MaxTreeNodeText.Replace("##PARENTNODEID##", sourceNode.TreeNode.NodeParentID.ToString());
            newNode.NavigateUrl = "#";
        }

        return newNode;
    }
}
