using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Net;
using System.Text.RegularExpressions;
using System.IO;

using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.TreeEngine;
using CMS.SiteProvider;
using CMS.PortalEngine;


public partial class CMSDesk_Content_MasterPage_PageEdit : CMSDeskPage
{
    protected string mSave = null;
    protected int nodeId = 0;
    protected TreeProvider tree = null;
    protected CMS.TreeEngine.TreeNode node = null;
    protected LayoutInfo li = null;

    protected string mHead = null;
    protected string mBeforeLayout = null;
    protected string mAfterLayout = null;
    protected string mBody = null;


    protected void Page_Load(object sender, EventArgs e)
    {
        try
        {

            // Title
            string title = CMSContext.CurrentTitle;
            if ((title != null) && (title != ""))
            {
                title = "<title>" + title + "</title>";
            }

            // Body class
            string bodyCss = CMSContext.CurrentBodyClass;
            if (bodyCss != null && bodyCss.Trim() != "")
            {
                bodyCss = "class=\"" + bodyCss + "\"";
            }
            else
            {
                bodyCss = "";
            }

            // metadata
            string meta = "<meta id=\"metaNoCache\" http-equiv=\"pragma\" content=\"no-cache\" />";

            string description = CMSContext.CurrentDescription;
            if (description != "")
            {
                meta += "<meta id=\"metaDescription\" name=\"description\" content=\"" + description + "\" />";

            }

            string keywords = CMSContext.CurrentKeyWords;
            if (keywords != "")
            {
                meta += "<meta id=\"metaKeyWords\" name=\"keywords\"  content=\"" + keywords + "\" />";
            }


            // Site style sheet
            string cssSiteSheet = "";
            CssStylesheetInfo cssInfo = CssStylesheetInfoProvider.GetCssStylesheetInfo(CMSContext.CurrentSite.SiteDefaultStylesheetID);
            if (cssInfo != null)
            {
                string path = HttpContext.Current.Request.ApplicationPath;
                if (path.EndsWith("/"))
                {
                    path = path.TrimEnd('/');
                }
                cssSiteSheet = "<link id=\"metaCssFile\" type=\"text/css\" rel=\"stylesheet\" href=\"" + path + "/CMSPages/GetCSS.aspx?stylesheetname=" + cssInfo.StylesheetName + "\" />";
            }


            // Theme css files
            string themeCssFiles = "";
            if (cssInfo != null)
            {
                try
                {
                    string directory = HttpContext.Current.Server.MapPath("~/") + "\\App_Themes\\" + cssInfo.StylesheetName + "\\";
                    if (Directory.Exists(directory))
                    {
                        foreach (string file in Directory.GetFiles(directory, "*.css"))
                        {
                            themeCssFiles += "<link href=\"App_Themes/" + cssInfo.StylesheetName + "/" + file.Remove(0, file.LastIndexOf('\\') + 1) + "\" type=\"text/css\" rel=\"stylesheet\" />";
                        }
                    }
                }
                catch { }
            }

            // Add values to page
            mHead = FormatHTML(HighlightHTML(title + meta + cssSiteSheet + themeCssFiles), 2);
            mBody = bodyCss;
        }
        catch
        {
            lblError.Visible = true;
            lblError.Text = ResHelper.GetString("MasterPage.PageEditErr");
        }

        string imagesUrl = ResolveUrl("~/App_Themes/" + this.Page.Theme + "/Images/CMSDesk/Content/");

        nodeId = ValidationHelper.GetInteger(Request.QueryString["NodeId"], 0);

        if ((nodeId > 0) && (CMSContext.CurrentPageInfo != null))
        {
            PageTemplateInfo templateInfo = CMSContext.CurrentPageInfo.PageTemplateInfo;
            tree = new TreeProvider(CMSContext.CurrentUser);
            node = tree.SelectSingleNode(nodeId);

            if (templateInfo != null)
            {
                li = LayoutInfoProvider.GetLayoutInfo(templateInfo.LayoutID);

                if (!IsPostBack)
                {

                    if ((li != null) && (li.LayoutCheckedOutByUserID == 0))
                    {
                        txtLayout.Text = li.LayoutCode;
                    }
                    else
                    {
                        if ((li != null) && (li.LayoutCheckedOutByUserID > 0))
                        {
                            lblChecked.Visible = true;
                            lblChecked.Text = "<br />" + AddSpaces(2) + ResHelper.GetString("MasterPage.LayoutCheckedOut");
                            txtLayout.Text = li.LayoutCode;
                            txtLayout.ReadOnly = true;
                        }
                        else
                        {
                            txtLayout.Text = ValidationHelper.GetString(templateInfo.PageTemplateLayout, "");
                            if (templateInfo.PageTemplateLayoutCheckedOutByUserID > 0)
                            {
                                lblChecked.Visible = true;
                                lblChecked.Text = "<br />" + AddSpaces(2) + ResHelper.GetString("MasterPage.LayoutCheckedOut");
                                txtLayout.ReadOnly = true;
                            }


                        }

                    }
                }
            }
            else
            {
                txtLayout.ReadOnly = true;
            }


            if ((node != null) && (!IsPostBack))
            {
                txtBodyCss.Text = node.NodeBodyCssClass;
                txtDocType.Text = node.NodeDocType;
                txtHeadTags.Text = node.NodeHeadTags;

            }

        }

        lblAfterDocType.Text = HighlightHTML("<html>") + "<br />" + AddSpaces(1) + HighlightHTML("<head>");
        lblAfterHeadTags.Text = AddSpaces(1) + HighlightHTML("</head>");
        lblAfterLayout.Text = AddSpaces(1) + HighlightHTML("</body>") + "<br />" + HighlightHTML("</html>");
        lblBodyEnd.Text = HighlightHTML(">");
        lblBodyStart.Text = AddSpaces(1) + HighlightHTML("<body " + HttpUtility.HtmlDecode(mBody));

        ltlScript.Text = ScriptHelper.GetScript("function SaveDocument(){\n " +
            Page.ClientScript.GetPostBackEventReference(this.btnSave, "") + "}");

    }


    /// <summary>
    /// Format HTML text
    /// </summary>
    /// <param name="inputHTML"></param>
    /// <param name="level"></param>
    /// <returns></returns>
    public string FormatHTML(string inputHTML, int level)
    {
        return AddSpaces(level) + inputHTML.Replace(HttpUtility.HtmlEncode(">"), HttpUtility.HtmlEncode(">") + "<br />" + AddSpaces(level));
    }


    /// <summary>
    /// Add spaces
    /// </summary>
    /// <param name="level"></param>
    /// <returns></returns>
    public string AddSpaces(int level)
    {

        string toReturn = "";
        for (int i = 0; i < level * 5; i++)
        {
            toReturn += "&nbsp;";
        }

        return toReturn;
    }

    /// <summary>
    /// Highlight HTML
    /// </summary>
    /// <param name="inputHtml"></param>
    /// <returns></returns>
    public string HighlightHTML(string inputHtml)
    {
        string tagspan = "<span style=\"color:blue;\">";
        string tagcontentspan = "<span style=\"color:#A31515;\">";
        string propertyspan = "<span style=\"color:blue;\">";
        string propertynamespan = "<span style=\"color:red;\">";
        string endspan = "</span>";


        // highlight for open and closed tags
        inputHtml = inputHtml.Replace("<", "%%tagsspan%%" + HttpUtility.HtmlEncode("<") + "%%endspan%%%%tagcontentspan%%");
        inputHtml = inputHtml.Replace("/>", "%%endspan%%%%tagsspan%%/" + HttpUtility.HtmlEncode(">") + "%%endspan%%");
        inputHtml = inputHtml.Replace(">", "%%endspan%%%%tagsspan%%" + HttpUtility.HtmlEncode(">") + "%%endspan%%");

        // tag properties
        inputHtml = Regex.Replace(inputHtml, "(\\s(\\w*|\\w*[\\S]*\\w*)\\s*(=))", " %%propertynamespan%%$2%%endspan%%$3", RegexOptions.IgnoreCase);
        inputHtml = Regex.Replace(inputHtml, "(=\"(\\w*|[^\"]*|\\s*)\")", "%%propertyspan%%$1%%endspan%%", RegexOptions.IgnoreCase);

        // Replace macros for tags
        inputHtml = inputHtml.Replace("%%tagcontentspan%%", tagcontentspan);
        inputHtml = inputHtml.Replace("%%propertynamespan%%", propertynamespan);
        inputHtml = inputHtml.Replace("%%propertyspan%%", propertyspan);
        inputHtml = inputHtml.Replace("%%tagsspan%%", tagspan);
        inputHtml = inputHtml.Replace("%%endspan%%", endspan);

        return inputHtml;
    }


    /// <summary>Returns the absolute URL representation for the given relative URL</summary>
    /// <param name="relativeUrl">Relative URL to convert to absolute</param>
    public string MakeAbsoluteUrl(string relativeUrl)
    {
        relativeUrl = ResolveUrl(relativeUrl);

        // Then add the base URL to the beginning of the URL where necessary
        if (relativeUrl.StartsWith("/"))
        {
            string webUrl = CMS.GlobalHelper.HTTPHelper.GetAbsoluteUri().Substring(0, CMS.GlobalHelper.HTTPHelper.GetAbsoluteUri().Length - HttpContext.Current.Request.Url.PathAndQuery.Length);
            if (!webUrl.EndsWith("/"))
            {
                webUrl += "/";
            }
            return webUrl + relativeUrl.TrimStart('/');
        }
        else
        {
            return relativeUrl;
        }
    }

    /// <summary>
    /// Return page Content
    /// </summary>
    /// <param name="URL"></param>
    /// <returns></returns>
    private string GetPageContent(string URL)
    {
        WebClient client = new WebClient();
        try
        {
            byte[] pageContent = client.DownloadData(URL);

            if (pageContent != null)
            {
                return System.Text.Encoding.UTF8.GetString(pageContent);
            }
        }
        catch
        {
        }

        return "";
    }

    protected void btnSave_Click(object sender, EventArgs e)
    {
        bool isCorrect = true;

        //Save layout
        if (li != null)
        {
            if (li.LayoutCheckedOutByUserID > 0)
            {
            }
            else
            {
                li.LayoutCode = txtLayout.Text;
                LayoutInfoProvider.SetLayoutInfo(li);
            }
        }
        else
        {
            PageTemplateInfo templateInfo = CMSContext.CurrentPageInfo.PageTemplateInfo;
            if (templateInfo != null)
            {
                if (templateInfo.PageTemplateLayoutCheckedOutByUserID > 0)
                {
                }
                else
                {
                    templateInfo.PageTemplateLayout = txtLayout.Text;
                    templateInfo.PageTemplateVersionGUID = Guid.NewGuid().ToString();
                    PageTemplateInfoProvider.SetPageTemplateInfo(templateInfo);
                }
            }
            else
            {
                isCorrect = false;
            }
        }

        if ((nodeId > 0) && (node != null) && (tree != null))
        {
            node.NodeBodyCssClass = txtBodyCss.Text;
            node.NodeDocType = txtDocType.Text;
            node.NodeHeadTags = txtHeadTags.Text;
            node.Update();
        }
        else
        {
            isCorrect = false;
        }


        if (isCorrect)
        {
            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
        }

    }
}
