using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.ExtendedControls;
using CMS.WorkflowEngine;
using CMS.PortalEngine;

public partial class CMSDesk_Content_Properties_metadata : CMSDeskPage
{
    int nodeId = 0;
    CMS.TreeEngine.TreeNode node = null;
    TreeProvider tree = null;
    protected bool hasModifyPermission = true;

    protected void Page_Load(object sender, EventArgs e)
    {
        this.PreRender +=new EventHandler(Page_PreRender);

        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);

        // Get the document
        tree = new TreeProvider(CMSContext.CurrentUser);
        node = DocumentHelper.GetDocument(nodeId, CMSContext.CurrentUser.PreferredCultureCode, tree);

        if (!IsPostBack)
        {
            ReloadData();
        }

        ltlScript.Text = "";

        lblPageDescription.Text = ResHelper.GetString("PageProperties.Decription");
        lblPageKeywords.Text = ResHelper.GetString("PageProperties.Keywords");
        lblPageTitle.Text = ResHelper.GetString("PageProperties.Title");
        lblCssStyle.Text = ResHelper.GetString("PageProperties.CssStyle");

        btnOK.Text = ResHelper.GetString("PageProperties.OK");
    }


    protected void Page_PreRender(object sender, EventArgs e)
    {
        if (hasModifyPermission)
        {
            this.pnlForm.Enabled = this.menuElem.AllowEdit;
        }
        this.lblWorkflowInfo.Text = this.menuElem.WorkflowInfo;
        this.lblWorkflowInfo.Visible = (this.lblWorkflowInfo.Text != "");
    }


    private void ReloadData()
    {
        if (node != null)
        {
            // Check read permissions
            if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                CMSDeskPage.RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
            }
            // Check modify permissions
            else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
            {
                hasModifyPermission = false;
                this.pnlForm.Enabled = false;
                this.lblWorkflowInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
            }

            if (node.GetValue("DocumentPageTitle") != null)
            {
                this.txtTitle.Text = node.GetValue("DocumentPageTitle").ToString();
            }
            if (node.GetValue("DocumentPageKeyWords") != null)
            {
                this.txtKeywords.Text = node.GetValue("DocumentPageKeyWords").ToString();
            }
            if (node.GetValue("DocumentPageDescription") != null)
            {
                this.txtDescription.Text = node.GetValue("DocumentPageDescription").ToString();
            }

            // fill drop down list
            DataSet ds = CssStylesheetInfoProvider.GetAllCssStylesheets();
            if ((ds != null) && (ds.Tables.Count > 0) && (ds.Tables[0].Rows.Count > 0))
            {
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    CssStylesheetInfo cs = new CssStylesheetInfo(dr);
                    if (cs.IsInSite(node.NodeSiteID))
                    {
                        ListItem li = new ListItem();
                        li.Text = dr["StylesheetDisplayName"].ToString();
                        li.Value = dr["StylesheetID"].ToString();
                        drpCssStyle.Items.Add(li);
                    }
                }
            }

            if (node.GetValue("DocumentStylesheetID") == null)
            {
                drpCssStyle.SelectedIndex = drpCssStyle.Items.IndexOf(drpCssStyle.Items.FindByValue("(default)"));
            }
            else
            {
                drpCssStyle.SelectedIndex = drpCssStyle.Items.IndexOf(drpCssStyle.Items.FindByValue(node.GetValue("DocumentStylesheetID").ToString()));
            }            
        }
    }


    /// <summary>
    /// Updates node properties in database
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void btnOK_Click(object sender, EventArgs e)
    {
        if (node != null)
        {
            bool autoCheck = !VersionManager.UseCheckInCheckOut(CMSContext.CurrentSite.SiteName);
            VersionManager vm = null;
            bool useWorkflow = false;
            WorkflowManager workflowMan = new WorkflowManager();

            // Get the document
            node = DocumentHelper.GetDocument(nodeId, CMSContext.CurrentUser.PreferredCultureCode, tree);

            //test if the document uses workflow
            if (workflowMan.GetNodeWorkflowScope(node) != null)
            {
                useWorkflow = true;
            }

            // If not using check-in/check-out, check out automatically
            if (autoCheck & useWorkflow)
            {
                if (node != null)
                {
                    // Check out
                    vm = new VersionManager(tree);
                    vm.CheckOut(node);

                }
            }
            
            // Update the data
            node.SetValue("DocumentPageTitle", this.txtTitle.Text);
            node.SetValue("DocumentPageKeyWords", this.txtKeywords.Text);
            node.SetValue("DocumentPageDescription", this.txtDescription.Text);

            if (drpCssStyle.SelectedItem.Text == "(default)" && drpCssStyle.SelectedItem.Value == Convert.ToString(-1))
            {
                node.SetValue("DocumentStylesheetID", null);    
            }
            else
            {
                node.SetValue("DocumentStylesheetID", drpCssStyle.SelectedValue);
            }
            DocumentHelper.UpdateDocument(node, tree);

            // Check in the document
            if (autoCheck & useWorkflow)
            {
                vm.CheckIn(node, null, null);
            }

            this.menuElem.ReloadMenu();

            lblInfo.Visible = true;
            lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
        }       
    }
   
}
