using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.SettingsProvider;
using CMS.CMSHelper;

public partial class CMSDesk_Content_Properties_workflow : CMSDeskPage
{
    // Current step ID
    int currentStepId = 0;
    // Current Node
    CMS.TreeEngine.TreeNode node = null;
    TreeProvider tree = null;
    WorkflowManager wm = null;
    int nodeId = 0;

    protected void Page_Load(object sender, EventArgs e)
    {
        this.gridHistory.GridView.GridLines = GridLines.Horizontal;
        // Setup the strings
        this.lblComment.Text = ResHelper.GetString("WorfklowProperties.Comment");
        this.lblApprove.Text = ResHelper.GetString("WorfklowProperties.Approve");
        this.lblSteps.Text = ResHelper.GetString("WorfklowProperties.Steps");
        this.lblHistory.Text = ResHelper.GetString("WorfklowProperties.History");
        this.lblSendMail.Text = ResHelper.GetString("WorfklowProperties.SendMail");

        this.btnApprove.Text = ResHelper.GetString("WorfklowProperties.btnApprove");
        this.btnReject.Text = ResHelper.GetString("WorfklowProperties.btnReject");
        this.btnArchive.Text = ResHelper.GetString("WorfklowProperties.btnArchive");

        // Current Node ID        
        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);        

        // Get the node
        tree = new TreeProvider();
        node = tree.SelectSingleNode(nodeId);

        if (node != null)
        {
            // Check read permissions
            if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                CMSDeskPage.RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
            }

            wm = new WorkflowManager();
            WorkflowInfo wi = wm.GetNodeWorkflow(node);

            // Prepare the query parameters
            object[,] parameters = new object[1, 3];
            parameters[0, 0] = "@DocumentID";
            parameters[0, 1] = 0;

            if (node != null)
            {
                parameters[0, 1] = node.DocumentID;
                currentStepId = ValidationHelper.GetInteger(node.GetValue("DocumentWorkflowStepID"), 0);
            }
            gridHistory.QueryParameters = parameters;
            gridHistory.OnExternalDataBound += new OnExternalDataBoundEventHandler(gridHistory_OnExternalDataBound);

            // Prepare the steps query parameters
            object[,] stepsParameters = new object[1, 3];
            stepsParameters[0, 0] = "@StepWorkflowID";
            stepsParameters[0, 1] = 0;
            if (wi != null)
            {
                stepsParameters[0, 1] = wi.WorkflowID;

            }
            gridSteps.QueryParameters = stepsParameters;
            gridSteps.OnExternalDataBound += new OnExternalDataBoundEventHandler(gridSteps_OnExternalDataBound);

            gridSteps.GridView.GridLines = GridLines.Horizontal;

            ReloadForm();

            // Check if document isn't checked out
            if (node.GetValue("DocumentCheckedOutByUserID") != null)
            {
                lblInfo.Text = ResHelper.GetString("WorfklowProperties.DocumentIsCheckedOut");
                DisableForm();
            }

            // Check if document has any version history record
            if (node.GetValue("DocumentCheckedOutVersionHistoryID") == null)
            {
                lblInfo.Text = ResHelper.GetString("WorfklowProperties.DocumentHasNoHistory");
                DisableForm();
            }

            // If workflow e-mails configured, show the e-mail checkbox
            if (SettingsKeyProvider.GetBoolValue("CMSSendWorkflowEmails"))
            {
                this.pnlSendMail.Visible = true;
            }

            // If no workflow scope set for node, hide the data  
            if (wi == null)
            {
                this.lblInfo.Text = ResHelper.GetString("WorfklowProperties.ScopeNotSet");
                DisableForm();
                this.pnlWorkflow.Visible = false;
                return;
            }
        }
    }


    /// <summary>
    /// External step binding
    /// </summary>
    object gridSteps_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        switch (sourceName.ToLower())
        {
            case "currentstepinfo":
                int stepId = ValidationHelper.GetInteger(parameter, -1);
                if (stepId == currentStepId)
                {
                    return "<img src=\"" + ResolveUrl("~/App_Themes/" +  this.Theme + "/Images/CMSDesk/Content/currentstep.gif") + "\" alt=\"\" />";
                }
                else
                {
                    return "";
                }
                //break;
        }
        return parameter;
    }


    /// <summary>
    /// External history binding
    /// </summary>
    object gridHistory_OnExternalDataBound(object sender, string sourceName, object parameter)
    {
        switch (sourceName.ToLower())
        {
            case "action":
                bool wasrejected = ValidationHelper.GetBoolean(parameter, false);
                if (wasrejected)
                {
                    return ResHelper.GetString("WorfklowProperties.Rejected");
                }
                else
                {
                    return ResHelper.GetString("WorfklowProperties.Approved");
                }
                //break;
        }
        return parameter;
    }


    /// <summary>
    /// Reloads the page data
    /// </summary>
    protected void ReloadData()
    {        
        ReloadForm();
        this.gridHistory.ReloadData();
        this.gridSteps.ReloadData();
    }


    protected void DisableForm()
    {
        pnlForm.Enabled = false;
        this.btnApprove.Enabled = false;
        this.btnReject.Enabled = false;
    }


    /// <summary>
    /// Reloads the form status
    /// </summary>
    protected void ReloadForm()
    {
        // Check modify permissions
        if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
        {
            DisableForm();
            this.btnArchive.Visible = false;
            this.lblInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
        }
        else
        {
            this.btnApprove.Visible = true;
            this.btnReject.Visible = false;
            this.pnlForm.Enabled = true;
            this.btnApprove.Enabled = true;
            this.btnReject.Enabled = true;
            this.btnArchive.Visible = false;
            this.pnlSendMail.Visible = true;
            if (currentStepId > 0)
            {
                // Setup the form
                WorkflowStepInfo si = WorkflowInfoProvider.GetWorkflowStepInfo(currentStepId);
                if (si != null)
                {
                    switch (si.StepName.ToLower())
                    {
                        case "published":
                            this.btnApprove.Visible = false;
                            this.btnArchive.Visible = true;
                            this.btnReject.Visible = true;
                            //this.pnlSendMail.Visible = false;
                            break;

                        case "archived":
                            this.btnReject.Visible = false;
                            this.btnArchive.Visible = false;
                            this.pnlSendMail.Visible = false;
                            DisableForm();
                            break;

                        case "edit":
                            this.btnReject.Visible = false;
                            //this.pnlSendMail.Visible = false;
                            break;

                        default:
                            this.btnReject.Visible = true;
                            if (!wm.CanUserApprove(node, CMSContext.CurrentUser))
                            {
                                this.btnArchive.Visible = false;
                                DisableForm();
                            }
                            break;
                    }
                }
            }
        }
    }


    /// <summary>
    /// Approve event handler
    /// </summary>
    protected void btnApprove_Click(object sender, EventArgs e)
    {
        // Go to next workflow step
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        WorkflowManager wm = new WorkflowManager(tree);

        // Get original step
        WorkflowStepInfo originalStep = wm.GetStepInfo(node);

        // Approve document
        WorkflowStepInfo nextStep = wm.MoveToNextStep(node, txtComment.Text);

        // Send workflow e-mails
        if (this.chkSendMail.Checked && SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSite.SiteName + ".CMSSendWorkflowEmails"))
        {
            if ((nextStep == null) || (nextStep.StepName.ToLower() == "published"))
            {
                // Publish e-mails
                wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Published, this.txtComment.Text);
            }
            else
            {
                // Approve e-mails
                wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Approved, this.txtComment.Text);
            }
        }

        // If no next step (workflow cancelled), hide the form
        if (nextStep == null)
        {
            this.lblInfo.Text = ResHelper.GetString("WorfklowProperties.WorkflowFinished");
            DisableForm();
            this.pnlWorkflow.Visible = false;
            return;
        }
        else
        {
            this.currentStepId = nextStep.StepID;
        }

        this.txtComment.Text = "";

        ReloadData();
    }


    /// <summary>
    /// Reject event handler
    /// </summary>
    protected void btnReject_Click(object sender, EventArgs e)
    {
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        WorkflowManager wm = new WorkflowManager(tree);

        // Get original step
        WorkflowStepInfo originalStep = wm.GetStepInfo(node);

        // Reject document
        WorkflowStepInfo previousStep = wm.MoveToPreviousStep(node, txtComment.Text);
        this.currentStepId = previousStep.StepID;

        // Send workflow e-mails
        if (this.chkSendMail.Checked && SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSite.SiteName + ".CMSSendWorkflowEmails"))
        {
            wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, previousStep, WorkflowActionEnum.Rejected, this.txtComment.Text);
        }

        this.txtComment.Text = "";

        ReloadData();
    }


    /// <summary>
    /// Archive event handler
    /// </summary>
    protected void btnArchive_Click(object sender, EventArgs e)
    {
        // Go to workflow step archived
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        WorkflowManager wm = new WorkflowManager(tree);

        // Get original step
        WorkflowStepInfo originalStep = wm.GetStepInfo(node);

        // Archive document
        WorkflowStepInfo nextStep = wm.ArchiveDocument(node, this.txtComment.Text);
        this.currentStepId = nextStep.StepID;

        // Send workflow e-mails
        if (this.chkSendMail.Checked && SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSite.SiteName + ".CMSSendWorkflowEmails"))
        {
            wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Archived, this.txtComment.Text);
        }

        this.txtComment.Text = "";

        ReloadData();
    }
}
