using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.CMSHelper;
using CMS.SiteProvider;
using CMS.EventLog;

public partial class CMSDesk_Content_delete : CMSDeskPage
{
    protected int nodeId = 0;

    protected void Page_Load(object sender, EventArgs e)
    {
        // Current Node ID to delete                        
        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);

        this.titleElem.TitleText = ResHelper.GetString("Content.DeleteTitle");
        this.titleElem.TitleImage = "~/App_Themes/" + this.Theme + "/Images/CMSDesk/Content/delete.gif";

        this.lblQuestion.Text = ResHelper.GetString("ContentDelete.Question");
        this.chkAllCultures.Text = ResHelper.GetString("ContentDelete.AllCultures");
        this.chkDestroy.Text = ResHelper.GetString("ContentDelete.Destroy");

        this.btnOK.Text = ResHelper.GetString("ContentDelete.Yes");
        this.btnCancel.Text = ResHelper.GetString("ContentDelete.No");

        // Get the node name
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        CMS.TreeEngine.TreeNode node = tree.SelectSingleNode(nodeId, TreeProvider.ALL_CULTURES);
        if (node != null)
        {
            this.titleElem.TitleText = ResHelper.GetString("Content.DeleteTitle") + " \"" + node.NodeName + "\"";
        }
    }


    protected void btnOK_Click(object sender, EventArgs e)
    {
        // Get the node
        TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
        CMS.TreeEngine.TreeNode node = tree.SelectSingleNode(nodeId);
        // Document not found
        if (node == null)
        {
            // If deleting all cultures, select any culture present
            if (this.chkAllCultures.Checked)
            {
                node = tree.SelectSingleNode(nodeId, TreeProvider.ALL_CULTURES);
            }
            else
            {
                AddAlert(ResHelper.GetString("DeleteDocument.CultureNotExists"));
                return;
            }
        }

        if (node != null)
        {
            // Check delete permissions
            bool hasUserDeletePermission = IsUserAuthorizedToDeleteDocument(node.NodeID, chkDestroy.Checked, chkAllCultures.Checked);

            if (hasUserDeletePermission)
            {
                // Delete the document
                try
                {
                    int parentId = node.NodeParentID;
                    if (parentId <= 0)
                    {
                        parentId = node.NodeID;
                    }
                    if (!DocumentHelper.DeleteDocument(node, tree, this.chkAllCultures.Checked, this.chkDestroy.Checked))
                    {
                        parentId = node.NodeID;
                    }
                    this.AddScript("     RefreshNode(" + parentId + ");");
                    this.AddScript("     DisplayDocument(" + parentId + ");");
                }
                catch (Exception ex)
                {
                    EventLogProvider log = new EventLogProvider(tree.Connection);
                    log.LogEvent("E", DateTime.Now, "Content", "DELETEDOC", CMSContext.CurrentUser.UserID, CMSContext.CurrentUser.UserName, nodeId, node.DocumentName, CMS.GlobalHelper.HTTPHelper.GetUserHostAddress(), ex.Message + ": " + ex.StackTrace, CMSContext.CurrentSite.SiteID, CMS.GlobalHelper.HTTPHelper.GetAbsoluteUri());
                    AddAlert(ResHelper.GetString("ContentRequest.DeleteFailed") + " : " + ex.Message);
                }
            }
            // Access denied - not authorized to delete the document
            else
            {
                RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtodeletedocument"), node.NodeAliasPath));
            }
        }
        else
        {
            AddAlert(ResHelper.GetString("ContentRequest.ErrorMissingSource"));
            return;
        }
    }


    /// <summary>
    /// Adds the alert message to the output request window
    /// </summary>
    /// <param name="message">Message to display</param>
    private void AddAlert(string message)
    {
        this.ltlScript.Text += ScriptHelper.GetAlertScript(message);
    }


    /// <summary>
    /// Adds the script to the output request window
    /// </summary>
    /// <param name="script">Script to add</param>
    private void AddScript(string script)
    {
        this.ltlScript.Text += ScriptHelper.GetScript(script);
    }


    protected void btnCancel_Click(object sender, EventArgs e)
    {
        this.AddScript("     DisplayDocument(" + Request.QueryString["nodeid"] + ");");
    }


    /// <summary>
    /// Checks whether the user is authorized to delete document.
    /// </summary>
    /// <param name="nodeId">Node id.</param>
    /// <param name="destroyDocHistory">Delete document history?</param>
    /// <param name="deleteAllCultureVersions">Delete all culture versions?</param>
    /// <returns></returns>
    protected bool IsUserAuthorizedToDeleteDocument(int nodeId, bool deleteDocHistory, bool deleteAllCultureVersions)
    {
        bool isAuthorized = true;

        CMS.TreeEngine.TreeNode node = null;
        TreeProvider tree = null;
        WorkflowManager wm = null;
        WorkflowStepInfo wsi = null;

        // Check delete permission
        if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Delete) == AuthorizationResultEnum.Allowed)
        {

            if (deleteDocHistory)
            {
                // Check destroy permisson
                if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Destroy) == AuthorizationResultEnum.Denied)
                {
                    isAuthorized = false;
                }
            }

            if (isAuthorized)
            {

                tree = new TreeProvider();
                node = tree.SelectSingleNode(nodeId);
                wm = new WorkflowManager(tree);


                // delete only current culture version
                //------------------------------------
                if (!deleteAllCultureVersions)
                {
                    // Check whether workflow is set
                    WorkflowInfo wi = wm.GetNodeWorkflow(node);
                    bool isWorkflow = (wi == null) ? false : true;

                    if (isWorkflow)
                    {
                        // Get current step info
                        wsi = wm.GetStepInfo(node);
                        if (wsi != null)
                        {
                            // Current step is 'edit', 'published', or 'archived'
                            if ((wsi.StepName.ToLower() == "edit") || (wsi.StepName.ToLower() == "published") || (wsi.StepName.ToLower() == "archived"))
                            {
                            }
                            // Another current step
                            else
                            {
                                bool canApprove = wm.CanUserApprove(node, CMSContext.CurrentUser);
                                if (!canApprove)
                                {
                                    isAuthorized = false;
                                }
                            }
                        }
                    }
                }
                // delete all culture versions
                //------------------------------------
                else
                {
                    if (node != null)
                    {
                        // Get the site info
                        SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                        if (si != null)
                        {
                            // Get all node culture versions
                            DataSet cultureNodes = tree.SelectNodes(si.SiteName, node.NodeAliasPath, TreeProvider.ALL_CULTURES, false, node.NodeClassName, null, null, -1, false);
                            if (!DataHelper.DataSourceIsEmpty(cultureNodes))
                            {
                                foreach (DataRow dr in cultureNodes.Tables[0].Rows)
                                {
                                    CMS.TreeEngine.TreeNode cultureNode = new CMS.TreeEngine.TreeNode(dr, node.NodeClassName, tree);

                                    // Check whether workflow is set
                                    WorkflowInfo wi = wm.GetNodeWorkflow(cultureNode);
                                    bool isWorkflow = (wi == null) ? false : true;

                                    if (isWorkflow)
                                    {
                                        // Get current step info
                                        wsi = wm.GetStepInfo(cultureNode);
                                        if (wsi != null)
                                        {
                                            // Current step is 'edit', 'published', or 'archived'
                                            if ((wsi.StepName.ToLower() == "edit") || (wsi.StepName.ToLower() == "published") || (wsi.StepName.ToLower() == "archived"))
                                            {
                                            }
                                            // Another current step
                                            else
                                            {
                                                bool canApprove = wm.CanUserApprove(cultureNode, CMSContext.CurrentUser);
                                                if (!canApprove)
                                                {
                                                    isAuthorized = false;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        else
        {
            isAuthorized = false;
        }

        return isAuthorized;
    }

}
