using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.CMSHelper;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.TreeEngine;
using CMS.WorkflowEngine;
using CMS.FormEngine;
using CMS.FCKeditorV2;

public partial class CMSDesk_Content_edit : CMSDeskPage
{
    int nodeId = 0;
    bool newdocument = false;
    bool newculture = false;
    CMS.TreeEngine.TreeNode node = null;
    TreeProvider tree = null;
    int templateId = 0;

    bool mShowToolbar = false;

    protected override void OnInit(EventArgs e)
    {
        base.OnInit(e);
        tree = new TreeProvider(CMSContext.CurrentUser);
        formElem.TreeProvider = tree;

        int classId = 0;

        //   current nodeID
        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);

        //   classID, used when creating new document                
        classId = ValidationHelper.GetInteger(Request.QueryString["classid"], 0);

        // templateID, used when Use template selection is enabled in actual class
        templateId = ValidationHelper.GetInteger(Request.QueryString["templateid"], 0);

        // Analyze the action parameter
        switch (ValidationHelper.GetString(Request.QueryString["action"], "").ToLower())
        {
            case "new":
                newdocument = true;
                break;

            case "newculture":
                newculture = true;
                break;

        }

        // Get the node
        if (newculture || newdocument)
        {
            node = DocumentHelper.GetDocument(nodeId, TreeProvider.ALL_CULTURES, tree);
            if (newculture)
            {
                DocumentHelper.ClearWorkflowInformation(node);
            }
        }
        else
        {
            node = DocumentHelper.GetDocument(nodeId, CMSContext.CurrentUser.PreferredCultureCode, tree);

        }

        // If node found, init the form
        if (node != null)
        {
            // CMSForm initialization
            formElem.NodeId = node.NodeID;
            formElem.CultureCode = CMSContext.CurrentUser.PreferredCultureCode; //node.DocumentCulture;

            DataClassInfo ci = null;
            // Set the form mode
            if (newdocument)
            {
                ci = DataClassInfoProvider.GetDataClass(classId);
                if (ci == null)
                {
                    throw new Exception("[Content/Edit.aspx]: Class ID '" + classId + "' not found.");
                }

                // Check if need template selection, if so, then redirect to template selection page
                if ((ci.ClassShowTemplateSelection) && (templateId == 0) && (ci.ClassName.ToLower() != "cms.menuitem"))
                {
                    Response.Redirect("TemplateSelection.aspx" + HttpContext.Current.Request.Url.Query);
                }

                formElem.FormMode = FormModeEnum.Insert;
                string newClassName = ci.ClassName;
                formElem.FormName = newClassName + ".default";
            }
            else if (newculture)
            {
                formElem.FormMode = FormModeEnum.InsertNewCultureVersion;
                // Default data document ID
                formElem.CopyDefaultDataFromDocumentId = ValidationHelper.GetInteger(Request.QueryString["sourcedocumentid"], 0);

                ci = DataClassInfoProvider.GetDataClass(node.NodeClassName);
                formElem.FormName = node.NodeClassName + ".default";
            }
            else
            {
                formElem.FormMode = FormModeEnum.Update;
                ci = DataClassInfoProvider.GetDataClass(node.NodeClassName);
            }
            formElem.Visible = true;

            // Display / hide the FCK toolbar area
            FormInfo fi = new FormInfo();
            fi.LoadXmlDefinition(ci.ClassFormDefinition);

            if (fi.UsesHtmlArea())
            {
                // Add script to display toolbar
                if (FCKeditor.UseFrameForToolbar || FCKeditor.UseIFrameForToolbar)
                {
                    if (this.formElem.HtmlAreaToolbarLocation.ToLower() == "out:fcktoolbar")
                    {
                        mShowToolbar = true;

                        if (FCKeditor.UseIFrameForToolbar)
                        {
                            this.formElem.HtmlAreaToolbarLocation = "Out:parent(FCKToolbar)";
                        }
                        else
                        {
                            this.formElem.HtmlAreaToolbarLocation = "Out:parent.frames['edittoolbar'](FCKToolbar)";
                        }
                    }
                }
                else
                {
                    this.plcToolbar.Visible = true;
                    this.pnlToolbarHeight.Visible = true;
                }
            }

            ReloadForm();
        }

        this.ltlScript.Text += ScriptHelper.GetScript(
            "function SaveDocument(nodeId) { " + ClientScript.GetPostBackEventReference(this.btnSave, null) + "; } \n" +
            "function Approve(nodeId) { " + ClientScript.GetPostBackEventReference(this.btnApprove, null) + "; } \n" +
            "function CheckIn(nodeId) { " + ClientScript.GetPostBackEventReference(this.btnCheckIn, null) + "; } \n"
            );
    }


    private void ReloadForm()
    {
        this.lblWorkflowInfo.Text = "";
        formElem.Enabled = true;
        if ( (node != null) && !newdocument && !newculture)
        {
            // Check read permissions
            if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
            {
                CMSDeskPage.RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"), node.NodeAliasPath));
            }
            // Check modify permissions
            else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
            {
                formElem.Enabled = false;
                pnlToolbarHeight.Visible = false;
                lblWorkflowInfo.Text = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
            }
            else
            {

                // Setup the workflow information
                WorkflowManager wm = new WorkflowManager(tree);
                WorkflowInfo wi = wm.GetNodeWorkflow(node);
                if ((wi != null) && (!newculture))
                {
                    // Get current step info
                    WorkflowStepInfo si = wm.GetStepInfo(node);
                    if (si != null)
                    {
                        bool allowApproval = true;
                        bool canApprove = wm.CanUserApprove(node, CMSContext.CurrentUser);
                        if (!(canApprove || (si.StepName.ToLower() == "edit") || (si.StepName.ToLower() == "published") || (si.StepName.ToLower() == "archived")))
                        {
                            formElem.Enabled = false;
                        }

                        // Check-in, Check-out
                        if (VersionManager.UseCheckInCheckOut(CMSContext.CurrentSite.SiteName))
                        {
                            if (node.GetValue("DocumentCheckedOutByUserID") == null)
                            {
                                // If not checked out, add the check-out information
                                if (canApprove || (si.StepName.ToLower() == "edit") || (si.StepName.ToLower() == "published") || (si.StepName.ToLower() == "archived"))
                                {
                                    this.lblWorkflowInfo.Text = ResHelper.GetString("EditContent.DocumentCheckedIn");
                                }
                                this.formElem.Enabled = newculture;
                                this.pnlToolbarHeight.Visible = false;
                            }
                            else
                            {
                                // If checked out by current user, add the check-in button
                                int checkedOutBy = ValidationHelper.GetInteger(node.GetValue("DocumentCheckedOutByUserID"), 0);
                                if (checkedOutBy == CMSContext.CurrentUser.UserID)
                                {
                                    // Document is checked out
                                    this.lblWorkflowInfo.Text = ResHelper.GetString("EditContent.DocumentCheckedOut");
                                }
                                else
                                {
                                    // Checked out by somebody else
                                    this.lblWorkflowInfo.Text = ResHelper.GetString("EditContent.DocumentCheckedOutByAnother");
                                    this.formElem.Enabled = newculture;
                                    this.pnlToolbarHeight.Visible = false;
                                }
                                allowApproval = false;
                            }
                        }

                        // Document approval
                        if (allowApproval)
                        {
                            switch (si.StepName.ToLower())
                            {
                                case "edit":
                                case "published":
                                case "archived":
                                    break;

                                default:
                                    // If the user is authorized to perform the step, display the approve and reject buttons
                                    if (!canApprove)
                                    {
                                        this.lblWorkflowInfo.Text += " " + ResHelper.GetString("EditContent.NotAuthorizedToApprove");
                                    }
                                    break;
                            }
                            this.lblWorkflowInfo.Text += " " + String.Format(ResHelper.GetString("EditContent.CurrentStepInfo"), si.StepDisplayName);

                        }

                    }
                }
            }
        }
        this.pnlWorkflowInfo.Visible = (this.lblWorkflowInfo.Text != "");
    }


    /// <summary>
    /// Adds the alert message to the output request window
    /// </summary>
    /// <param name="message">Message to display</param>
    private void AddAlert(string message)
    {
        this.ltlScript.Text += ScriptHelper.GetAlertScript(message);
    }


    /// <summary>
    /// Adds the script to the output request window
    /// </summary>
    /// <param name="script">Script to add</param>
    private void AddScript(string script)
    {
        this.ltlScript.Text += ScriptHelper.GetScript(script);
    }


    protected void btnSave_Click(object sender, EventArgs e)
    {
        SaveDocument();
    }


    private void SaveDocument()
    {
        if (nodeId > 0)
        {
            bool autoCheck = false;
            bool useWorkflow = false;
            VersionManager vm = null;
            if (!newdocument && !newculture)
            {
                autoCheck= !VersionManager.UseCheckInCheckOut(CMSContext.CurrentSite.SiteName);
                WorkflowManager workflowMan = new WorkflowManager();

                // Get the document
                node = DocumentHelper.GetDocument(nodeId, CMSContext.CurrentUser.PreferredCultureCode, tree);

                //test if the document uses workflow
                if (workflowMan.GetNodeWorkflow(node) != null)
                {
                    useWorkflow = true;
                }
            }

            try
            {
                // If not using check-in/check-out, check out automatically
                if (autoCheck & useWorkflow)
                {
                    if (node != null)
                    {
                        // Check out
                        vm = new VersionManager(tree);
                        vm.CheckOut(node);

                    }
                }

                // tree node is returned from CMSForm.Save method
                if ((node != null) && !newdocument && !newculture)
                {
                    formElem.Save(node);
                    //CMS.TreeEngine.TreeNode nodeData = 
                    //if (nodeData != null)
                    //{
                    //    DocumentHelper.CopyNodeData(nodeData, node, false, true, true, true, false);
                    //}
                }
                else
                {
                    node = formElem.Save();
                }



                if (node != null)
                {
                    // Check in the document
                    if (autoCheck & useWorkflow)
                    {
                        vm.CheckIn(node, null, null);
                        ReloadForm();
                    }
                    
                    if ((templateId > 0) && (newdocument))
                    {
                        node.SetValue("DocumentPageTemplateID", templateId);
                        node.Update();
                    }
                    
                    int newNodeId = node.NodeID;
                    if (newdocument || newculture)
                    {
                        // document tree is refreshed and new document is displayed
                        this.AddScript("     RefreshNode(" + newNodeId + "," + newNodeId + ");");
                        this.AddScript("     SelectNode(" + newNodeId + ");");
                    }
                    else
                    {
                        this.AddScript("     PassiveRefresh(" + nodeId + ", " + node.NodeParentID + ");");
                        // displays - "Changes saved"
                        //lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
                    }

                    // If not menuitem type, switch to form mode to keep editing the form
                    if (!TreePathUtils.IsMenuItemType(node.NodeClassName))
                    {
                        CMSContext.CurrentUser.ViewMode = ViewModeEnum.EditForm;
                    }
                }
            }
            catch (Exception ex)
            {
                AddAlert(ResHelper.GetString("ContentRequest.SaveFailed") + " : " + ex.Message);
            }
        }
    }


    protected void btnApprove_Click(object sender, EventArgs e)
    {
        if (node != null)
        {
            // Save the changes first
            bool autoCheck = !VersionManager.UseCheckInCheckOut(CMSContext.CurrentSite.SiteName);
            if (autoCheck)
            {
                //SaveDocument();
            }

            // Approve the document - Go to next workflow step
            WorkflowManager wm = new WorkflowManager(tree);

            // Get original step
            WorkflowStepInfo originalStep = wm.GetStepInfo(node);

            // Approve the document
            WorkflowStepInfo nextStep = wm.MoveToNextStep(node, "");
            if (nextStep != null)
            {

                // Send workflow e-mails
                if (SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSite.SiteName + ".CMSSendWorkflowEmails"))
                {
                    if (nextStep.StepName.ToLower() == "published")
                    {
                        // Publish e-mails
                        wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Published, "");
                    }
                    else
                    {
                        // Approve e-mails
                        wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Approved, "");
                    }
                }
                this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");
            }
            else
            {
                // Workflow has been removed
                this.AddScript("     SelectNode(" + nodeId + ");");
                //node = DocumentHelper.GetDocument(nodeId, CMSContext.CurrentUser.PreferredCultureCode, tree);
            }
            
            this.formElem.lblInfo.Text += ResHelper.GetString("ContentEdit.WasApproved");
        }
        ReloadForm();
    }


    protected void btnCheckIn_Click(object sender, EventArgs e)
    {
        // Save the document first
        SaveDocument();
        // Check in the document
        if (node != null)
        {
            VersionManager verMan = new VersionManager(tree);

            // Check in the document        
            verMan.CheckIn(node, null, null);

            this.formElem.lblInfo.Text += "<br/>" + ResHelper.GetString("ContentEdit.WasCheckedIn");
        }

        //this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");

        ReloadForm();
    }


    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        if (this.formElem.Enabled && mShowToolbar)
        {
            this.ClientScript.RegisterClientScriptBlock(typeof(string), ScriptHelper.TOOLBAR_SCRIPT_KEY, ScriptHelper.ToolbarScript);
        }
    }

}

