using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.TreeEngine;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.WorkflowEngine;
using CMS.SettingsProvider;
using CMS.FCKeditorV2;

public partial class ContentEditMenu : System.Web.UI.UserControl
{
    int nodeId = 0;
    bool newdocument = false;
    string imagesUrl = null;
    CMS.TreeEngine.TreeNode node = null;

    bool mAllowEdit = true;
    bool mAllowSave = true;
    bool mEnablepasiveRefresh = true;

    string mWorkflowInfo = "";

    public event EventHandler LocalSave;

    int mMenuItems = 0;

    #region "Button display options"

    private bool mShowSave = true;
    private bool mShowCheckOut = true;
    private bool mShowCheckIn = true;
    private bool mShowUndoCheckOut = true;
    private bool mShowApprove = true;
    private bool mShowSubmitToApproval = true;
    private bool mShowReject = true;
    private bool mShowProperties = true;

    /// <summary>
    /// Show the Save button?
    /// </summary>
    public bool ShowSave
    {
        get { return mShowSave; }
        set { mShowSave = value; }
    }

    /// <summary>
    /// Show the Check Out button?
    /// </summary>
    public bool ShowCheckOut
    {
        get { return mShowCheckOut; }
        set { mShowCheckOut = value; }
    }

    /// <summary>
    /// Show the Check In button?
    /// </summary>
    public bool ShowCheckIn
    {
        get { return mShowCheckIn; }
        set { mShowCheckIn = value; }
    }

    /// <summary>
    /// Show the Undo CheckOut button?
    /// </summary>
    public bool ShowUndoCheckOut
    {
        get { return mShowUndoCheckOut; }
        set { mShowUndoCheckOut = value; }
    }

    /// <summary>
    /// Show the Approve button?
    /// </summary>
    public bool ShowApprove
    {
        get { return mShowApprove; }
        set { mShowApprove = value; }
    }

    /// <summary>
    /// Show the Reject button?
    /// </summary>
    public bool ShowReject
    {
        get { return mShowReject; }
        set { mShowReject = value; }
    }

    /// <summary>
    /// Show the Submit To Approval button?
    /// </summary>
    public bool ShowSubmitToApproval
    {
        get { return mShowSubmitToApproval; }
        set { mShowSubmitToApproval = value; }
    }

    /// <summary>
    /// Show the Properties button?
    /// </summary>
    public bool ShowProperties
    {
        get { return mShowProperties; }
        set { mShowProperties = value; }
    }

    #endregion


    #region "Public properties"

    /// <summary>
    /// Enable passive refresh
    /// </summary>
    public bool EnablePassiveRefresh
    {
        get
        {
            return mEnablepasiveRefresh;
        }
        set
        {
            mEnablepasiveRefresh = value;
        }
    }


    /// <summary>
    /// Returns true if the document editing is allowed
    /// </summary>
    public bool AllowEdit
    {
        get
        {
            return mAllowEdit;
        }
    }


    /// <summary>
    /// Save action allowed
    /// </summary>
    public bool AllowSave
    {
        get
        {
            return mAllowSave;
        }
        set
        {
            mAllowSave = value;
        }
    }


    /// <summary>
    /// Returns the 
    /// </summary>
    public string WorkflowInfo
    {
        get
        {
            return mWorkflowInfo;
        }
    }


    /// <summary>
    /// Count of the menu items currently displayed
    /// </summary>
    public int MenuItems
    {
        get
        {
            return mMenuItems;
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        imagesUrl = ResolveUrl("~/App_Themes/" + this.Page.Theme + "/Images/CMSDesk/Content/");

        // Get the document ID
        nodeId = ValidationHelper.GetInteger(Request.QueryString["nodeid"], 0);

        // Get information whether new document is created
        newdocument = ((Request.QueryString["action"] != null) && (Request.QueryString["action"].ToLower() == "new"));

        ReloadMenu();

        string script = "function LocalSave(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnSave, null) + "; } \n" +
                        "function LocalApprove(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnApprove, null) + "; } \n" +
                        "function Reject(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnReject, null) + "; } \n" +
                        "function CheckOut(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnCheckOut, null) + "; } \n" +
                        "function LocalCheckIn(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnCheckIn, null) + "; } \n" +
                        "function UndoCheckOut(nodeId) { " + this.Page.ClientScript.GetPostBackEventReference(this.btnUndoCheckout, null) + "; } \n";

        // Add control frame name
        if (!FCKeditor.UseFrameForToolbar || FCKeditor.UseIFrameForToolbar)
        {
            script += "var controlFrame = 'editview'; \n";
        }
        else
        {
            script += "var controlFrame = 'edittoolbar'; \n";
        }

        this.ltlScript.Text += ScriptHelper.GetScript(script);

        this.menuElem.Layout = CMS.skmMenuControl.MenuLayout.Horizontal;
    }


    public void ReloadMenu()
    {
        CMS.skmMenuControl.MenuItem newItem = null;

        // Create the edit menu
        menuElem.Items.Clear();

        mMenuItems = 0;

        // Save button
        CMS.skmMenuControl.MenuItem saveItem = null;
        if (mShowSave)
        {
            saveItem = new CMS.skmMenuControl.MenuItem();
            saveItem.ToolTip = ResHelper.GetString("EditMenu.Save");
            saveItem.JavascriptCommand = "SaveDocument(" + nodeId.ToString() + ");";
            saveItem.ImageAltText = saveItem.ToolTip;
            saveItem.Text = ResHelper.GetString("EditMenu.IconSave");
            saveItem.Image = imagesUrl + "save.gif";
            saveItem.MouseOverImage = saveItem.Image;
            saveItem.MouseOverCssClass = "MenuItemEdit";
            saveItem.CssClass = "MenuItemEdit";
            menuElem.Items.Add(saveItem);
            mMenuItems += 1;
        }

        mWorkflowInfo = "";

        mAllowEdit = true;
        mAllowSave = true;
        bool canApprove = false;

        if (!newdocument)
        {
            // Get the document
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            //node = tree.SelectSingleNode(nodeId);
            node = DocumentHelper.GetDocument(nodeId, CMSContext.CurrentUser.PreferredCultureCode, tree);

            if (node != null)
            {
                // Check read permissions
                if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
                {
                    CMSDeskPage.RedirectToAccessDenied(String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoreaddocument"),node.NodeAliasPath));
                }
                // Check modify permissions
                else if (CMSContext.CurrentUser.IsAuthorizedPerTreeNode(nodeId, NodePermissionsEnum.Modify) == AuthorizationResultEnum.Denied)
                {
                    mAllowSave = false;
                    mWorkflowInfo = String.Format(ResHelper.GetString("cmsdesk.notauthorizedtoeditdocument"), node.NodeAliasPath);
                }
                else
                {

                    WorkflowManager wm = new WorkflowManager(tree);
                    WorkflowInfo wi = wm.GetNodeWorkflow(node);
                    // If workflow not null, process the workflow information to display the items
                    if (wi != null)
                    {
                        // Get current step info
                        WorkflowStepInfo si = wm.GetStepInfo(node);
                        if (si != null)
                        {
                            bool allowApproval = true;
                            canApprove = wm.CanUserApprove(node, CMSContext.CurrentUser);
                            if (!(canApprove || (si.StepName.ToLower() == "edit") || (si.StepName.ToLower() == "published") || (si.StepName.ToLower() == "archived")))
                            {
                                mAllowSave = false;
                            }

                            // Check-in, Check-out
                            if (VersionManager.UseCheckInCheckOut(CMSContext.CurrentSite.SiteName))
                            {
                                if (node.GetValue("DocumentCheckedOutByUserID") == null)
                                {
                                    mAllowSave = false;
                                    mAllowEdit = false;

                                    // If not checked out, add the check-out button
                                    if (mShowCheckOut && (canApprove || (si.StepName.ToLower() == "edit") || (si.StepName.ToLower() == "published") || (si.StepName.ToLower() == "archived")))
                                    {
                                        newItem = new CMS.skmMenuControl.MenuItem();
                                        newItem.ToolTip = ResHelper.GetString("EditMenu.CheckOut");
                                        newItem.JavascriptCommand = "CheckOut(" + nodeId.ToString() + ");";
                                        newItem.ImageAltText = newItem.ToolTip;
                                        newItem.Text = ResHelper.GetString("EditMenu.IconCheckOut");
                                        newItem.Image = imagesUrl + "checkout.gif";
                                        newItem.MouseOverImage = newItem.Image;
                                        newItem.MouseOverCssClass = "MenuItemEdit";
                                        newItem.CssClass = "MenuItemEdit";
                                        menuElem.Items.Add(newItem);
                                        mMenuItems += 1;
                                    }
                                    // Set workflow info
                                    this.mWorkflowInfo += ResHelper.GetString("EditContent.DocumentCheckedIn");
                                }
                                else
                                {
                                    // If checked out by current user, add the check-in button
                                    int checkedOutBy = ValidationHelper.GetInteger(node.GetValue("DocumentCheckedOutByUserID"), 0);
                                    if (checkedOutBy == CMSContext.CurrentUser.UserID)
                                    {
                                        if (mShowCheckIn)
                                        {
                                            newItem = new CMS.skmMenuControl.MenuItem();
                                            newItem.ToolTip = ResHelper.GetString("EditMenu.CheckIn");
                                            newItem.JavascriptCommand = "CheckIn(" + nodeId.ToString() + ");";
                                            newItem.ImageAltText = newItem.ToolTip;
                                            newItem.Text = ResHelper.GetString("EditMenu.IconCheckIn");
                                            newItem.Image = imagesUrl + "checkin.gif";
                                            newItem.MouseOverImage = newItem.Image;
                                            newItem.MouseOverCssClass = "MenuItemEdit";
                                            newItem.CssClass = "MenuItemEdit";
                                            menuElem.Items.Add(newItem);
                                            mMenuItems += 1;
                                        }

                                        if (mShowUndoCheckOut)
                                        {
                                            newItem = new CMS.skmMenuControl.MenuItem();
                                            newItem.ToolTip = ResHelper.GetString("EditMenu.UndoCheckout");
                                            newItem.JavascriptCommand = "UndoCheckOut(" + nodeId.ToString() + ");";
                                            newItem.ImageAltText = newItem.ToolTip;
                                            newItem.Text = ResHelper.GetString("EditMenu.IconUndoCheckout");
                                            newItem.Image = imagesUrl + "undocheckout.gif";
                                            newItem.MouseOverImage = newItem.Image;
                                            newItem.MouseOverCssClass = "MenuItemEdit";
                                            newItem.CssClass = "MenuItemEdit";
                                            menuElem.Items.Add(newItem);
                                            mMenuItems += 1;
                                        }
                                        // Set workflow info

                                            this.mWorkflowInfo += ResHelper.GetString("EditContent.DocumentCheckedOut");
                                    }
                                    else
                                    {
                                        mAllowSave = false;
                                        mAllowEdit = false;
                                        // Set workflow info
                                        this.mWorkflowInfo += ResHelper.GetString("EditContent.DocumentCheckedOutByAnother");
                                    }
                                    allowApproval = false;
                                }
                            }

                            // Document approval
                            if (allowApproval)
                            {
                                switch (si.StepName.ToLower())
                                {
                                    case "edit":
                                        // When edit step, display submit to approval
                                        if (mShowSubmitToApproval)
                                        {
                                            newItem = new CMS.skmMenuControl.MenuItem();
                                            newItem.ToolTip = ResHelper.GetString("EditMenu.SubmitToApproval");
                                            newItem.JavascriptCommand = "Approve(" + nodeId.ToString() + ");";
                                            newItem.ImageAltText = newItem.ToolTip;
                                            newItem.Text = ResHelper.GetString("EditMenu.IconSubmitToApproval");
                                            newItem.Image = imagesUrl + "approve.gif";
                                            newItem.MouseOverImage = newItem.Image;
                                            newItem.MouseOverCssClass = "MenuItemEdit";
                                            newItem.CssClass = "MenuItemEdit";
                                            menuElem.Items.Add(newItem);
                                            mMenuItems += 1;
                                        }
                                        break;

                                    case "published":
                                    case "archived":
                                        // No workflow options in these steps
                                        break;

                                    default:
                                        // If the user is authorized to perform the step, display the approve and reject buttons
                                        if (canApprove)
                                        {
                                            if (mShowApprove)
                                            {
                                                newItem = new CMS.skmMenuControl.MenuItem();
                                                newItem.ToolTip = ResHelper.GetString("EditMenu.Approve");
                                                newItem.JavascriptCommand = "Approve(" + nodeId.ToString() + ");";
                                                newItem.ImageAltText = newItem.ToolTip;
                                                newItem.Text = ResHelper.GetString("EditMenu.IconApprove");
                                                newItem.Image = imagesUrl + "approve.gif";
                                                newItem.MouseOverImage = newItem.Image;
                                                newItem.MouseOverCssClass = "MenuItemEdit";
                                                newItem.CssClass = "MenuItemEdit";
                                                menuElem.Items.Add(newItem);
                                                mMenuItems += 1;
                                            }

                                            if (mShowReject)
                                            {
                                                newItem = new CMS.skmMenuControl.MenuItem();
                                                newItem.ToolTip = ResHelper.GetString("EditMenu.Reject");
                                                newItem.JavascriptCommand = "Reject(" + nodeId.ToString() + ");";
                                                newItem.ImageAltText = newItem.ToolTip;
                                                newItem.Text = ResHelper.GetString("EditMenu.IconReject");
                                                newItem.Image = imagesUrl + "reject.gif";
                                                newItem.MouseOverImage = newItem.Image;
                                                newItem.MouseOverCssClass = "MenuItemEdit";
                                                newItem.CssClass = "MenuItemEdit";
                                                menuElem.Items.Add(newItem);
                                                mMenuItems += 1;
                                            }
                                        }
                                        break;
                                }

                                // Set workflow info
                                if (!wm.CanUserApprove(node, CMSContext.CurrentUser))
                                {
                                    this.mWorkflowInfo += "<br/>" + ResHelper.GetString("EditContent.NotAuthorizedToApprove");
                                }
                                this.mWorkflowInfo += "<br/>" + String.Format(ResHelper.GetString("EditContent.CurrentStepInfo"), si.StepDisplayName);
                            }
                        }
                    }
                }             
                                

                if (mShowProperties)
                {
                    newItem = new CMS.skmMenuControl.MenuItem();
                    newItem.ToolTip = ResHelper.GetString("EditMenu.Properties");
                    newItem.JavascriptCommand = "Properties(" + nodeId.ToString() + ");";
                    newItem.ImageAltText = newItem.ToolTip;
                    newItem.Text = ResHelper.GetString("EditMenu.IconProperties");
                    newItem.Image = imagesUrl + "properties.gif";
                    newItem.MouseOverImage = newItem.Image;
                    newItem.MouseOverCssClass = "MenuItemEdit";
                    newItem.CssClass = "MenuItemEdit";
                    menuElem.Items.Add(newItem);
                    mMenuItems += 1;
                }
            }
            // If not allowed to save, disable the save item
            if (!mAllowSave && mShowSave)
            {
                saveItem.JavascriptCommand = "";
                saveItem.MouseOverCssClass = "MenuItemEditDisabled";
                saveItem.CssClass = "MenuItemEditDisabled";
                saveItem.Image = imagesUrl + "savedisabled.gif";
                saveItem.MouseOverImage = saveItem.Image;
            }
        }
    }


    protected void btnApprove_Click(object sender, EventArgs e)
    {
        if (node != null)
        {
            // Approve the document - Go to next workflow step
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            WorkflowManager wm = new WorkflowManager(tree);

            // Get original step
            WorkflowStepInfo originalStep = wm.GetStepInfo(node);

            // Approve the document
            WorkflowStepInfo nextStep = wm.MoveToNextStep(node, "");

            // Send workflow e-mails
            if (SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSite.SiteName + ".CMSSendWorkflowEmails"))
            {
                if (nextStep.StepName.ToLower() == "published")
                {
                    // Publish e-mails
                    wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Published, "");
                }
                else
                {
                    // Approve e-mails
                    wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, nextStep, WorkflowActionEnum.Approved, "");
                }
            }
        }

        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");
        }
    }


    protected void btnReject_Click(object sender, EventArgs e)
    {
        // Reject the document - Go to previous workflow step
        if (node != null)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            WorkflowManager wm = new WorkflowManager(tree);

            // Get original step
            WorkflowStepInfo originalStep = wm.GetStepInfo(node);

            // Reject the document
            WorkflowStepInfo previousStep = wm.MoveToPreviousStep(node, "");

            // Send workflow e-mails
            if (SettingsKeyProvider.GetBoolValue(CMSContext.CurrentSite.SiteName + ".CMSSendWorkflowEmails"))
            {
                wm.SendWorkflowEmails(node, CMSContext.CurrentUser, originalStep, previousStep, WorkflowActionEnum.Rejected, "");
            }
        }
        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");
        }
    }


    protected void btnCheckIn_Click(object sender, EventArgs e)
    {
        // Check in the document
        if (node != null)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            // Check in the document        
            verMan.CheckIn(node, null, null);
        }
        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");
        }
    }


    protected void btnCheckOut_Click(object sender, EventArgs e)
    {
        // Check out the document
        if (node != null)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            verMan.EnsureVersion(node);

            // Check out the document
            verMan.CheckOut(node);
        }
        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");
        }
    }


    protected void btnUndoCheckout_Click(object sender, EventArgs e)
    {
        // Check out the document
        if (node != null)
        {
            TreeProvider tree = new TreeProvider(CMSContext.CurrentUser);
            VersionManager verMan = new VersionManager(tree);

            // Undo check out
            verMan.UndoCheckOut(node);
        }
        ReloadMenu();
        if (this.EnablePassiveRefresh)
        {
            this.ltlScript.Text += ScriptHelper.GetScript("PassiveRefresh(" + nodeId + ");");
        }
    }


    protected void btnSave_Click(object sender, EventArgs e)
    {
        if (LocalSave != null)
        {
            LocalSave(sender, e);
        }
    }

}
