using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.Forums;
using CMS.GlobalHelper;
using CMS.ExtendedControls;
using CMS.CMSHelper;
using CMS.URLRewritingEngine;

public partial class CMSForum_ForumGroup : System.Web.UI.UserControl
{

    public ForumProperties mForumProperties = new ForumProperties();

    /// <summary>
    /// If set public user can read, no insert new post
    /// </summary>
    public bool NoAuthenticatedRead
    {
        get { return mForumProperties.NoAuthenticatedRead; }
        set { mForumProperties.NoAuthenticatedRead = value; }
    }
    
    /// <summary>
    /// Alternating style
    /// </summary>
    public bool UseAlternatingStyle
    {
        get { return mForumProperties.UseAlternatingStyle; }
        set { mForumProperties.UseAlternatingStyle = value; }
    }

    /// <summary>
    /// Query strin represent Forum
    /// </summary>
    public string QueryStringForumID
    {
        get { return mForumProperties.QueryStringForumID; }
        set { mForumProperties.QueryStringForumID = value; }
    }

    /// <summary>
    /// GroupId
    /// </summary>
    public int GroupID
    {
        get { return mForumProperties.GroupID; }
        set { mForumProperties.GroupID = value; }
    }

    /// <summary>
    /// DisplayOnlyOpen
    /// </summary>
    public bool DisplayOnlyOpen
    {
        get { return mForumProperties.DisplayOnlyOpen; }
        set { mForumProperties.DisplayOnlyOpen = value; }
    }

    /// <summary>
    /// CheckPermissions
    /// </summary>
    public bool CheckPermissions
    {
        get { return mForumProperties.CheckPermissions; }
        set { mForumProperties.CheckPermissions = value; }
    }

    protected void Page_Load(object sender, EventArgs e)
    {
        if (mForumProperties.StopProcessing)
        {
            return;
        }

        lblForum.Text = ResHelper.GetString("ForumGroup.lblForum");
        lblPosts.Text = ResHelper.GetString("ForumGroup.lblPosts");
        lblThreads.Text = ResHelper.GetString("ForumGroup.lblThreads");
        lblLastPost.Text = ResHelper.GetString("ForumGroup.lblLastPost");

        ForumGroupInfo fgi = ForumGroupInfoProvider.GetForumGroupInfo(this.GroupID);

        if (fgi != null)
        {

            lblForumHeader.Text = fgi.GroupDisplayName;
            lblForumHeaderDescription.Text = fgi.GroupDescription;

            DataSet ds = ForumInfoProvider.GetForumsByGroupID(this.GroupID);

            string mForums = "";

            bool even = false;

            if (!DataHelper.DataSourceIsEmpty(ds))
            {
                foreach (DataRow dr in ds.Tables[0].Rows)
                {
                    even = !even;

                    if ((!this.DisplayOnlyOpen) || (ValidationHelper.GetBoolean(dr["ForumOpen"], false)))
                    {
                        int mForumAcess = ValidationHelper.GetInteger(dr["ForumAccess"], 0);
                        //string roleName = ForumInfoProvider.GetRoleNameForForum(ValidationHelper.GetInteger(dr["ForumID"], 0));

                        
                        if ((!this.CheckPermissions) || mForumProperties.CheckPermission(ValidationHelper.GetInteger(dr["ForumID"], 0)) || NoAuthenticatedRead)
                        {
                            DataSet mDs = null;
                            mDs = ForumPostInfoProvider.SelectForumPosts(ValidationHelper.GetInteger(dr["ForumID"], 0), "/%", "PostUserName = '" + Server.HtmlEncode(ValidationHelper.GetString(dr["ForumLastPostUserName"], "")) + "'", "'PostTime'");

                            mForums += CreateForumRow(Server.HtmlEncode(ValidationHelper.GetString(dr["ForumDisplayName"], "")), Server.HtmlEncode(ValidationHelper.GetString(dr["ForumDescription"], "")),
                                                      ValidationHelper.GetInteger(dr["ForumThreads"], 0), ValidationHelper.GetInteger(dr["forumPosts"], 0),
                                                      Server.HtmlEncode(ValidationHelper.GetString(dr["ForumLastPostUserName"], "")), ValidationHelper.GetDateTime(dr["ForumLastPostTime"],
                                                      DateTimePicker.NOT_SELECTED), ValidationHelper.GetInteger(dr["ForumID"], 0), even, mDs, ValidationHelper.GetBoolean(dr["ForumDisplayEmails"], false));
                        }
                    }
                }

                ltlForums.Text = mForums;
            }
        }
    }

    /// <summary>
    /// Create forum row
    /// </summary>
    /// <param name="forumName"></param>
    /// <param name="forumDescription"></param>
    /// <param name="threads"></param>
    /// <param name="posts"></param>
    public string CreateForumRow(string forumName, string forumDescription, int threads, int posts, string lastUser, DateTime lastTime, int forumId, bool even, DataSet lastPost, bool displayEmail)
    {
        string row = "";

        string url = URLRewriter.CurrentURL;
        url = UrlHelper.AddParameterToUrl(url, this.QueryStringForumID, forumId.ToString());

        // Row
        if (this.UseAlternatingStyle)
        {
            if (even)
            {
                row += "<tr class=\"Forum ForumEven\">";
            }
            else
            {
                row += "<tr class=\"Forum ForumOdd\">";
            }
        }
        else
        {
            row += "<tr class=\"Forum\">";
        }

        row += "<td class=\"ForumInfo\">\n";
        row += "<div>\n";
        row += "<a href=\"" + url + "\" class=\"ForumName\">" + forumName + "</a>\n";
        row += "</div>\n";
        row += "<div class=\"ForumDescription\">\n";
        row += forumDescription;
        row += "</div>\n";
        row += "</td>\n";

        row += "<td class=\"Threads\">\n";
        row += "<span>\n";
        row += threads.ToString();
        row += "</span>\n";
        row += "</td>\n";

        row += "<td class=\"Posts\">\n";
        row += "<span>\n";
        row += posts.ToString();
        row += "</span>\n";
        row += "</td>\n";

        row += "<td class=\"LastPost\">\n";
        row += "<div class=\"PostUser\">\n";
        if ((displayEmail) && (!DataHelper.DataSourceIsEmpty(lastPost)) && (ValidationHelper.GetString(lastPost.Tables[0].Rows[0]["PostUserMail"], "") != ""))
        {
            row += "<a href=\"mailto:" + Server.HtmlEncode(ValidationHelper.GetString(lastPost.Tables[0].Rows[0]["PostUserMail"], "")) + " \" >" + lastUser + "</a>\n";
        }
        else
        {
            row += lastUser;
        }
        row += "</div>\n";
        row += "<div class=\"PostTime\">\n";
        row += "(" + lastTime.ToString() + ")\n";
        row += "</div>\n";
        row += "</td>\n";
        row += "</tr>\n";

        return row;
    }
}
