using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using CMS.GlobalHelper;
using CMS.SiteProvider;
using CMS.CMSHelper;


public partial class SiteManager_Administration_Users_User_Edit_General : AdministrationPage
{
    protected int userId = 0;
    protected string password;
    protected string myCulture = "";
    protected string myUICulture = "";

    protected void Page_Load(object sender, EventArgs e)
    {
        LabelUserName.Text = ResHelper.GetString("Administration-User_Edit_General.UserName");
        LabelFullName.Text = ResHelper.GetString("Administration-User_Edit_General.FullName");
        LabelFirstName.Text = ResHelper.GetString("Administration-User_Edit_General.FirstName");
        LabelMiddleName.Text = ResHelper.GetString("Administration-User_Edit_General.MiddleName");
        LabelLastName.Text = ResHelper.GetString("Administration-User_Edit_General.LastName");
        LabelEmail.Text = ResHelper.GetString("Administration-User_Edit_General.Email");
        LabelEnabled.Text = ResHelper.GetString("Administration-User_Edit_General.Enabled");
        LabelIsEditor.Text = ResHelper.GetString("Administration-User_Edit_General.IsEditor");
        LabelIsGlobalAdministrator.Text = ResHelper.GetString("Administration-User_Edit_General.IsGlobalAdministrator");
        lblCulture.Text = ResHelper.GetString("Administration-User_Edit_General.Culture");
        lblUICulture.Text = ResHelper.GetString("Administration-User_Edit_General.UICulture");
        ButtonOK.Text = ResHelper.GetString("Administration-User_Edit_General.OK");
        RequiredFieldValidatorFullName.ErrorMessage = ResHelper.GetString("Administration-User_New.RequiresFullName");
        RequiredFieldValidatorUserName.ErrorMessage = ResHelper.GetString("Administration-User_New.RequiresUserName");
        lblIsExternal.Text = ResHelper.GetString("Administration-User_Edit_General.IsExternal");

        if (CMSContext.CurrentUser.IsGlobalAdministrator)
        {
            LabelIsGlobalAdministrator.Visible = true;
            CheckBoxLabelIsGlobalAdministrator.Visible = true;
        }
        if ((Request.QueryString["userid"] != null) && (Request.QueryString["userid"] != ""))
        {
            userId = Convert.ToInt32(Request.QueryString["userid"]);
            if (!IsPostBack) LoadData();
        }

    }

    /// <summary>
    /// Loads data of edited user from DB into TextBoxes.
    /// </summary>
    protected void LoadData()
    {

        // Fill lstCulture
        DataSet ds = new DataSet();
        ds = CultureInfoProvider.GetAllCultures();
        lstCulture.DataSource = ds.Tables[0];
        lstCulture.DataTextField = ds.Tables[0].Columns["CultureName"].ColumnName.ToString();
        lstCulture.DataValueField = ds.Tables[0].Columns["CultureID"].ColumnName.ToString();
        lstCulture.DataBind();


        lstCulture.Items.Insert(0, ResHelper.GetString("Administration-User_Edit.Default"));
        lstCulture.Items[0].Value = "0";

        // Fill lstUICulture

        ds = UICultureInfoProvider.GetAllUICultures();
        lstUICulture.DataSource = ds.Tables[0];
        lstUICulture.DataTextField = ds.Tables[0].Columns["UICultureName"].ColumnName.ToString();
        lstUICulture.DataValueField = ds.Tables[0].Columns["UICultureID"].ColumnName.ToString();
        lstUICulture.DataBind();

        lstUICulture.Items.Insert(0, ResHelper.GetString("Administration-User_Edit.Default"));
        lstUICulture.Items[0].Value = "0";

        string userName = UserInfoProvider.GetUserNameById(userId);
        UserInfo ui = UserInfoProvider.GetUserInfo(userName);
        TextBoxEmail.Text = ui.Email;
        TextBoxFirstName.Text = ui.FirstName;
        TextBoxFullName.Text = ui.FullName;
        TextBoxLastName.Text = ui.LastName;
        TextBoxMiddleName.Text = ui.MiddleName;
        TextBoxUserName.Text = ui.UserName;
        CheckBoxEnabled.Checked = ui.Enabled;
        CheckBoxIsEditor.Checked = ui.IsEditor;
        if (CheckBoxLabelIsGlobalAdministrator.Visible)
        {
            CheckBoxLabelIsGlobalAdministrator.Checked = ui.IsGlobalAdministrator;
        }
        chkIsExternal.Checked = ui.IsExternal;
        password = ui.GetValue("UserPassword").ToString();




        myCulture = ui.PreferredCultureCode;
        myUICulture = ui.PreferredUICultureCode;

        // select cultures for actual user or set default
        if ((myCulture != null) && (myCulture != ""))
        {
            try
            {
                CultureInfo ci = CultureInfoProvider.GetCultureInfo(myCulture);
                lstCulture.SelectedIndex = lstCulture.Items.IndexOf(lstCulture.Items.FindByValue(ci.CultureID.ToString()));
            }
            catch
            {
                lstCulture.SelectedIndex = lstCulture.Items.IndexOf(lstCulture.Items.FindByValue("0"));
            }
        }
        else
            lstCulture.SelectedIndex = lstCulture.Items.IndexOf(lstCulture.Items.FindByValue("0"));

        if ((myUICulture != null) && (myUICulture != ""))
        {
            try
            {
                UICultureInfo ciUI = UICultureInfoProvider.GetUICultureInfo(myUICulture);
                lstUICulture.SelectedIndex = lstUICulture.Items.IndexOf(lstUICulture.Items.FindByValue(ciUI.UICultureID.ToString()));
            }
            catch
            {
                lstUICulture.SelectedIndex = lstUICulture.Items.IndexOf(lstUICulture.Items.FindByValue("0"));
            }
        }
        else
            lstUICulture.SelectedIndex = lstUICulture.Items.IndexOf(lstUICulture.Items.FindByValue("0"));


    }

    /// <summary>
    /// Saves data of edited user from TextBoxes into DB.
    /// </summary>
    protected void ButtonOK_Click(object sender, EventArgs e)
    {
        // Check modify permission
        if ((CMSContext.CurrentSite != null) && (!CMSContext.CurrentUser.IsAuthorizedPerResource("cms.administration", "Modify")))
        {
            RedirectToCMSDeskAccessDenied("cms.administration", "Modify");
        }


        // finds whether required fields are not empty
        string result = new Validator().NotEmpty(TextBoxUserName.Text, ResHelper.GetString("Administration-User_New.RequiresUserName")).NotEmpty(TextBoxFullName.Text, ResHelper.GetString("Administration-User_New.RequiresFullName")).Result;

        if (result == "")
        {
            UserInfo ui = UserInfoProvider.GetUserInfo(userId);
            password = ui.GetValue("UserPassword").ToString();
            userId = ui.UserID;

            UserInfo uiTest = UserInfoProvider.GetUserInfo(TextBoxUserName.Text);
            if ((uiTest == null) || (uiTest.UserID == userId))
            {
                if (ui == null)
                {
                    ui = new UserInfo();
                }
                ui.Email = TextBoxEmail.Text;
                ui.FirstName = TextBoxFirstName.Text;
                ui.FullName = TextBoxFullName.Text;
                ui.LastName = TextBoxLastName.Text;
                ui.MiddleName = TextBoxMiddleName.Text;
                ui.UserName = TextBoxUserName.Text;
                ui.Enabled = CheckBoxEnabled.Checked;
                ui.IsEditor = CheckBoxIsEditor.Checked;
                if (CheckBoxLabelIsGlobalAdministrator.Visible)
                {
                    ui.IsGlobalAdministrator = CheckBoxLabelIsGlobalAdministrator.Checked;
                }
                else
                {
                    ui.IsGlobalAdministrator = false;
                }
                ui.IsExternal = chkIsExternal.Checked;
                ui.SetValue("UserPassword", password);
                ui.UserID = userId;




                // Set values of cultures.
                if (lstCulture.SelectedValue == "0")
                    ui.PreferredCultureCode = "";
                else
                {
                    CultureInfo ci = CultureInfoProvider.GetCultureInfo(Convert.ToInt32(lstCulture.SelectedValue));
                    ui.PreferredCultureCode = ci.CultureCode;
                }
                if (lstUICulture.SelectedValue == "0")
                    ui.PreferredUICultureCode = "";
                else
                {
                    UICultureInfo ciUI = UICultureInfoProvider.GetUICultureInfo(Convert.ToInt32(lstUICulture.SelectedValue));
                    ui.PreferredUICultureCode = ciUI.UICultureCode;
                }


                UserInfoProvider.SetUserInfo(ui);

                lblInfo.Visible = true;
                lblInfo.Text = ResHelper.GetString("General.ChangesSaved");
            }
            else
            {
                lblError.Visible = true;
                lblError.Text = ResHelper.GetString("Administration-User_New.UserExists");
            }
        }
        else
        {
            lblError.Visible = true;
            lblError.Text = result;
        }
    }
}
