using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;

public partial class ImportGridView : System.Web.UI.UserControl
{
    private string mCheckBoxClass;

    /// <summary>
    /// Name of the checkbox class
    /// </summary>
    public string CheckBoxClass
    {
        get { return mCheckBoxClass; }
        set { mCheckBoxClass = value; }
    }


    private DataSet mDataSet;


    /// <summary>
    /// DataSet for internal data
    /// </summary>
    public DataSet DataSet
    {
        get { return mDataSet; }
        set { mDataSet = value; }
    }


    private bool mResolveConflicts=true;


    /// <summary>
    /// True if the conflicts are resolved
    /// </summary>
    public bool ResolveConflicts
    {
        get { return mResolveConflicts; }
        set { mResolveConflicts = value; }
    }


    private string mFirstBoundFiel;


    /// <summary>
    /// Name of the column in the dataset which will be bound on the first column of the gridview
    /// </summary>
    public string FirstBoundField
    {
        get { return mFirstBoundFiel; }
        set { mFirstBoundFiel = value; }
    }


    private string mSecondBoundField;


    /// <summary>
    /// Name of the column in the dataset which will be bound on the second column of the gridview
    /// </summary>
    public string SecondBoundField
    {
        get { return mSecondBoundField; }
        set { mSecondBoundField = value; }
    }


    private string mFirstColumnHeader = ResHelper.GetString("SiteImport.Step.Overwrite");


    /// <summary>
    /// First column header
    /// </summary>
    public string FirstColumnHeader
    {
        get { return mFirstColumnHeader; }
        set { mFirstColumnHeader = value; }
    }


    private string mSecondColumnHeader = ResHelper.GetString("SiteImport.Step.DisplayName");


    /// <summary>
    /// Second column header
    /// </summary>
    public string SecondColumnHeader
    {
        get 
        {
            return mSecondColumnHeader; 
        }
        set { mSecondColumnHeader = value; }
    }


    private string mThirdColumnHeader = ResHelper.GetString("SiteImport.Step.CodeName");


    /// <summary>
    /// Third column header
    /// </summary>
    public string ThirdColumnHeader
    {
        get 
        {
            if (SecondBoundField == null)
            {
                return "";
            }
            return mThirdColumnHeader; 
        }
        set { mThirdColumnHeader = value; }
    }


    private string mLabelCaption = "";


    /// <summary>
    /// Label caption
    /// </summary>
    public string LabelCaption
    {
        get { return mLabelCaption; }
        set { mLabelCaption = value; }
    }


    private string mLabelNoDataCaption = "";


    /// <summary>
    /// Label caption if there is no data to bind
    /// </summary>
    public string LabelNoDataCaption
    {
        get { return mLabelNoDataCaption; }
        set { mLabelNoDataCaption = value; }
    }


    protected void Page_Load(object sender, EventArgs e)
    {
    }


    /// <summary>
    /// Bind the data
    /// </summary>
    /// <param name="ds">Source data set</param>
    public void Bind()
    {
        chk.Attributes.Add("onclick", "SelectCheckboxes(this,'" + CheckBoxClass + "');");
        this.chk.Text = ResHelper.GetString("ImportSite.SelectDeselectAll");

        // Add bound fields
        BoundField first = new BoundField();
        first.DataField = FirstBoundField;
        gv.Columns.Add(first);

        BoundField second = new BoundField();
        second.DataField = SecondBoundField;
        gv.Columns.Add(second);

        if (this.ResolveConflicts)
        {
            BoundField conflicted = new BoundField();
            conflicted.DataField = "conflicted";
            gv.Columns.Add(conflicted);
        }

        if ((!DataHelper.DataSourceIsEmpty(DataSet)) && (FirstBoundField != null))// && (SecondBoundField != null))
        {
            if (this.ResolveConflicts)
            {
                gv.Columns[4].Visible = true;
            }
            pnl.Visible = true;

            gv.Columns[1].HeaderText = FirstColumnHeader;
            gv.Columns[2].HeaderText = SecondColumnHeader;
            gv.Columns[3].HeaderText = ThirdColumnHeader;
            DataView sortedView = DataSet.Tables[0].DefaultView;
            sortedView.Sort = FirstBoundField + " ASC";
            gv.DataSource = sortedView;
            gv.DataBind();

            if (this.ResolveConflicts)
            {
                for (int i = 0; i < gv.Rows.Count; ++i)
                {
                    if ((gv.Rows[i].FindControl("chkSelect") != null) && (gv.Rows[i].FindControl("lblAsterisk") != null))
                    {
                        if (Convert.ToBoolean(gv.Rows[i].Cells[4].Text))
                        {
                            ((HtmlInputCheckBox)gv.Rows[i].FindControl("chkSelect")).Checked = false;
                            ((Label)gv.Rows[i].FindControl("lblAsterisk")).Text = "*";
                        }
                        else
                        {
                            ((HtmlInputCheckBox)gv.Rows[i].FindControl("chkSelect")).Checked = true;
                        }
                    }
                }
                gv.Columns[4].Visible = false;
            }

            lbl.Text = LabelCaption;
        }
        else
        {
            lbl.Text = LabelNoDataCaption;
        }
    }


    /// <summary>
    /// Get selected rows
    /// </summary>
    /// <returns></returns>
    public string[] GetSelectedObjects()
    {
        ArrayList tmpList = new ArrayList();
        for (int i = 0; i < gv.Rows.Count; ++i)
        {
            if ((gv.Rows[i].FindControl("chkSelect") != null) && (((HtmlInputCheckBox)gv.Rows[i].FindControl("chkSelect")).Checked))
            {
                tmpList.Add(gv.Rows[i].Cells[3].Text.ToLower());
            }
        }
        return (string[])tmpList.ToArray(typeof(string));
    }

}
