using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;

using CMS.GlobalHelper;
using CMS.ImportExport;
using CMS.SiteProvider;
using CMS.CMSHelper;
using CMS.SettingsProvider;

public partial class ImportSitePage : SiteManagerPage, ICallbackEventHandler
{

    #region Public properties

    /// <summary>
    /// import settings stored in viewstate
    /// </summary>
    public SiteImportSettings ImportSettings
    {
        get
        {
            if (ViewState["settings"] == null)
            {
                ViewState["settings"] = new SiteImportSettings();
            }
            return (SiteImportSettings)ViewState["settings"];
        }
    }


    /// <summary>
    /// imported css styles
    /// </summary>
    public ArrayList ImportedStyles
    {
        get
        {
            if (ViewState["cssStyles"] == null)
            {
                ViewState["cssStyles"] = new ArrayList();
            }
            return (ArrayList)ViewState["cssStyles"];
        }
    }


    /// <summary>
    /// Flag if documents are imported
    /// </summary>
    public bool DocumentsImported
    {
        get
        {
            return ValidationHelper.GetBoolean(ViewState["docImported"], false);
        }
        set
        {
            ViewState["docImported"] = value;
        }
    }


    /// <summary>
    /// Wizard index
    /// </summary>
    public int WizardIndex
    {
        get
        {
            return ValidationHelper.GetInteger(ViewState["wzdIndex"], 1);
        }
        set
        {
            ViewState["wzdIndex"] = value;
        }
    }

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsCallback)
        {
            // Register the script to perform get flags for showing buttons retrieval callback
            Page.ClientScript.RegisterClientScriptBlock(GetType(), "GetImportState", ScriptHelper.GetScript("function GetImportState(cancel){ return " + this.Page.ClientScript.GetCallbackEventReference(this, "cancel", "SetImportStateMessage", null) + " } \n"));

            // Add refference to the wizard
            ucObjectWizard.Wizard = wzdImport;
            ucObjectWizard.OnFinish += new Import_ObjectsWizard.OnFinishHandler(OnFinish);
            ucObjectWizard.OnAfterAsyncStart += new Import_ObjectsWizard.OnAfterAsyncStartHandler(OnAfterAsyncStart);
            ucObjectWizard.OnBeforeAsyncStart += new Import_ObjectsWizard.OnBeforeAsyncStartHandler(OnBeforeAsyncStart);

            string sites = ResHelper.GetString("ImportSite.Sites");
            string title = ResHelper.GetString("ImportSite.Title");

            btnFinish.Text = ResHelper.GetString("ImportSite.Finish");
            btnCancel.Text = ResHelper.GetString("general.cancel");
            lblResult.Text = ResHelper.GetString("SiteImport.PreparingImport");

            //initializes PageTitle
            string[,] pageTitleTabs = new string[2, 3];
            pageTitleTabs[0, 0] = sites;
            pageTitleTabs[0, 1] = "~/CMSSiteManager/Sites/site_list.aspx";
            pageTitleTabs[1, 0] = title;
            pageTitleTabs[1, 1] = "";
            ptImport.Breadcrumbs = pageTitleTabs;
            ptImport.TitleText = title;
            ptImport.TitleImage = "~/App_Themes/" + this.Theme + "/Images/CMSSiteManager/Sites/import.gif";

            this.ltlScript.Text = ScriptHelper.GetScript("function NextStep(btnNext,elementDiv){btnNext.disabled=true;elementDiv.disabled=true;" + this.ClientScript.GetPostBackEventReference(btnHidden, null) + "}");

            this.ltlScriptAfter.Text = ScriptHelper.GetScript("function SetImportStateMessage(rValue, context){	var values = rValue.split('\\n');" +
                "if((values[0]=='F') || (values[0]=='E') || values==''){StopImportStateTimer();document.getElementById('" + btnFinish.ClientID + "').disabled = false;document.getElementById('" + btnCancel.ClientID + "').disabled = true;}" +
                "document.getElementById('" + lblResult.ClientID + "').innerHTML = values[1];" +
                "if(values[0]=='E'){document.getElementById('" + lblAsyncError.ClientID + "').innerHTML = values[2];}" +
                "else if((values[0]=='I') || (values[0]=='F')){document.getElementById('" + lblAsyncWarning.ClientID + "').innerHTML = values[3];}" +
                "else if((values=='')){document.getElementById('" + lblAsyncError.ClientID + "').innerHTML = values[2];" +
                "document.getElementById('" + lblAsyncWarning.ClientID + "').innerHTML = values[3];document.getElementById('" + lblFinish.ClientID + "').innerHTML = '" + ResHelper.GetString("siteimport.wzdstepfinish.completedWithLoggingProblems") + "';}}");

            this.btnCancel.Attributes.Add("onclick", "document.getElementById('" + btnFinish.ClientID + "').disabled = false;document.getElementById('" + btnCancel.ClientID + "').disabled = true; return CancelImport();");

            if (!IsPostBack)
            {
                //wzdImport.StartNextButtonText = ResHelper.GetString("ImportSite.NextStep");
                //wzdImport.StepNextButtonText = ResHelper.GetString("ImportSite.NextStep");
                //wzdImport.StepPreviousButtonText = ResHelper.GetString("ImportSite.BackStep");
                //wzdImport.FinishCompleteButtonText = ResHelper.GetString("ImportSite.Finish");
                //wzdImport.FinishPreviousButtonText = ResHelper.GetString("ImportSite.BackStep");

                lblStepDesc.Text = ResHelper.GetString("ImportSite.Step.lblAsteriskExplanation");


                wzdImport.ActiveStepIndex = 0;

                lblStepTitle.Text = string.Format(ResHelper.GetString("ImportSite.StepFileSelection.Caption"), WizardIndex);
                if (HttpContext.Current != null)
                {
                    lblStepDesc.Text = string.Format(ResHelper.GetString("ImportSite.StepFileSelection.Desc"), HttpContext.Current.Server.MapPath("~/" + ImportProvider.IMPORT_PATH));
                }
            }
        }
    }

   
    void btnCancelHidden_Click(object sender, EventArgs e)
    {
        if (ucObjectWizard.Manager.Settings != null)
        {
            ucObjectWizard.Manager.Settings.Cancel();
        }
    }


    protected void wzdImport_FinishButtonClick(object sender, EventArgs e)    {
        Response.Redirect("~/CMSSiteManager/Sites/site_list.aspx");
    }


    protected void btnHidden_onClick(object sender, EventArgs e)
    {
        ++WizardIndex;
        lblError.Visible = false;

        // handles wizards steps
        switch (wzdImport.ActiveStepIndex)
        {
            case 0:
                // if there is selected file, allows to proceed to the next step
                if ((ucFileSelection.FileName != null) && (ucFileSelection.FileName != ""))
                {
                    try
                    {
                        ImportSettings.SourceFilePath = Path.Combine(ImportProvider.IMPORT_RELATIVE_PATH, ucFileSelection.FileName);

                        // Create temporary files
                        ImportProvider.CreateTemporaryFiles(ImportSettings);
                        // Fill site display name and site name
                        DataSet ds = ImportProvider.GetDataSetFromXml(ImportSettings, "siteinfo.xml" + ExportProvider.EXPORT_EXTENSION, Tags.SITES_TAG);

                        if (ds != null)
                        {
                            DataRow dr = ds.Tables[0].Rows[0];

                            ucSiteDetails.SiteDisplayName = Convert.ToString(dr["SiteDisplayName"]);
                            ucSiteDetails.SiteName = Convert.ToString(dr["SiteName"]);

                            ucObjectWizard.StartIndex = 2;
                            ucObjectWizard.CaptionIndex = 3;
                            wzdImport.ActiveStepIndex++;
                            LoadData(wzdImport.ActiveStepIndex);
                        }
                        // Import only settings
                        else
                        {
                            ucObjectWizard.ImportOnlySettings = true;
                            ucObjectWizard.StartIndex = 2;
                            ucObjectWizard.CaptionIndex = 2;
                            wzdImport.ActiveStepIndex += 2;
                            LoadData(wzdImport.ActiveStepIndex);
                        }
                    }
                    catch (Exception ex)
                    {
                        lblError.Visible = true;
                        lblError.Text = ex.Message;
                        return;
                    }
                }
                else
                {
                    return;
                }
                break;

            case 1:
                // if fields are filled, allows proceed to the next step
                string siteName = ucSiteDetails.SiteName;
                string siteDisplayName = ucSiteDetails.SiteDisplayName;
                if ((siteName != "") && (siteDisplayName != ""))
                {
                    ImportSettings.NewSiteName = siteName;
                    ImportSettings.NewSiteDisplayName = siteDisplayName;

                    // if site with the same name already exists, fails
                    try
                    {
                        SiteInfo si = SiteInfoProvider.GetSiteInfo(siteName);
                        if (si == null)
                        {
                            throw new Exception();
                        }

                        lblError.Visible = true;
                        lblError.Text = ResHelper.GetString("SiteImport.StepSiteDetails.ErrorSiteExists");
                        return;
                    }
                    catch
                    {
                        // binds gridviews
                        wzdImport.ActiveStepIndex++;
                        LoadData(wzdImport.ActiveStepIndex);
                    }
                }
                else
                {
                    return;
                }
                break;
            default:
                break;
        }

        WizardIndex = ucObjectWizard.HandleNextButtonClick(e, ImportSettings, WizardIndex, wzdImport.ActiveStepIndex);
    }


    private void LoadData(int stepIndex)
    {
        lblStepDesc.Visible = false;

        switch (stepIndex)
        {
            // Load data for site details step
            case 1:
                lblStepTitle.Text = string.Format(ResHelper.GetString("ImportSite.StepSiteDetails.Caption"), WizardIndex);
                break;
            default:
                break;
        }
    }


    private void HandleError(Exception ex, WizardNavigationEventArgs e)
    {
        lblError.Visible = true;
        lblError.Text = string.Format(ResHelper.GetString("ImportExportProvider.DetailedError"), ex.Message, ex.StackTrace);
        e.Cancel = true;
    }


    private void OnFinish()
    {
        // Load data for finish step
        lblStepTitle.Text = string.Format(ResHelper.GetString("ImportSite.wzdStepFinish.Caption"), WizardIndex);
        lblStepDesc.Visible = false;
        btnFinish.Enabled = true;

        if (ucObjectWizard.Result == ResultEnum.Warning)
        {
            lblResult.Text = ResHelper.GetString("siteimport.wzdstepfinish.completedWarnings");
            lblError.Visible = true;
            lblError.Text = GetWarnings();
        }
        else
        {
            lblResult.Text = ResHelper.GetString("siteimport.wzdstepfinish.completed");
        }
    }


    private void OnAfterAsyncStart()
    {
        // Load data for finish step
        lblStepTitle.Text = string.Format(ResHelper.GetString("ImportSite.wzdStepFinish.Caption"), WizardIndex);
        lblStepDesc.Visible = false;
    }


    private void OnBeforeAsyncStart()
    {
        // Start the timer for the callbacks
        ltlScript.Text = ScriptHelper.GetScript("StartImportStateTimer();");
    }


    private string GetWarnings()
    {
        string result = "";
        for (int i = 0; i < ucObjectWizard.WarningMessages.Count; ++i)
        {
            if (i == (ucObjectWizard.WarningMessages.Count - 1))
            {
                result += ucObjectWizard.WarningMessages[i];
            }
            else
            {
                result += ucObjectWizard.WarningMessages[i] + "<br/>";
            }
        }

        if (result != "")
        {
            result += "<br/>";
        }

        for (int i = 0; i < ucObjectWizard.PermissionMessages.Count; ++i)
        {
            if (i == (ucObjectWizard.PermissionMessages.Count - 1))
            {
                result += ucObjectWizard.PermissionMessages[i];
            }
            else
            {
                result += ucObjectWizard.PermissionMessages[i] + "<br/>";
            }
        }

        return result;
    }


    /// <summary>
    /// Callback event handler
    /// </summary>
    /// <param name="argument">Callback argument</param>
    public void RaiseCallbackEvent(string argument)
    {
        SiteImportSettings settings = ucObjectWizard.Manager.Settings;

        if (settings != null)
        {
            if (ValidationHelper.GetBoolean(argument, false))
            {
                    // Cancel the import
                    settings.Cancel();
            }

            hdnStatus.Value = settings.GlobalImportState;
        }
        // Server has been restarted, loggin information has lost, inform user
        else
        {
            hdnStatus.Value += "F" + SiteImportSettings.SEPARATOR + ResHelper.GetString("SiteImport.ServerRestart") + SiteImportSettings.SEPARATOR + SiteImportSettings.SEPARATOR;
        }
    }


    /// <summary>
    /// Callback result retrieving handler
    /// </summary>
    public string GetCallbackResult()
    {
        return hdnStatus.Value;
    }

}
