using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.PortalControls;
using CMS.GlobalHelper;
using CMS.TreeEngine;
using CMS.CMSHelper;
using CMS.SiteProvider;

public partial class CMSWebParts_Membership_ChangePassword : CMSAbstractWebPart
{

    #region "Public properties"

    /// <summary>
    /// Show only when authenticated
    /// </summary>
    public bool ShowOnlyWhenAuthenticated
    {
        get
        {
            return ValidationHelper.GetBoolean(this.GetValue("ShowOnlyWhenAuthenticated"), true);
        }
        set
        {
            this.SetValue("ShowOnlyWhenAuthenticated", value);
            this.Visible = (!value || CMSContext.CurrentUser.IsAuthenticated());
        }
    }


    /// <summary>
    /// Maximal new password length
    /// </summary>
    public int MaximalPasswordLength
    {
        get
        {
            return ValidationHelper.GetInteger(this.GetValue("MaximalPasswordLength"), 0);
        }
        set
        {
            this.SetValue("MaximalPasswordLength", value);
            this.txtNewPassword.MaxLength = value;
            this.txtConfirmPassword.MaxLength = value;
        }
    }


    #endregion


    /// <summary>
    /// Content loaded event handler
    /// </summary>
    public override void OnContentLoaded()
    {
        base.OnContentLoaded();
        SetupControl();
    }


    /// <summary>
    /// Initializes the control properties
    /// </summary>
    protected void SetupControl()
    {
        if ((this.PageManager.ViewMode == ViewModeEnum.Design) || (this.HideOnCurrentPage) || (!this.IsVisible))
        {
            // Do not process
        }
        else
        {
            this.Visible = (!this.ShowOnlyWhenAuthenticated || CMSContext.CurrentUser.IsAuthenticated());
            this.txtNewPassword.MaxLength = this.MaximalPasswordLength;
            this.txtConfirmPassword.MaxLength = this.MaximalPasswordLength;

            // Set labels text
            this.lblOldPassword.Text = ResHelper.GetString("ChangePassword.lblOldPassword");
            this.lblNewPassword.Text = ResHelper.GetString("ChangePassword.lblNewPassword");
            this.lblConfirmPassword.Text = ResHelper.GetString("ChangePassword.lblConfirmPassword");
            this.btnOk.Text = ResHelper.GetString("ChangePassword.btnOK");
        }
    }

    protected void btnOk_Click(object sender, EventArgs e)
    {
        CurrentUserInfo ui = CMSContext.CurrentUser;
        CurrentSiteInfo si = CMSContext.CurrentSite;
        if ((ui != null) && (si!=null))
        {
            string userName = ui.UserName;
            string siteName = si.SiteName;

            // new password correctly filled
            if (txtConfirmPassword.Text == txtNewPassword.Text)
            {
                // Old password match
                if (UserInfoProvider.AuthenticateUser(userName, txtOldPassword.Text.Trim(), siteName) != null)
                {
                    UserInfoProvider.SetPassword(userName, txtNewPassword.Text.Trim());
                    lblInfo.Visible = true;
                    lblInfo.Text = ResHelper.GetString("ChangePassword.ChangesSaved");
                }
                else
                {
                    lblError.Visible = true;
                    lblError.Text = ResHelper.GetString("ChangePassword.ErrorOldPassword");
                }
            }
            else
            {
                lblError.Visible = true;
                lblError.Text = ResHelper.GetString("ChangePassword.ErrorNewPassword");
            }
        }
    }
}