using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using CMS.GlobalHelper;
using CMS.PortalControls;
using CMS.CMSHelper;

public partial class CMSWebParts_Newsletters_NewsletterSubscriptionWebPart : CMSAbstractWebPart
{
    #region "Public properties"

    /// <summary>
    /// Display first name
    /// </summary>
    public bool DisplayFirstName
    {
        get
        {
            return ValidationHelper.GetBoolean(this.GetValue("DisplayFirstName"), this.subscription.DisplayFirstName);
        }
        set
        {
            this.SetValue("DisplayFirstName", value);
            this.subscription.DisplayFirstName = value;
        }
    }


    /// <summary>
    /// First name text
    /// </summary>
    public string FirstNameText
    {
        get
        {
            return DataHelper.GetNotEmpty(this.GetValue("FirstNameText"), ResHelper.LocalizeString("{$NewsletterSubscription.FirstName$}"));
        }
        set
        {
            this.SetValue("FirstNameText", value);
            this.subscription.LabelFirstName.Text = value;
        }
    }


    /// <summary>
    /// Display Last name
    /// </summary>
    public bool DisplayLastName
    {
        get
        {
            return ValidationHelper.GetBoolean(this.GetValue("DisplayLastName"), this.subscription.DisplayLastName);
        }
        set
        {
            this.SetValue("DisplayLastName", value);
            this.subscription.DisplayLastName = value;
        }
    }


    /// <summary>
    /// Last name text
    /// </summary>
    public string LastNameText
    {
        get
        {
            return DataHelper.GetNotEmpty(this.GetValue("LastNameText"), ResHelper.LocalizeString("{$NewsletterSubscription.LastName$}"));
        }
        set
        {
            this.SetValue("LastNameText", value);
            this.subscription.LabelLastName.Text = value;
        }
    }


    /// <summary>
    /// Newsletter name
    /// </summary>
    public string NewsletterName
    {
        get
        {
            return ValidationHelper.GetString(this.GetValue("NewsletterName"), this.subscription.NewsletterName);
        }
        set
        {
            this.SetValue("NewsletterName", value);
            this.subscription.NewsletterName = value;
        }
    }


    /// <summary>
    /// E-mail text
    /// </summary>
    public string EmailText
    {
        get
        {
            return DataHelper.GetNotEmpty(this.GetValue("EmailText"), ResHelper.LocalizeString("{$NewsletterSubscription.Email$}"));
        }
        set
        {
            this.SetValue("EmailText", value);
            this.subscription.LabelEmail.Text = value;
        }
    }


    /// <summary>
    /// Button text
    /// </summary>
    public string ButtonText
    {
        get
        {
            return DataHelper.GetNotEmpty(this.GetValue("ButtonText"), ResHelper.LocalizeString("{$NewsletterSubscription.Submit$}"));
        }
        set
        {
            this.SetValue("ButtonText", value);
            this.subscription.ButtonOK.Text = value;
        }
    }


    /// <summary>
    /// Skin ID
    /// </summary>
    public override string SkinID
    {
        get
        {
            return base.SkinID;
        }
        set
        {
            base.SkinID = value;
            this.subscription.LabelFirstName.SkinID = value;
            this.subscription.LabelLastName.SkinID = value;
            this.subscription.LabelEmail.SkinID = value;
            this.subscription.TextBoxFirstName.SkinID = value;
            this.subscription.TextBoxLastName.SkinID = value;
            this.subscription.TextBoxEmail.SkinID = value;
            this.subscription.ButtonOK.SkinID = value;
        }
    }

    #endregion


    /// <summary>
    /// Content loaded event handler
    /// </summary>
    public override void OnContentLoaded()
    {
        base.OnContentLoaded();
        SetupControl();
    }


    /// <summary>
    /// Initializes the control properties
    /// </summary>
    protected void SetupControl()
    {
        if ((this.PageManager.ViewMode == ViewModeEnum.Design) || (this.HideOnCurrentPage) || (!this.IsVisible))
        {
        }
        else
        {
            this.subscription.DisplayFirstName = this.DisplayFirstName;
            this.subscription.LabelFirstName.Text = this.FirstNameText;

            this.subscription.DisplayLastName = this.DisplayLastName;
            this.subscription.LabelLastName.Text = this.LastNameText;

            // Set newsletter name
            this.subscription.NewsletterName = this.NewsletterName;
            // Set email label text
            this.subscription.LabelEmail.Text = this.EmailText;
            // Set submit button text
            this.subscription.ButtonOK.Text = this.ButtonText;
            // Set SkinID properties
            if (!this.StandAlone && (this.PageCycle < PageCycleEnum.Loaded))
            {
                string skinId = this.SkinID;
                if (skinId != "")
                {
                    this.subscription.LabelFirstName.SkinID = skinId;
                    this.subscription.LabelLastName.SkinID = skinId;
                    this.subscription.LabelEmail.SkinID = skinId;
                    this.subscription.TextBoxFirstName.SkinID = skinId;
                    this.subscription.TextBoxLastName.SkinID = skinId;
                    this.subscription.TextBoxEmail.SkinID = skinId;
                    this.subscription.ButtonOK.SkinID = skinId;
                }
            }
        }
    }
}
